package com.maryanovsky.gwtutils.client;

import com.google.gwt.user.client.Window;

/**
 * Various browser related or browser specific utilities.
 * 
 * @author Maryanovsky Alexander
 */
public class BrowserUtils {

    /***/
    private static native String getUserAgent()/*-{
		return navigator.userAgent;
	}-*/;

    /**
	 * The user-agent string of the browser we are running in.
	 */
    public static final String USER_AGENT = getUserAgent();

    /**
	 * Returns whether we are running in MSIE.
	 */
    public static boolean isIE() {
        return USER_AGENT.toLowerCase().indexOf("msie") != -1;
    }

    /**
	 * Returns whether we are running in MSIE 6.
	 */
    public static boolean isIE6() {
        return USER_AGENT.toLowerCase().indexOf("msie 6.") != -1;
    }

    /**
	 * Returns whether we are running in MSIE 7.
	 */
    public static boolean isIE7() {
        return USER_AGENT.toLowerCase().indexOf("msie 7.") != -1;
    }

    /**
	 * Returns whether we are running in MSIE 8.
	 */
    public static boolean isIE8() {
        return USER_AGENT.toLowerCase().indexOf("msie 8.") != -1;
    }

    /**
	 * Returns the version of IE we're running in.
	 */
    public static String getIEVersion() {
        String b = USER_AGENT.toLowerCase();
        int c = b.indexOf("msie ") + "msie ".length();
        int d = c;
        while (d < b.length()) {
            char e = b.charAt(d);
            if ((e != '.') && ((e < '0') || (e > '9'))) break;
            d++;
        }
        return b.substring(c, d);
    }

    /**
	 * Returns whether we are running in Opera.
	 */
    public static boolean isOpera() {
        return USER_AGENT.toLowerCase().indexOf("opera") != -1;
    }

    /**
	 * Returns whether we are running in Safari.
	 */
    public static boolean isSafari() {
        return USER_AGENT.toLowerCase().indexOf("safari") != -1;
    }

    /***/
    private static boolean a() {
        return isSafari() && (USER_AGENT.indexOf("Mobile") != -1);
    }

    /**
	 * Returns the Safari version number (the one used for marketing, like 4.0).
	 * Returns an empty string if unable to detect.
	 */
    public static String getSafariVersion() {
        final String b = "Version/";
        int c = USER_AGENT.indexOf(b);
        if (c == -1) return "";
        c += b.length();
        int d = USER_AGENT.indexOf(' ', c);
        if (d == -1) return "";
        return USER_AGENT.substring(c, d);
    }

    /**
	 * Returns whether we are running in an AppleWebKit browser.
	 */
    public static boolean isAppleWebKit() {
        return USER_AGENT.indexOf("AppleWebKit") != -1;
    }

    /**
	 * Returns whether we are running in a mobile version of an AppleWebKit
	 * browser.
	 */
    public static boolean isMobileAppleWebKit() {
        return isAppleWebKit() && (USER_AGENT.indexOf("Mobile") != -1);
    }

    /**
	 * Returns the AppleWebKit version number. Returns an empty string if unable
	 * to detect (possibly because we're not running in an AppleWebKit browser).
	 */
    public static String getAppleWebKitVersion() {
        final String b = "AppleWebKit/";
        int c = USER_AGENT.indexOf(b);
        if (c == -1) return "";
        c += b.length();
        int d = USER_AGENT.indexOf(' ', c);
        if (d == -1) return "";
        return USER_AGENT.substring(c, d);
    }

    /**
	 * Returns whether we are running on an Android device.
	 */
    public static boolean isAndroid() {
        return USER_AGENT.indexOf("Android") != -1;
    }

    /**
	 * Returns whether we are running in an AppleWebKit browser, on an Android
	 * device.
	 */
    public static boolean isAppleWebKitOnAndroid() {
        return isAppleWebKit() && isAndroid();
    }

    /**
	 * Returns whether we are running in a Mozilla browser.
	 */
    public static boolean isMozilla() {
        return (USER_AGENT.toLowerCase().indexOf("mozilla") != -1) && !isIE() && !isSafari() && !isOpera();
    }

    /**
	 * Returns whether we are running in a Gecko based browser.
	 */
    public static boolean isGecko() {
        return (USER_AGENT.toLowerCase().indexOf("gecko/") != -1);
    }

    /**
	 * Returns the Gecko version number. Returns an empty string if unable to
	 * detect (possibly because we're not running in a Gecko browser).
	 */
    public static String getGeckoVersion() {
        final String b = "rv:";
        int c = USER_AGENT.indexOf(b);
        if (c == -1) return "";
        c += b.length();
        int d = USER_AGENT.indexOf(")", c);
        if (d == -1) return "";
        return USER_AGENT.substring(c, d);
    }

    /**
	 * Returns whether we're running on MS Windows.
	 */
    public static boolean isWindows() {
        return USER_AGENT.indexOf("Win") != -1;
    }

    /**
	 * Returns whether we're running on Mac OS.
	 */
    public static boolean isMacOS() {
        return USER_AGENT.indexOf("Mac") != -1;
    }

    /**
	 * Returns whether we're running on Linux.
	 */
    public static boolean isLinux() {
        return USER_AGENT.indexOf("Linux") != -1;
    }

    /**
	 * Compares two version numbers of something. Returns a negative value
	 * if the first number is less than the 2nd, zero if they are equal, and a
	 * positive value if the first version number is larger than the second
	 * version number. The separator between major, minor and other sub-version
	 * numbers is assumed to be a period.
	 */
    public static int compareVersions(String version1, String version2) {
        String[] b = version1.split("\\.");
        String[] c = version2.split("\\.");
        for (int d = 0; d < b.length; d++) {
            if (c.length <= d) return 1;
            String e = b[d];
            String f = c[d];
            int g = Integer.parseInt(e);
            int h = Integer.parseInt(f);
            if (g < h) return -1; else if (g > h) return 1;
        }
        if (c.length > b.length) return -1; else return 0;
    }

    /**
	 * Returns the full URL of the host webpage.
	 */
    public static native String getHostPageUrl()/*-{
		return $doc.location.href;
	}-*/;

    /**
	 * Returns the hostname of the URL at which this application is running.
	 */
    public static native String getHostname()/*-{
		return $wnd.location.hostname;
	}-*/;

    /**
	 * Returns the pathname of the URL at which this application is running.
	 */
    public static native String getPathname()/*-{
		return $wnd.location.pathname;
	}-*/;

    /**
	 * Sets <code>document.location</code> to the specified value.
	 */
    public static native void setDocumentLocation(String location)/*-{
		$doc.location = location;
	}-*/;

    /**
	 * Sets the status of the window we're running in to the specified string.
	 */
    public static void setStatus(String statusText) {
        Window.setStatus(statusText);
    }

    ;

    /**
	 * Returns the value of <code>window.devicePixelRatio</code>, or 1 if
	 * there's no such property.
	 */
    public static native double getDevicePixelRatio()/*-{
		return $wnd.devicePixelRatio || 1.0;
	}-*/;

    /**
	 * Returns whether the browser supports touch events. Note that his doesn't
	 * necessarily mean that we're running on a touch device, as some desktop
	 * browsers may report touch events support.
	 */
    public static native boolean hasTouchEventsSupport()/*-{
		return 'ontouchstart' in $doc.documentElement;
	}-*/;
}
