package com.maryanovsky.gwtutils.client;

import com.google.gwt.core.client.JavaScriptObject;

/**
 * An overlay for a non-homogenous JavaScript array. It's up to the user to
 * know the actual types of the elements. 
 * 
 * @author Maryanovsky Alexander
 */
public final class JsMixedArray extends JavaScriptObject {

    /**
	 * The required no-arg protected constructor.
	 */
    protected JsMixedArray() {
    }

    /**
	 * Returns the size of the array.
	 */
    public native int length()/*-{
		return this.length;
	}-*/;

    /**
	 * Sets the length of the array.
	 */
    public native void setLength(int length)/*-{
		this.length = length;
	}-*/;

    /**
	 * Returns the element at the specified index, as an integer.
	 */
    public native int getInt(int index)/*-{
		return this[index];
	}-*/;

    /**
	 * Returns the element at the specified index, as a String. 
	 */
    public native String getString(int index)/*-{
		return this[index];
	}-*/;

    /**
	 * Returns the element at the specified index, as a double.
	 */
    public native double getDouble(int index)/*-{
		return this[index];
	}-*/;

    /**
	 * Returns the element at the specified index, as a boolean.
	 */
    public native boolean getBoolean(int index)/*-{
		return this[index];
	}-*/;

    /**
	 * Returns the element at the specified index, as a raw
	 * {@link JavaScriptObject}.
	 */
    public native JavaScriptObject getObject(int index)/*-{
		return this[index];
	}-*/;

    /**
	 * Sets the value at the specified index to the specified integer. 
	 */
    public native void setInt(int index, int value)/*-{
		this[index] = value;
	}-*/;

    /**
	 * Sets the value at the specified index to the specified String. 
	 */
    public native void setString(int index, String value)/*-{
		this[index] = value;
	}-*/;

    /**
	 * Sets the value at the specified index to the specified double. 
	 */
    public native void setDouble(int index, double value)/*-{
		this[index] = value;
	}-*/;

    /**
	 * Sets the value at the specified index to the specified boolean. 
	 */
    public native void setBoolean(int index, boolean value)/*-{
		this[index] = value;
	}-*/;

    /**
	 * Sets the value at the specified index to the specified raw
	 * {@link JavaScriptObject}. 
	 */
    public native void setObject(int index, JavaScriptObject value)/*-{
		this[index] = value;
	}-*/;

    /**
	 * Pushes the specified integer value onto the end of the array.
	 */
    public native void pushInt(int value)/*-{
		this[this.length] = value;
	}-*/;

    /**
	 * Pushes the specified String value onto the end of the array.
	 */
    public native void pushString(String value)/*-{
		this[this.length] = value;
	}-*/;

    /**
	 * Pushes the specified double value onto the end of the array.
	 */
    public native void pushDouble(double value)/*-{
		this[this.length] = value;
	}-*/;

    /**
	 * Pushes the specified String value onto the end of the array.
	 */
    public native void pushBoolean(boolean value)/*-{
		this[this.length] = value;
	}-*/;

    /**
	 * Pushes the specified raw {@link JavaScriptObject} value onto the end of
	 * the array.
	 */
    public native void pushObject(JavaScriptObject value)/*-{
		this[this.length] = value;
	}-*/;

    /**
	 * Removes the first element of the array and returns it, as an integer.
	 */
    public native int shiftInt()/*-{
		return this.shift();
	}-*/;

    /**
	 * Removes the first element of the array and returns it, as an String.
	 */
    public native String shiftString()/*-{
		return this.shift();
	}-*/;

    /**
	 * Removes the first element of the array and returns it, as a double.
	 */
    public native double shiftDouble()/*-{
		return this.shift();
	}-*/;

    /**
	 * Removes the first element of the array and returns it, as a boolean.
	 */
    public native boolean shiftBoolean()/*-{
		return this.shift();
	}-*/;

    /**
	 * Removes the first element of the array and returns it, as a raw 
	 * {@link JavaScriptObject}.
	 */
    public native JavaScriptObject shiftObject()/*-{
		return this.shift();
	}-*/;

    /**
	 * Pushes the specified integer value into the beginning of the array.
	 */
    public native void unshiftInt(int value)/*-{
		this.unshift(value);
	}-*/;

    /**
	 * Pushes the specified String value into the beginning of the array.
	 */
    public native void unshiftString(String value)/*-{
		this.unshift(value);
	}-*/;

    /**
	 * Pushes the specified double value into the beginning of the array.
	 */
    public native void unshiftDouble(double value)/*-{
		this.unshift(value);
	}-*/;

    /**
	 * Pushes the specified boolean value into the beginning of the array.
	 */
    public native void unshiftBoolean(boolean value)/*-{
		this.unshift(value);
	}-*/;

    /**
	 * Pushes the specified raw {@link JavaScriptObject} value into the
	 * beginning of the array.
	 */
    public native void unshiftObject(JavaScriptObject value)/*-{
		this.unshift(value);
	}-*/;

    /**
	 * Converts each element of the array into a string and joins them together
	 * with a comma separator.
	 */
    public native String join()/*-{
		return this.join();
	}-*/;

    /**
	 * Converts each element of the array into a string and joins them together
	 * with the specified separator.
	 */
    public native String join(String separator)/*-{
		return this.join(separator);
	}-*/;
}
