package com.maryanovsky.gwtutils.client;

/**
 * Mathematical utilities.
 * 
 * @author Maryanovsky Alexander
 */
public class MathUtils {

    /**
	 * Coefficient for converting degrees to radians.
	 */
    public static final double DEG_TO_RAD = Math.PI / 180;

    /**
	 * Coefficient for converting radians to degrees.
	 */
    public static final double RAD_TO_DEG = 180 / Math.PI;

    /**
	 * Powers of two.
	 */
    protected static final int[] POWERS_OF_TWO;

    static {
        POWERS_OF_TWO = new int[31];
        int b = 1;
        for (int c = 0; c < POWERS_OF_TWO.length; c++) {
            POWERS_OF_TWO[c] = b;
            b *= 2;
        }
    }

    /***/
    private static final double a = 1 / Math.log(2);

    /**
	 * Returns the largest integer smaller or equal to the specified one which
	 * is a multiple of the specified factor.
	 */
    public static int roundDownToMultiple(int value, int factor) {
        int b = Math.abs(value) % factor;
        if (b == 0) return value; else if (value >= 0) return value - b; else return value - (factor - b);
    }

    /**
	 * Returns the smallest integer larger or equal to the specified one, which
	 * is a multiple of the specified factor.
	 */
    public static int roundUpToMultiple(int value, int factor) {
        int b = Math.abs(value) % factor;
        if (b == 0) return value; else if (value >= 0) return value + (factor - b); else return value + b;
    }

    /**
	 * Returns a string representation of the specified double, with the
	 * specified number of digits following the decimal point.
	 */
    public static native String doubleToFixedString(double number, int precision)/*-{
		var x = number;
		return number.toFixed(precision);
	}-*/;

    /**
	 * Returns 2^n, for values of n between 0 and 31.
	 */
    public static int getPowerOfTwo(int n) {
        return POWERS_OF_TWO[n];
    }

    /**
	 * Returns the sign of the specified number, that is -1 if it is negative,
	 * 0 if it is zero and 1 if it positive.
	 */
    public static int sgn(int n) {
        return n == 0 ? 0 : (n > 0 ? 1 : -1);
    }

    /**
	 * The square function.
	 */
    public static double sqr(double x) {
        return x * x;
    }

    /**
	 * Returns the angle component of the polar coordinates of the specified
	 * cartesian coordinates (for some reason, there's no atan2 method in GWT's
	 * <code>java.lang.Math</code>).
	 */
    public static native double atan2(double dy, double dx)/*-{
		return Math.atan2(dy, dx);
	}-*/;

    /**
	 * Returns the maximum of the specified values.
	 */
    public static int max(int value0, int... values) {
        int b = value0;
        for (int c : values) if (c > b) b = c;
        return b;
    }

    /**
	 * Returns the minimum of the specified values.
	 */
    public static int min(int value0, int... values) {
        int b = value0;
        for (int c : values) if (c < b) b = c;
        return b;
    }

    /**
	 * Returns the logarithm of x, base-2.
	 */
    public static double log2(double x) {
        return Math.log(x) * a;
    }
}
