package com.maryanovsky.gwtutils.client;

/**
 * A wrapper for any two other given objects.
 * 
 * @author Maryanovsky Alexander
 */
public final class Pair<T1, T2> {

    /***/
    private final T1 first;

    /***/
    private final T2 second;

    /**
	 * Creates a new {@link Pair} with the two given objects. Either of the
	 * objects may be <code>null</code>.
	 */
    public Pair(T1 first, T2 second) {
        this.first = first;
        this.second = second;
    }

    /**
	 * Returns the first object.
	 */
    public T1 getFirst() {
        return first;
    }

    /**
	 * Returns the second object.
	 */
    public T2 getSecond() {
        return second;
    }

    /**
	 * Returns a hashcode combined from the hashcodes of the two target objects.
	 */
    public int hashCode() {
        return Utils.hashCode(Utils.hashCode(first), Utils.hashCode(second));
    }

    /**
	 * Returns true iff the given Object is a {@link Pair}, and its two objects
	 * are the same as this one's (comparison done via
	 * {@linkplain Utils#areEqual(Object, Object)}).
	 */
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof Pair<?, ?>)) return false;
        Pair<?, ?> a = (Pair<?, ?>) o;
        return Utils.areEqual(a.first, first) && Utils.areEqual(a.second, second);
    }
}
