package com.maryanovsky.gwtutils.client;

/**
 * A wrapper for any three other given objects.
 * 
 * @author Maryanovsky Alexander
 */
public final class Triple<T1, T2, T3> {

    /***/
    private final T1 first;

    /***/
    private final T2 second;

    /***/
    private final T3 third;

    /**
	 * Creates a new <code>Triple</code> with the three given objects. Either
	 * of the objects may be <code>null</code>.
	 */
    public Triple(T1 first, T2 second, T3 third) {
        this.first = first;
        this.second = second;
        this.third = third;
    }

    /**
	 * Returns the first object.
	 */
    public T1 getFirst() {
        return first;
    }

    /**
	 * Returns the second object.
	 */
    public T2 getSecond() {
        return second;
    }

    /**
	 * Returns the third object.
	 */
    public T3 getThird() {
        return third;
    }

    /**
	 * Returns a hashcode combined from the hashcodes of the three target
	 * objects.
	 */
    public int hashCode() {
        return Utils.hashCode(Utils.hashCode(first), Utils.hashCode(Utils.hashCode(second), Utils.hashCode(third)));
    }

    /**
	 * Returns true iff the given Object is a {@link Triple}, and its objects
	 * are the same as this one's (comparison done via
	 * {@linkplain Utils#areEqual(Object, Object)}).
	 */
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof Triple<?, ?, ?>)) return false;
        Triple<?, ?, ?> a = (Triple<?, ?, ?>) o;
        return Utils.areEqual(a.first, first) && Utils.areEqual(a.second, second) && Utils.areEqual(a.third, third);
    }
}
