package com.maryanovsky.gwtutils.client;

import java.util.*;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.i18n.client.Dictionary;
import com.google.gwt.user.client.Window;

/**
 * A general utility class.
 *
 * @author Maryanovsky Alexander
 */
public class Utils {

    /**
	 * Returns the hostname in the specified URL. Returns <code>null</code> if
	 * if the URL is relative.
	 */
    public static String getHostname(String url) {
        String a = url;
        int b = a.indexOf("://");
        if (b == -1) return null;
        a = a.substring(b + 3);
        int c = a.indexOf('/');
        if (c == -1) c = a.length();
        a = a.substring(0, c);
        int d = a.indexOf('@');
        if (d == -1) d = 0;
        a = a.substring(d);
        int e = a.indexOf(':');
        if (e == -1) e = a.length();
        a = a.substring(0, e);
        return a;
    }

    /**
	 * Creates a shallow copy of the specified String array.
	 */
    public static String[] clone(String[] arr) {
        String[] a = new String[arr.length];
        for (int b = 0; b < a.length; b++) a[b] = arr[b];
        return a;
    }

    /**
	 * Returns a hash code for the specified object, or if the object is
	 * <code>null</code>, returns 0.
	 */
    public static int hashCode(Object o) {
        return o == null ? 0 : o.hashCode();
    }

    /**
	 * Returns the hash code for the specified <code>double</code> value.
	 */
    public static int hashCode(double d) {
        return new Double(d).hashCode();
    }

    /**
	 * Returns the hash code for the specified <code>long</code> value.
	 */
    public static int hashCode(long l) {
        return new Long(l).hashCode();
    }

    /**
	 * Returns a hash code for the two specified hash codes.
	 */
    public static int hashCode(int hash1, int hash2) {
        return 37 * hash1 + hash2;
    }

    /**
	 * Returns whether the two specified objects are equal.
	 */
    public static boolean areEqual(Object o1, Object o2) {
        return o1 == null ? o2 == null : (o1 == o2) || o1.equals(o2);
    }

    /**
	 * Returns whether the two specified lists are equal by value, that is, they
	 * contain equal items, in the same order. Note that the comparison is not
	 * recursive, that is, even if some of the elements in the lists are lists
	 * themselves, they are compared using the regular <code>equals</code>
	 * method.
	 */
    public static boolean areEqual(List<?> list1, List<?> list2) {
        if (list1 == null) return list2 == null; else if (list2 == null) return false;
        if (list1.size() != list2.size()) return false;
        Iterator<?> a = list1.iterator();
        Iterator<?> b = list2.iterator();
        while (a.hasNext()) {
            Object c = a.next();
            Object d = b.next();
            if (!areEqual(c, d)) return false;
        }
        return true;
    }

    /**
	 * Returns the specified property of the specified JavaScript object.
	 */
    public native JavaScriptObject getProperty(JavaScriptObject object, String propertyName)/*-{
		return object[propertyName];
	}-*/;

    /**
	 * Sets the specified property of the specified JavaScript object.
	 */
    public static native void setProperty(JavaScriptObject object, String propertyName, JavaScriptObject propertyValue)/*-{
		object[propertyName] = propertyValue;
	}-*/;

    /**
	 * Returns the specified string property of the specified JavaScript object.
	 */
    public static native String getStringProperty(JavaScriptObject object, String propertyName)/*-{
		return object[propertyName];
	}-*/;

    /**
	 * Sets the specified string property of the specified JavaScript object.
	 */
    public static native void setStringProperty(JavaScriptObject object, String propertyName, String propertyValue)/*-{
		object[propertyName] = propertyValue;
	}-*/;

    /**
	 * Returns the specified integer property of the specified JavaScript
	 * object.
	 */
    public static native int getIntProperty(JavaScriptObject object, String propertyName)/*-{
		return object[propertyName];
	}-*/;

    /**
	 * Sets the specified integer property of the specified JavaScript object.
	 */
    public static native void setIntProperty(JavaScriptObject object, String propertyName, int propertyValue)/*-{
		object[propertyName] = propertyValue;
	}-*/;

    /**
	 * Returns the specified floating point property of the specified JavaScript
	 * object.
	 */
    public static native double getDoubleProperty(JavaScriptObject object, String propertyName)/*-{
		return object[propertyName];
	}-*/;

    /**
	 * Sets the specified floating point property of the specified JavaScript
	 * object.
	 */
    public static native void setDoubleProperty(JavaScriptObject object, String propertyName, double propertyValue)/*-{
		object[propertyName] = propertyValue;
	}-*/;

    /**
	 * Returns the specified boolean property of the specified JavaScript
	 * object.
	 */
    public static native boolean getBooleanProperty(JavaScriptObject object, String propertyName)/*-{
		return object[propertyName];
	}-*/;

    /**
	 * Sets the specified boolean property of the specified JavaScript object.
	 */
    public static native void setBooleanProperty(JavaScriptObject object, String propertyName, boolean propertyValue)/*-{
		object[propertyName] = propertyValue;
	}-*/;

    /**
	 * Adds the URL parameters of <code>window.location</code> to specified map
	 * and returns it. If a parameter has been specified more than once, only
	 * the last value is added to the map.
	 */
    public static Map<String, String> getUrlParams(Map<String, String> params) {
        Map<String, List<String>> a = Window.Location.getParameterMap();
        for (Map.Entry<String, List<String>> b : a.entrySet()) {
            List<String> c = b.getValue();
            params.put(b.getKey(), c.get(c.size() - 1));
        }
        return params;
    }

    /**
	 * Returns the URL parameters of <code>window.location</code>. The returned
	 * map is unmodifiable. If a parameter has been specified more than once,
	 * only the last value appears in the map.
	 */
    public static Map<String, String> getUrlParams() {
        return Collections.unmodifiableMap(getUrlParams(new HashMap<String, String>()));
    }

    /**
	 * Converts a {@link Dictionary} to a regular string-to-string map.
	 */
    public static Map<String, String> dictionaryToMap(Dictionary dictionary) {
        Map<String, String> a = new HashMap<String, String>();
        Set<String> b = dictionary.keySet();
        for (String c : b) a.put(c, dictionary.get(c));
        return a;
    }

    /**
	 * The JavaScript escape function.
	 */
    public static native String escape(String s)/*-{
		return escape(s);
	}-*/;

    /**
	 * The JavaScript unescape function.
	 */
    public static native String unescape(String s)/*-{
		return unescape(s);
	}-*/;
}
