package com.maryanovsky.gwtutils.client.event;

import java.util.LinkedList;
import java.util.List;

/**
 * A ready listener, which simply reports what it sees to a list of other
 * ready listener.
 * 
 * @author Maryanovsky Alexander
 */
public class DelegatingReadyListenerCollection<T> implements ReadyListener<T> {

    /***/
    private final List<ReadyListener<T>> a = new LinkedList<ReadyListener<T>>();

    /**
	 * Creates a new <code>DelegatingReadyListenerCollection</code>.
	 */
    public DelegatingReadyListenerCollection() {
    }

    /**
	 * Creates a new <code>DelegatingReadyListenerCollection</code> with the
	 * specified ready listener as the first of many to report to. This
	 * constructor is useful if you just have one listener to delegate to. 
	 */
    public DelegatingReadyListenerCollection(ReadyListener<T> readyListener) {
        add(readyListener);
    }

    /**
	 * Adds the specified ready listener to get notified when one of the sources
	 * we are registered with changes its ready state.
	 */
    public void add(ReadyListener<T> readyListener) {
        a.add(readyListener);
    }

    /**
	 * Removes the specified ready listener from getting notified when one of
	 * the sources with are registered with changes its ready state.
	 */
    public void remove(ReadyListener<T> readyListener) {
        a.remove(readyListener);
    }

    /**
	 * Notifies our ready listeners of the change.
	 */
    public void becameReady(T source) {
        for (ReadyListener<T> b : a) b.becameReady(source);
    }

    /**
	 * Notifies our ready listeners of the change.
	 */
    public void becameUnready(T source) {
        for (ReadyListener<T> b : a) b.becameUnready(source);
    }
}
