package com.maryanovsky.gwtutils.client.event;

import java.util.*;
import com.maryanovsky.gwtutils.client.Utils;

/**
 * A convenience class for managing {@linkplain PropertyChangeListener}s.
 * 
 * @author Maryanovsky Alexander
 */
public class PropertyChangeListenerList {

    /***/
    private final Map<String, List<PropertyChangeListener>> a = new HashMap<String, List<PropertyChangeListener>>();

    /**
	 * Creates a new <code>PropertyChangeListenerList</code> which will manage
	 * listeners to changes in the specified list of properties.
	 */
    public PropertyChangeListenerList(String[] propertyNames) {
        if (propertyNames == null) throw new IllegalArgumentException("propertyNames may not be null");
        if (propertyNames.length == 0) throw new IllegalArgumentException("propertyNames may not be empty");
        for (int b = 0; b < propertyNames.length; b++) a.put(propertyNames[b], new ArrayList<PropertyChangeListener>());
    }

    /**
	 * Adds the specified listener to be notified when the property with the
	 * specified name changes. 
	 */
    public void add(String propertyName, PropertyChangeListener listener) {
        List<PropertyChangeListener> b = a.get(propertyName);
        if (b == null) throw new IllegalArgumentException("Cannot add property change listener - unknown property: " + propertyName);
        b.add(listener);
    }

    /**
	 * Removes the specified listener from being notified when the property with
	 * the specified name changes.
	 */
    public void remove(String propertyName, PropertyChangeListener listener) {
        List<PropertyChangeListener> b = a.get(propertyName);
        if (b == null) throw new IllegalArgumentException("Cannot remove property change listener - unknown property: " + propertyName);
        b.remove(listener);
    }

    /**
	 * Notifies all registered listeners of a change in the specified property.
	 * Note that if <code>oldValue</code> and <code>newValue</code> are equal,
	 * no listeners are notified.
	 */
    public void firePropertyChanged(Object source, String propertyName, Object oldValue, Object newValue) {
        List<PropertyChangeListener> b = a.get(propertyName);
        if (b == null) throw new IllegalArgumentException("Cannot notify of property change - unknown property: " + propertyName);
        if (b.isEmpty()) return;
        if (Utils.areEqual(oldValue, newValue)) return;
        PropertyChangeEvent c = new PropertyChangeEvent(source, propertyName, oldValue, newValue);
        List<PropertyChangeListener> d = new ArrayList<PropertyChangeListener>(b);
        for (PropertyChangeListener e : d) e.propertyChanged(c);
    }
}
