package com.maryanovsky.gwtutils.client.event;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;

/**
 * The base class for all events which are synthesized from other
 * {@link GwtEvent}s. A {@link SyntheticEvent} is parameterized by the event
 * handler type that handles it and the event type that triggers it.
 * 
 * @author Maryanovsky Alexander
 */
public abstract class SyntheticEvent<H extends EventHandler, E extends GwtEvent<? extends EventHandler>> extends GwtEvent<H> {

    /***/
    private final E a;

    /**
	 * Creates a new {@link SyntheticEvent} with the specified source event from
	 * which it was synthesized.
	 */
    protected SyntheticEvent(E sourceEvent) {
        if (sourceEvent == null) throw new IllegalArgumentException("sourceEvent may not be null");
        this.a = sourceEvent;
    }

    /**
	 * Returns the event that caused this event to be synthesized. 
	 */
    public E getSourceEvent() {
        return a;
    }
}
