package com.maryanovsky.gwtutils.client.geom;

/**
 * Provides a view of an integer 2D point without allowing modifications.
 * 
 * @author Maryanovsky Alexander
 */
public interface PointView {

    /**
	 * Returns the X coordinate.
	 */
    int getX();

    /**
	 * Returns the Y coordinate.
	 */
    int getY();

    /**
	 * Returns the distance from this point to 0,0.
	 */
    double getDistanceFromOrigin();

    /**
	 * Returns the distance from this point to the specified one.
	 */
    double getDistanceFrom(PointView point);

    /**
	 * Returns the distance from this point to the specified one, as a
	 * <code>PointView</code>.
	 */
    PointView getDistancePoint(PointView point);

    /**
	 * Returns the distance from this point to the specified one, as a
	 * <code>Point</code>. If the specified 2nd point is not <code>null</code>,
	 * it is reused and returned.
	 */
    Point getDistancePoint(PointView point, Point reusable);

    /**
	 * Returns the distance from this point to the specified one, as a
	 * <code>SizeView</code>. Note that this may differ from
	 * {@linkplain #getDistancePoint(PointView)} because the dimensions of a 
	 * <code>SizeView</code> are always non-negative.
	 */
    SizeView getDistanceSize(PointView point);

    /**
	 * Returns the distance from this point to the specified one in the same
	 * manner as {@linkplain #getDistanceSize(PointView)}. If the specified
	 * <code>Size</code> is not <code>null</code>, it is reused and returned.
	 */
    Size getDistanceSize(PointView point, Size reusable);

    /**
	 * Returns the point opposite to this one.
	 */
    PointView getOpposite();

    /**
	 * Returns the point opposite to this one. If the specified reusable point
	 * is not <code>null</code>, it is reused and returned.
	 */
    Point getOpposite(Point reusable);

    /**
	 * Returns whether this point represents the specified coordinates.
	 */
    boolean equals(int x, int y);
}
