package com.maryanovsky.gwtutils.client.net;

import com.google.gwt.http.client.*;

/**
 * An implementation of RequestMaker&lt;String&gt; that uses a
 * {@link RequestBuilder} to make the request.
 * 
 * @author Maryanovsky Alexander
 */
public class HttpTextRequestMaker implements RequestMaker<String> {

    /***/
    private final RequestBuilder a;

    /**
	 * Creates a new {@link HttpTextRequestMaker} which will send the request
	 * using the specified {@link RequestBuilder}. 
	 */
    public HttpTextRequestMaker(RequestBuilder requestBuilder) {
        if (requestBuilder == null) throw new IllegalArgumentException("requestBuilder may not be null");
        this.a = requestBuilder;
    }

    /**
	 * {@inheritDoc} 
	 */
    public void sendRequest(final ResultCallback<String> callback) throws RequestException {
        try {
            a.sendRequest(null, new RequestCallback() {

                public void onError(Request request, Throwable exception) {
                    callback.requestFinished();
                    callback.errorOcurred(exception.getMessage());
                }

                public void onResponseReceived(Request request, Response response) {
                    callback.requestFinished();
                    int b = response.getStatusCode();
                    if ((b != 200) && (b != 0)) {
                        callback.errorOcurred("Status code " + response.getStatusCode());
                        return;
                    }
                    callback.resultReceived(response.getText());
                }
            });
        } catch (com.google.gwt.http.client.RequestException b) {
            throw new RequestException(b);
        }
    }

    /**
	 * {@inheritDoc}
	 */
    public String getUrl() {
        return a.getUrl();
    }
}
