package com.maryanovsky.gwtutils.client.net;

import com.google.gwt.jsonp.client.JsonpRequestBuilder;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * An implementation of RequestMaker&lt;String&gt; that uses a
 * {@link JsonpRequestBuilder} to make the request.
 * 
 * @author Maryanovsky Alexander
 */
public class JsonpTextRequestMaker implements RequestMaker<String> {

    /***/
    private final JsonpRequestBuilder a;

    /***/
    private final String b;

    /**
	 * Creates a new {@link JsonpTextRequestMaker} which will send the request
	 * to the specified URL using the specified {@link JsonpRequestBuilder}.
	 */
    public JsonpTextRequestMaker(JsonpRequestBuilder requestBuilder, String baseUrl) {
        if (requestBuilder == null) throw new IllegalArgumentException("requestBuilder may not be null");
        if (baseUrl == null) throw new IllegalArgumentException("baseUrl may not be null");
        this.a = requestBuilder;
        this.b = baseUrl;
    }

    /**
	 * {@inheritDoc}
	 */
    public void sendRequest(final ResultCallback<String> callback) {
        a.requestString(b, new AsyncCallback<String>() {

            public void onFailure(Throwable caught) {
                callback.requestFinished();
                callback.errorOcurred(caught.getMessage());
            }

            public void onSuccess(String result) {
                callback.requestFinished();
                callback.resultReceived(result.toString());
            }
        });
    }

    /**
	 * {@inheritDoc}
	 */
    public String getUrl() {
        return b;
    }
}
