package com.maryanovsky.gwtutils.client.net;

/**
 * The interface to implement for getting notified when a result to a request
 * arrives. See {@link XmlResponseHandler} for example.
 */
public interface ResultCallback<T> {

    /**
	 * Invoked when any kind of result occurs, whether an error occurred while
	 * sending or receiving or a bad/good response received. It is invoked
	 * before any of the other methods. 
	 */
    void requestFinished();

    /**
	 * Invoked when a successfully parsed response arrives.
	 * 
	 * @param result The parsed result object.
	 */
    void resultReceived(T result);

    /**
	 * Invoked when an error occurs while sending or receiving the request. 
	 */
    void errorOcurred(String errorMessage);
}
