package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.HasVerticalAlignment.VerticalAlignmentConstant;

/**
 * A panel which positions its sole child in one of the 9 locations defined by
 * the combination of the 3 horizontal positions left,center,right and the 3
 * vertical positions top,center,bottom.
 *
 * @author Maryanovsky Alexander
 */
public class AlignPanel extends SimplePanel {

    /***/
    private final Element a;

    /***/
    private final Element b;

    /**
	 * Creates a new <code>DockPositionPanel</code> with the specified
	 * horizontal and vertical alignment. 
	 */
    public AlignPanel(HorizontalAlignmentConstant horizontalAlignment, VerticalAlignmentConstant verticalAlignment) {
        super(DOM.createTable());
        Element d = c(getElement());
        Element e = c(DOM.createTBody());
        Element f = c(DOM.createTR());
        Element g = c(DOM.createTD());
        DOM.setElementPropertyInt(d, "cellPadding", 0);
        DOM.setElementPropertyInt(d, "cellSpacing", 0);
        DOM.appendChild(f, g);
        DOM.appendChild(e, f);
        DOM.appendChild(d, e);
        Element h = c(DOM.createTable());
        Element i = c(DOM.createTBody());
        Element j = c(DOM.createTR());
        Element k = c(DOM.createTD());
        DOM.setElementPropertyInt(h, "cellPadding", 0);
        DOM.setElementPropertyInt(h, "cellSpacing", 0);
        DOM.appendChild(j, k);
        DOM.appendChild(i, j);
        DOM.appendChild(h, i);
        DOM.appendChild(g, h);
        this.a = k;
        this.b = g;
        setHorizontalAlignment(horizontalAlignment);
        setVerticalAlignment(verticalAlignment);
    }

    /**
	 * Creates a new <code>AlignPanel</code> with the specified alignment.
	 */
    public AlignPanel(Alignment alignment) {
        this(alignment.getHorizontalAlignment(), alignment.getVerticalAlignment());
    }

    /**
	 * Creates a new <code>AlignPanel</code> with the specified child placed at
	 * at the specified position.
	 */
    public AlignPanel(Widget child, HorizontalAlignmentConstant horizontalAlignment, VerticalAlignmentConstant verticalAlignment) {
        this(horizontalAlignment, verticalAlignment);
        if (child != null) add(child);
    }

    /**
	 * Creates a new <code>AlignPanel</code> with the specified child placed at
	 * the specified position.
	 */
    public AlignPanel(Widget child, Alignment alignment) {
        this(child, alignment.getHorizontalAlignment(), alignment.getVerticalAlignment());
    }

    /***/
    private static Element c(Element d) {
        DOM.setStyleAttribute(d, "margin", "0px");
        DOM.setStyleAttribute(d, "border", "0px");
        DOM.setStyleAttribute(d, "padding", "0px");
        return d;
    }

    /**
	 * Sets the horizontal alignment.
	 */
    public void setHorizontalAlignment(HorizontalAlignmentConstant horizontalAlignment) {
        DOM.setElementProperty(b, "align", horizontalAlignment.getTextAlignString());
    }

    /**
	 * Sets the vertical alignment.
	 */
    public void setVerticalAlignment(VerticalAlignmentConstant verticalAlignment) {
        DOM.setStyleAttribute(b, "verticalAlign", verticalAlignment.getVerticalAlignString());
    }

    /**
	 * Returns the element into which content should be added.
	 */
    protected Element getContainerElement() {
        return a;
    }

    /**
	 * Returns an {@link AlignPanel} which positions its child in the center.
	 */
    public static AlignPanel makeCenter() {
        return new AlignPanel(HasHorizontalAlignment.ALIGN_CENTER, HasVerticalAlignment.ALIGN_MIDDLE);
    }

    /**
	 * Returns an {@link AlignPanel} which positions the specified child in
	 * the center.
	 */
    public static AlignPanel makeCenter(Widget child) {
        return new AlignPanel(child, Alignment.CENTER);
    }
}
