package com.maryanovsky.gwtutils.client.ui;

import static com.google.gwt.user.client.ui.HasHorizontalAlignment.ALIGN_CENTER;
import static com.google.gwt.user.client.ui.HasHorizontalAlignment.ALIGN_LEFT;
import static com.google.gwt.user.client.ui.HasHorizontalAlignment.ALIGN_RIGHT;
import static com.google.gwt.user.client.ui.HasVerticalAlignment.ALIGN_BOTTOM;
import static com.google.gwt.user.client.ui.HasVerticalAlignment.ALIGN_MIDDLE;
import static com.google.gwt.user.client.ui.HasVerticalAlignment.ALIGN_TOP;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.HasVerticalAlignment.VerticalAlignmentConstant;

/**
 * Convenient constants for placement to be used instead of a pair of alignment
 * constants.
 * 
 * @author Maryanovsky Alexander
 */
public enum Alignment {

    TOP_LEFT(ALIGN_LEFT, ALIGN_TOP), TOP_CENTER(ALIGN_CENTER, ALIGN_TOP), TOP_RIGHT(ALIGN_RIGHT, ALIGN_TOP), MIDDLE_LEFT(ALIGN_LEFT, ALIGN_MIDDLE), CENTER(ALIGN_CENTER, ALIGN_MIDDLE), MIDDLE_RIGHT(ALIGN_RIGHT, ALIGN_MIDDLE), BOTTOM_LEFT(ALIGN_LEFT, ALIGN_BOTTOM), BOTTOM_CENTER(ALIGN_CENTER, ALIGN_BOTTOM), BOTTOM_RIGHT(ALIGN_RIGHT, ALIGN_BOTTOM);

    /***/
    private static final Alignment[][] a = new Alignment[][] { { TOP_LEFT, TOP_CENTER, TOP_RIGHT }, { MIDDLE_LEFT, CENTER, MIDDLE_RIGHT }, { BOTTOM_LEFT, BOTTOM_CENTER, BOTTOM_RIGHT } };

    /***/
    private final HorizontalAlignmentConstant b;

    /***/
    private final VerticalAlignmentConstant c;

    /**
	 * Creates a new <code>Alignment</code> with the specified horizontal
	 * and vertical alignments.
	 */
    private Alignment(HorizontalAlignmentConstant d, VerticalAlignmentConstant e) {
        this.b = d;
        this.c = e;
    }

    /**
	 * Returns the Alignment
	 */
    public static Alignment getInstance(HorizontalAlignmentConstant horizontalAlignment, VerticalAlignmentConstant verticalAlignment) {
        int d, e;
        if (verticalAlignment == ALIGN_TOP) d = 0; else if (verticalAlignment == ALIGN_MIDDLE) d = 1; else if (verticalAlignment == ALIGN_BOTTOM) d = 2; else throw new IllegalArgumentException("Unrecognized vertical alignment value: " + verticalAlignment);
        if (horizontalAlignment == ALIGN_LEFT) e = 0; else if (horizontalAlignment == ALIGN_CENTER) e = 1; else if (horizontalAlignment == ALIGN_RIGHT) e = 2; else throw new IllegalArgumentException("Unrecognized horizontal alignment value: " + horizontalAlignment);
        return a[d][e];
    }

    /**
	 * Returns the horizontal alignment constant.
	 */
    public HorizontalAlignmentConstant getHorizontalAlignment() {
        return b;
    }

    /**
	 * Returns the vertical alignment constant.
	 */
    public VerticalAlignmentConstant getVerticalAlignment() {
        return c;
    }
}
