package com.maryanovsky.gwtutils.client.ui;

import java.util.Iterator;
import com.google.gwt.dom.client.Style.Overflow;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.dom.client.*;
import com.google.gwt.layout.client.Layout.Alignment;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.ui.*;

/**
 * A scroll panel which scrolls its contents with two buttons instead of a scroll
 * bar.
 * 
 * @author Maryanovsky Alexander
 */
public class ButtonScrollPanel extends Composite implements HasWidgets {

    /***/
    private final Resources a;

    /***/
    private final ScrollPanel b;

    /***/
    private final FocusPanel c;

    /***/
    private final FocusPanel d;

    /***/
    private final Timer e = new Timer() {

        public void run() {
            int k = b.getVerticalScrollPosition();
            int l = Math.max(0, k - getScrollStep());
            b.setVerticalScrollPosition(l);
            if (k == b.getVerticalScrollPosition()) cancel();
        }
    };

    /***/
    private final Timer f = new Timer() {

        public void run() {
            int k = b.getVerticalScrollPosition();
            int l = b.getElement().getPropertyInt("scrollHeight");
            int m = Math.min(l, k + getScrollStep());
            b.setVerticalScrollPosition(m);
            if (k == b.getVerticalScrollPosition()) cancel();
        }
    };

    /***/
    private int g = 10;

    /***/
    private int h = 100;

    /***/
    private boolean i = false;

    /**
	 * Creates a new {@link ButtonScrollPanel} with the specified resources.
	 */
    public ButtonScrollPanel(Resources resources) {
        if (resources == null) throw new IllegalArgumentException("resources may not be null");
        Widget k = new AlignPanel(new Image(resources.upButtonImage()), com.maryanovsky.gwtutils.client.ui.Alignment.CENTER);
        Widget l = new AlignPanel(new Image(resources.downButtonImage()), com.maryanovsky.gwtutils.client.ui.Alignment.CENTER);
        k.setWidth("100%");
        l.setWidth("100%");
        this.a = resources;
        this.b = new ScrollPanel();
        this.c = new FocusPanel(k);
        this.d = new FocusPanel(l);
        UiUtils.extraImageCellHeightWorkaround(c);
        UiUtils.extraImageCellHeightWorkaround(d);
        c.addMouseDownHandler(new MouseDownHandler() {

            public void onMouseDown(MouseDownEvent event) {
                if (!isScrollOnMouseOver()) {
                    e.scheduleRepeating(getScrollDelay());
                    f.cancel();
                }
            }
        });
        c.addMouseUpHandler(new MouseUpHandler() {

            public void onMouseUp(MouseUpEvent event) {
                if (!isScrollOnMouseOver()) e.cancel();
            }
        });
        d.addMouseDownHandler(new MouseDownHandler() {

            public void onMouseDown(MouseDownEvent event) {
                if (!isScrollOnMouseOver()) {
                    f.scheduleRepeating(getScrollDelay());
                    e.cancel();
                }
            }
        });
        d.addMouseUpHandler(new MouseUpHandler() {

            public void onMouseUp(MouseUpEvent event) {
                if (!isScrollOnMouseOver()) f.cancel();
            }
        });
        c.addMouseOverHandler(new MouseOverHandler() {

            public void onMouseOver(MouseOverEvent event) {
                if (isScrollOnMouseOver()) {
                    e.scheduleRepeating(getScrollDelay());
                    f.cancel();
                }
            }
        });
        c.addMouseOutHandler(new MouseOutHandler() {

            public void onMouseOut(MouseOutEvent event) {
                if (isScrollOnMouseOver()) e.cancel();
            }
        });
        d.addMouseOverHandler(new MouseOverHandler() {

            public void onMouseOver(MouseOverEvent event) {
                if (isScrollOnMouseOver()) {
                    f.scheduleRepeating(getScrollDelay());
                    e.cancel();
                }
            }
        });
        d.addMouseOutHandler(new MouseOutHandler() {

            public void onMouseOut(MouseOutEvent event) {
                if (isScrollOnMouseOver()) f.cancel();
            }
        });
        b.getElement().getStyle().setOverflow(Overflow.HIDDEN);
        initWidget(j());
    }

    /***/
    private Widget j() {
        LayoutPanel k = new LayoutPanel();
        k.add(c);
        k.add(b);
        k.add(d);
        k.setWidgetHorizontalPosition(c, Alignment.STRETCH);
        k.setWidgetTopHeight(c, 0, Unit.PX, a.upButtonImage().getHeight(), Unit.PX);
        k.setWidgetHorizontalPosition(d, Alignment.STRETCH);
        k.setWidgetBottomHeight(d, 0, Unit.PX, a.downButtonImage().getHeight(), Unit.PX);
        k.setWidgetLeftRight(b, 0, Unit.PX, 0, Unit.PX);
        k.setWidgetTopBottom(b, a.upButtonImage().getHeight(), Unit.PX, a.downButtonImage().getHeight(), Unit.PX);
        return k;
    }

    /**
	 * Returns the panel where the scrollable child should be placed.
	 */
    public SimplePanel getContentPanel() {
        return b;
    }

    /**
	 * Adds a widget. Note that you can't add more than one widget.
	 */
    public void add(Widget widget) {
        getContentPanel().add(widget);
    }

    /**
	 * Sets the content widget.
	 */
    public void setContentWidget(Widget widget) {
        getContentPanel().setWidget(widget);
    }

    /**
	 * Removes the widget.
	 */
    public void clear() {
        getContentPanel().clear();
    }

    /**
	 * Returns an iterator listing the sole child, if any.
	 */
    public Iterator<Widget> iterator() {
        return getContentPanel().iterator();
    }

    /**
	 * Removes the widget.
	 */
    public boolean remove(Widget widget) {
        return getContentPanel().remove(widget);
    }

    /**
	 * Sets the number of pixels a single step will scroll.
	 */
    public void setScrollStep(int scrollStep) {
        if (scrollStep <= 0) throw new IllegalArgumentException("scrollStep must be positive");
        this.g = scrollStep;
    }

    /**
	 * Returns the number of pixels a single step scrolls.
	 */
    public int getScrollStep() {
        return g;
    }

    /**
	 * Sets the delay between scrolls, in milliseconds.
	 */
    public void setScrollDelay(int scrollDelay) {
        if (scrollDelay <= 0) throw new IllegalArgumentException("scrollDelay must be positive");
        this.h = scrollDelay;
    }

    /**
	 * Returns the delay between scrolls, in milliseconds.
	 */
    public int getScrollDelay() {
        return h;
    }

    /**
	 * Returns whether the scrolling occurs on mouse-over. If
	 * <code>false</code>, the scrolling will only occur when the user actually
	 * presses the buttons.
	 */
    public boolean isScrollOnMouseOver() {
        return i;
    }

    /**
	 * Sets whether the scrolling will occur on mouse-over. If not, the user
	 * will need to press the buttons to scroll.
	 */
    public void setScrollOnMouseOver(boolean isScrollOnMouseOver) {
        this.i = isScrollOnMouseOver;
    }

    /**
	 * The interface for the resources required by the scroll pane.
	 */
    public interface Resources extends ClientBundle {

        /**
		 * The image for the "up" button.
		 */
        ImageResource upButtonImage();

        /**
		 * The image for the "down" button.
		 */
        ImageResource downButtonImage();
    }
}
