package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.uibinder.client.UiConstructor;
import com.google.gwt.user.client.ui.Widget;

/**
 * An {@link Accordion} where a default height for the widgets and their headers
 * is specified in the constructor. This class is mostly needed because
 * currently, GWT (2.0) does not allow a widget container to specify its own
 * layout elements for a UiBinder XML file.
 * 
 * @author Maryanovsky Alexander
 */
public class DefaultSizeAccordion extends Accordion {

    /***/
    private final double a;

    /***/
    private final double b;

    /**
	 * Creates a new <code>FixedSizeAccordion</code> with the specified size
	 * unit and sizes for the headers and widgets.
	 */
    public DefaultSizeAccordion(Unit unit, double headerHeight, double widgetHeight) {
        super(unit);
        this.a = headerHeight;
        this.b = widgetHeight;
    }

    /**
	 * Creates a new <code>FixedSizeAccordion</code> with the specified size
	 * unit and heights for the headers and widgets. This constructor is meant to
	 * be used by the LayoutUiBinder, so it takes String arguments.
	 */
    @UiConstructor
    public DefaultSizeAccordion(String unit, String headerHeight, String widgetHeight) {
        this(Unit.valueOf(unit), Double.parseDouble(headerHeight), Double.parseDouble(widgetHeight));
    }

    /**
	 * Inserts a widget and a header at the specified position.
	 */
    public void insert(Widget widget, Widget header, int index) {
        super.insert(widget, b, header, a, index);
    }

    /**
	 * Adds a widget and a header to the end of the <code>Accordion</code>.
	 */
    public void add(Widget widget, Widget header) {
        insert(widget, header, getWidgetCount());
    }
}
