package com.maryanovsky.gwtutils.client.ui;

import java.util.ArrayList;
import java.util.List;
import com.google.gwt.dom.client.Style;
import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.logical.shared.*;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.BrowserUtils;

/**
 * A simple drop down list.
 * 
 * The CSS style names of this widget are
 * <ul>
 *   <li>.gwtutils-DropDownList for the widget itself</li>
 *   <li>.gwtutils-DropDownListOpen for the widget itself, when the popup is
 *   open.</li>
 *   <li>.gwtutils-DropDownListSelectedItemLabel for the label where the current
 *   item appears</li>
 *   <li>.gwtutils-DropDownListArrow for the drop-down arrow widget.</li>
 *   <li>.gwtutils-DropDownListPopup for the popup.</li>
 *   <li>.gwtutils-DropDownListPopupPanel for the panel inside the popup that
 *   holds the items.</li>
 *   <li>.gwtutils-DropDownListPopupItem for an item in the popup menu.</li>
 *   <li>.gwtutils-DropDownListPopupItemSelected for the item in the popup menu
 *   that the user currently has the mouse cursor over.</li>
 * </ul>
 * 
 * @author Maryanovsky Alexander
 */
public class DropDownList extends Composite implements HasValueChangeHandlers<Integer> {

    /***/
    private final Resources a;

    /**
	 * The popup.
	 */
    public final PopupPanel popup;

    /***/
    private final Label b = new InlineLabel();

    /***/
    private final List<String> c = new ArrayList<String>();

    /***/
    private int d = -1;

    /**
	 * Creates a new {@link DropDownList} which will use a character to
	 * represent the arrow. 
	 */
    public DropDownList() {
        this(null);
    }

    /**
	 * Creates a new {@link DropDownList} with the specified {@link Resources}.
	 */
    public DropDownList(Resources resources) {
        this.a = resources;
        this.popup = new PopupPanel(true);
        Widget g = e();
        FocusPanel h = new FocusPanel(g);
        initWidget(h);
        setStyleName("gwtutils-DropDownList");
        b.setStyleName("gwtutils-DropDownListSelectedItemLabel");
        popup.setStyleName("gwtutils-DropDownListPopup");
        h.addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent event) {
                DropDownList.this.onClick(event);
            }
        });
        popup.setAnimationEnabled(false);
        popup.addCloseHandler(new CloseHandler<PopupPanel>() {

            public void onClose(CloseEvent<PopupPanel> event) {
                removeStyleName("gwtutils-DropDownListOpen");
            }
        });
        Style i = getElement().getStyle();
        i.setDisplay(Display.INLINE);
        i.setProperty("hideFocus", "true");
        i.setProperty("outline", "0px");
    }

    /***/
    private Widget e() {
        Widget g = createDropDownArrow(a);
        g.setStyleName("gwtutils-DropDownListArrow");
        FlowPanel h = new FlowPanel();
        h.add(b);
        h.add(g);
        h.getElement().getStyle().setDisplay(Display.INLINE);
        return h;
    }

    /**
	 * Creates the dropdown arrow widget from the specified resources. If
	 * <code>resources</code> is not <code>null</code>, the default
	 * implementation returns an {@link Image} created from the
	 * {@link ImageResource} returned by {@link Resources#dropDownArrowIcon()}.
	 * Otherwise, it returns a label with a down-pointing triangle character.
	 */
    protected Widget createDropDownArrow(Resources resources) {
        return resources == null ? f() : new Image(resources.dropDownArrowIcon());
    }

    /***/
    private Widget f() {
        Widget g;
        if (BrowserUtils.isIE()) {
            Label h = new InlineLabel("6");
            Style i = h.getElement().getStyle();
            i.setProperty("fontFamily", "WebDings");
            i.setPosition(Position.ABSOLUTE);
            i.setBottom(-5, Unit.PX);
            SimplePanel j = new InlineSimplePanel();
            j.setWidget(h);
            j.getElement().getStyle().setDisplay(Display.INLINE_BLOCK);
            j.getElement().getStyle().setPosition(Position.RELATIVE);
            j.getElement().getStyle().setWidth(1, Unit.EM);
            j.getElement().getStyle().setHeight(1, Unit.EM);
            g = j;
        } else {
            g = new InlineLabel("▾");
        }
        return g;
    }

    /**
	 * Adds a handler for change of the selected index.
	 */
    public HandlerRegistration addValueChangeHandler(ValueChangeHandler<Integer> handler) {
        return addHandler(handler, ValueChangeEvent.getType());
    }

    /**
	 * Adds an item.
	 */
    public void addItem(String item) {
        insertItem(item, c.size());
    }

    /**
	 * Inserts an item at the specified index.
	 */
    public void insertItem(String item, int index) {
        c.add(index, item);
    }

    /**
	 * Removes the item at the specified index.
	 */
    public void removeItem(int index) {
        c.remove(index);
        if (getSelectedIndex() == index) setSelectedIndex(-1);
    }

    /**
	 * Returns the item at the specified index.
	 */
    public String getItem(int index) {
        return c.get(index);
    }

    /**
	 * Returns the number of items.
	 */
    public int getItemCount() {
        return c.size();
    }

    /**
	 * Returns the currently selected index; -1 if none.
	 */
    public int getSelectedIndex() {
        return d;
    }

    /**
	 * Sets the item with the specified index to be selected.
	 */
    public void setSelectedIndex(int index) {
        b.setText(index == -1 ? "" : c.get(index));
        d = index;
        ValueChangeEvent.fire(this, d);
    }

    /**
	 * Returns the popup panel in which the items are displayed for selection.
	 */
    public final PopupPanel getPopup() {
        return popup;
    }

    /***/
    private void onClick(ClickEvent g) {
        if (getItemCount() == 0) return;
        popup.setWidget(createPopupSelectionList());
        popup.showRelativeTo(this);
        addStyleName("gwtutils-DropDownListOpen");
    }

    /**
	 * Creates the widget to be displayed in the popup to select an item.
	 */
    protected Widget createPopupSelectionList() {
        MouseOverHandler g = new MouseOverHandler() {

            public void onMouseOver(MouseOverEvent event) {
                Label h = (Label) event.getSource();
                h.addStyleName("gwtutils-DropDownListPopupItemSelected");
            }
        };
        MouseOutHandler h = new MouseOutHandler() {

            public void onMouseOut(MouseOutEvent event) {
                Label i = (Label) event.getSource();
                i.removeStyleName("gwtutils-DropDownListPopupItemSelected");
            }
        };
        ClickHandler i = new ClickHandler() {

            public void onClick(ClickEvent event) {
                Label j = (Label) event.getSource();
                setSelectedIndex(j.getElement().getPropertyInt("itemIndex"));
                popup.hide();
            }
        };
        VerticalPanel j = new VerticalPanel();
        j.setStyleName("gwtutils-DropDownListPopupPanel");
        for (int k = 0; k < getItemCount(); k++) {
            Label l = new Label(getItem(k));
            l.setStyleName("gwtutils-DropDownListPopupItem");
            l.addMouseOverHandler(g);
            l.addMouseOutHandler(h);
            l.addClickHandler(i);
            l.getElement().setPropertyInt("itemIndex", k);
            j.add(l);
        }
        return j;
    }

    /**
	 * The interface for the resources required by a drop-down list.
	 */
    public interface Resources extends ClientBundle {

        /**
		 * The arrow displayed at the right of the currently selected item.
		 */
        ImageResource dropDownArrowIcon();
    }
}
