package com.maryanovsky.gwtutils.client.ui;

import static com.google.gwt.user.client.ui.HasHorizontalAlignment.ALIGN_CENTER;
import static com.google.gwt.user.client.ui.HasVerticalAlignment.ALIGN_MIDDLE;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyDownEvent;
import com.google.gwt.event.dom.client.KeyDownHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.ui.HTMLTable.CellFormatter;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.HasVerticalAlignment.VerticalAlignmentConstant;
import com.maryanovsky.gwtutils.client.ui.GwtUiDefaults.WidgetElementType;

/**
 * A panel which manages most of the tedious work behind setting up a
 * well-arranged and properly aligned form. A form consists of 3 main elements:
 * a title, a set of named fields and a set of buttons.
 *
 * @author Maryanovsky Alexander
 */
public class Form extends Composite {

    /***/
    private static final int a = 50;

    /***/
    private final Label b;

    /***/
    private final FieldsGrid c;

    /***/
    private final HorizontalPanel d;

    /***/
    private Button e;

    /***/
    private Focusable f = null;

    /***/
    private boolean g = false;

    /**
	 * Creates a new <code>Form</code>.
	 */
    public Form() {
        this.b = new Label("", false);
        this.c = new FieldsGrid();
        this.d = new HorizontalPanel();
        DOM.setStyleAttribute(b.getElement(), "fontSize", "x-large");
        UiUtils.setTrailingMargin(d, -GwtUiDefaults.getHorizontalVisualMargin(WidgetElementType.BUTTON));
        HTMLTable h = UiUtils.prepareLayoutTable(new Grid(5, 1));
        h.setWidget(0, 0, b);
        UiUtils.fillRow(h, 1, GwtUiDefaults.getFormTitleFieldsGap());
        h.setWidget(2, 0, c);
        UiUtils.fillRow(h, 3, GwtUiDefaults.getFormFieldsButtonsGap());
        h.setWidget(4, 0, d);
        CellFormatter i = h.getCellFormatter();
        i.setHorizontalAlignment(0, 0, ALIGN_CENTER);
        i.setHorizontalAlignment(2, 0, UiUtils.getLeadingAlignment());
        i.setHorizontalAlignment(4, 0, UiUtils.getTrailingAlignment());
        FocusPanel j = new FocusPanel();
        j.setTabIndex(-1);
        j.setWidget(h);
        j.addKeyDownHandler(new KeyDownHandler() {

            public void onKeyDown(KeyDownEvent evt) {
                if ((evt.getNativeKeyCode() == KeyCodes.KEY_ENTER) && (e != null) && g) e.click();
            }
        });
        initWidget(j);
        sinkEvents(Event.KEYEVENTS);
    }

    /**
	 * Sets whether the "OK" button will be clicked programmatically when the
	 * user hits ENTER.
	 */
    public void setClickOkOnEnter(boolean isClickOkOnEnter) {
        this.g = isClickOkOnEnter;
    }

    /**
	 * Sets the title of this form.
	 */
    public void setFormTitle(String title) {
        b.setText(title);
    }

    /**
	 * Adds a field with the specified name.
	 */
    public void addField(String name, Widget field) {
        addField(name, field, null);
    }

    /**
	 * Adds a field with the specified name and suffix. The suffix is displayed
	 * after the field.
	 */
    public void addField(String name, Widget field, String suffix) {
        addField(name, ALIGN_MIDDLE, null, field, ALIGN_MIDDLE, null, suffix, ALIGN_MIDDLE, null);
    }

    /**
	 * Adds a field with the specified name and suffix. The elements are aligned
	 * as specified. Vertical <code>null</code> alignments are interpreted as
	 * {@link HasVerticalAlignment#ALIGN_MIDDLE}. Horizontal <code>null</code>
	 * alignments are interpreted as textual leading alignment, except that
	 * field names are aligned to the trailing side.
	 */
    public void addField(String name, VerticalAlignmentConstant nameVerticalAlignment, HorizontalAlignmentConstant nameHorizontalAlignment, Widget field, VerticalAlignmentConstant fieldVerticalAlignment, HorizontalAlignmentConstant fieldHorizontalAlignment, String suffix, VerticalAlignmentConstant suffixVerticalAlignment, HorizontalAlignmentConstant suffixHorizontalAlignment) {
        if (field == null) throw new IllegalArgumentException("field may not be null");
        HorizontalAlignmentConstant h = UiUtils.getLeadingAlignment();
        HorizontalAlignmentConstant i = UiUtils.getTrailingAlignment();
        VerticalAlignmentConstant j = ALIGN_MIDDLE;
        if (nameVerticalAlignment == null) nameVerticalAlignment = j;
        if (nameHorizontalAlignment == null) nameHorizontalAlignment = i;
        if (fieldVerticalAlignment == null) fieldVerticalAlignment = j;
        if (fieldHorizontalAlignment == null) fieldHorizontalAlignment = h;
        if (suffixVerticalAlignment == null) suffixVerticalAlignment = j;
        if (suffixHorizontalAlignment == null) suffixHorizontalAlignment = h;
        c.resizeRows(c.getRowCount() + 1);
        Vector<Widget> k = new Vector<Widget>();
        Vector<VerticalAlignmentConstant> l = new Vector<VerticalAlignmentConstant>();
        Vector<HorizontalAlignmentConstant> m = new Vector<HorizontalAlignmentConstant>();
        if (name != null) {
            Label n = UiUtils.makeLabelFor(name);
            n.setHorizontalAlignment(nameHorizontalAlignment);
            k.add(n);
            k.add(UiUtils.getRelatedHorizontalStrut());
            l.add(nameVerticalAlignment);
            l.add(null);
            m.add(nameHorizontalAlignment);
            m.add(null);
        } else {
            k.add(UiUtils.getRigidArea(0, 0));
            k.add(UiUtils.getRigidArea(0, 0));
            l.add(null);
            l.add(null);
            m.add(null);
            m.add(null);
        }
        k.add(field);
        l.add(fieldVerticalAlignment);
        m.add(fieldHorizontalAlignment);
        if (suffix != null) {
            k.add(UiUtils.getRelatedHorizontalStrut());
            k.add(new Label(suffix));
            l.add(null);
            l.add(suffixVerticalAlignment);
            m.add(null);
            m.add(suffixHorizontalAlignment);
        } else {
            k.add(UiUtils.getRigidArea(0, 0));
            k.add(UiUtils.getRigidArea(0, 0));
            l.add(null);
            l.add(null);
            m.add(null);
            m.add(null);
        }
        UiUtils.setGridRow(c, c.getRowCount() - 1, k, l, m);
    }

    /**
	 * Adds an unrelated gap following the last field.
	 */
    public void addUnrelatedFieldGap() {
        c.resizeRows(c.getRowCount() + 1);
        UiUtils.fillRow(c, c.getRowCount() - 1, GwtUiDefaults.getUnrelatedVerticalDistance());
    }

    /**
	 * Adds a related gap following the last field.
	 */
    public void addRelatedFieldGap() {
        c.resizeRows(c.getRowCount() + 1);
        UiUtils.fillRow(c, c.getRowCount() - 1, GwtUiDefaults.getRelatedVerticalDistance());
    }

    /**
	 * Sets the form buttons. Either or both of the two buttons may be
	 * <code>null</code>.
	 */
    public void setButtons(Button okButton, Button cancelButton) {
        d.clear();
        List<Button> h = new ArrayList<Button>(2);
        if (okButton != null) h.add(okButton);
        if (cancelButton != null) h.add(cancelButton);
        UiUtils.addDialogButtons(d, h);
        this.e = okButton;
    }

    /**
	 * Returns the index of the field with the specified widget; <code>-1</code>
	 * if none.
	 */
    public int getFieldRow(Widget field) {
        for (int h = 0; h < c.getRowCount(); h++) if (field == c.getWidget(h, 2)) return h;
        return -1;
    }

    /**
	 * Removes the specified field from the form.
	 */
    public void removeField(Widget field) {
        int h = getFieldRow(field);
        if (h == -1) throw new IllegalArgumentException("The field " + field + " is not a child of this form");
        removeRow(h);
    }

    /**
	 * Removes the specified row. Unlike {@link #removeField(Widget)}, this
	 * method can be used to remove gaps.
	 */
    public void removeRow(int row) {
        c.removeRow(row);
    }

    /**
	 * Sets the visibility of the specified field.
	 */
    public void setFieldVisible(Widget field, boolean isVisible) {
        int h = getFieldRow(field);
        if (h == -1) throw new IllegalArgumentException("The field " + field + " is not a child of this form");
        for (int i = 0; i < c.getColumnCount(); i++) c.getWidget(h, i).setVisible(isVisible);
    }

    /**
	 * Sets the widget to get the focus by default.
	 */
    public void setDefaultFocusedWidget(Focusable widget) {
        this.f = widget;
    }

    /**
	 * Invoked when this widget is attached to the DOM.
	 */
    protected void onLoad() {
        super.onLoad();
        for (int h = 0; h < d.getWidgetCount(); h++) {
            Widget i = d.getWidget(h);
            if ((i instanceof Button) && (i.getOffsetWidth() < a)) i.setWidth(a + "px");
        }
        if (f != null) {
            Scheduler.get().scheduleDeferred(new ScheduledCommand() {

                public void execute() {
                    f.setFocus(true);
                }
            });
        }
    }

    /***/
    private static class FieldsGrid extends Grid {

        /**
		 * Creates a new Fields
		 */
        public FieldsGrid() {
            super(0, 5);
            UiUtils.prepareLayoutTable(this);
        }

        /**
		 * Removes the specified row.
		 */
        public void removeRow(int row) {
            numRows--;
            super.removeRow(row);
        }
    }
}
