package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.Style;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.BrowserUtils;

/**
 * A panel which can be used to cover a panel and prevent mouse events from
 * reaching it. By default, the glasspane's top and left are set to 0, its
 * width and height to 100% and its z-index to {@link Integer#MAX_VALUE}. 
 *
 * @author Maryanovsky Alexander
 */
public class GlassPane extends Widget {

    /**
	 * Creates a new <code>GlassPane</code>.
	 */
    public GlassPane() {
        setElement(Document.get().createDivElement());
        setColor("black");
        setOpacity(0);
        Style a = getElement().getStyle();
        a.setTop(0, Unit.PX);
        a.setLeft(0, Unit.PX);
        a.setWidth(100, Unit.PCT);
        a.setHeight(100, Unit.PCT);
        a.setZIndex(Integer.MAX_VALUE);
    }

    /**
	 * Sets the color of the glass pane.
	 */
    public void setColor(String color) {
        getElement().getStyle().setBackgroundColor(color);
    }

    /**
	 * Sets the opacity of the glass pane.
	 */
    public void setOpacity(double opacity) {
        if (BrowserUtils.isIE() && opacity == 0) opacity = 0.01;
        getElement().getStyle().setOpacity(opacity);
    }
}
