package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style;
import com.google.gwt.dom.client.Style.Overflow;
import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.resources.client.ImageResource.RepeatStyle;
import com.google.gwt.uibinder.client.UiConstructor;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.SimplePanel;

/**
 * A panel which decorates its contents with a border specified by 16 images
 * (3 at each side and one at each corner). Note that the panel must be either
 * absolutely or relatively positioned, and its size must be set explicitly.
 * 
 * @author Maryanovsky Alexander
 */
public class ImageBordersPanel extends SimplePanel {

    /***/
    private final BorderImages a;

    /***/
    private final Element b;

    /**
	 * Creates a new {@link ImageBordersPanel} with the specified 
	 * {@link BorderImages}.
	 */
    @UiConstructor
    public ImageBordersPanel(BorderImages borderImages) {
        if (borderImages == null) throw new IllegalArgumentException("borderImages may not be null");
        this.a = borderImages;
        this.b = DOM.createDiv();
        c();
    }

    /***/
    private void c() {
        Element f = getElement();
        f.getStyle().setPosition(Position.RELATIVE);
        f.appendChild(d(a.cornerTopLeft(), "0px", "0px", null, null));
        f.appendChild(d(a.cornerTopRight(), "0px", null, null, "0px"));
        f.appendChild(d(a.cornerBottomLeft(), null, "0px", "0px", null));
        f.appendChild(d(a.cornerBottomRight(), null, null, "0px", "0px"));
        f.appendChild(d(a.topLeft(), "0px", a.cornerTopLeft().getWidth() + "px", null, null));
        f.appendChild(d(a.topRight(), "0px", null, null, a.cornerTopRight().getWidth() + "px"));
        f.appendChild(d(a.rightTop(), a.cornerTopRight().getHeight() + "px", null, null, "0px"));
        f.appendChild(d(a.rightBottom(), null, null, a.cornerBottomRight().getHeight() + "px", "0px"));
        f.appendChild(d(a.bottomRight(), null, null, "0px", a.cornerBottomRight().getWidth() + "px"));
        f.appendChild(d(a.bottomLeft(), null, a.cornerBottomLeft().getWidth() + "px", "0px", null));
        f.appendChild(d(a.leftBottom(), null, "0px", a.cornerBottomLeft().getHeight() + "px", null));
        f.appendChild(d(a.leftTop(), a.cornerTopLeft().getHeight() + "px", "0px", null, null));
        f.appendChild(e(a.top(), RepeatStyle.Horizontal, "0px", (a.cornerTopLeft().getWidth() + a.topLeft().getWidth()) + "px", null, (a.cornerTopRight().getWidth() + a.topRight().getWidth()) + "px"));
        f.appendChild(e(a.right(), RepeatStyle.Vertical, (a.cornerTopRight().getHeight() + a.rightTop().getHeight()) + "px", null, (a.cornerBottomRight().getHeight() + a.rightBottom().getHeight()) + "px", "0px"));
        f.appendChild(e(a.bottom(), RepeatStyle.Horizontal, null, (a.cornerBottomLeft().getWidth() + a.bottomLeft().getWidth()) + "px", "0px", (a.cornerBottomRight().getWidth() + a.bottomRight().getWidth()) + "px"));
        f.appendChild(e(a.left(), RepeatStyle.Vertical, (a.cornerTopLeft().getHeight() + a.leftTop().getHeight()) + "px", "0px", (a.cornerBottomLeft().getHeight() + a.leftBottom().getHeight()) + "px", null));
        Style g = b.getStyle();
        g.setPosition(Position.ABSOLUTE);
        g.setTop(a.top().getHeight(), Unit.PX);
        g.setRight(a.right().getWidth(), Unit.PX);
        g.setBottom(a.bottom().getHeight(), Unit.PX);
        g.setLeft(a.left().getWidth(), Unit.PX);
        g.setOverflow(Overflow.HIDDEN);
        f.appendChild(b);
    }

    /**
	 * Returns the element where the child widget is to be placed.
	 */
    @Override
    protected com.google.gwt.user.client.Element getContainerElement() {
        return b.cast();
    }

    /***/
    private static Element d(ImageResource f, String g, String h, String i, String j) {
        Element k = DOM.createDiv();
        Style l = k.getStyle();
        l.setBackgroundImage("url('" + f.getSafeUri().asString() + "')");
        l.setWidth(f.getWidth(), Unit.PX);
        l.setHeight(f.getHeight(), Unit.PX);
        l.setPosition(Position.ABSOLUTE);
        if (g != null) l.setProperty("top", g);
        if (h != null) l.setProperty("left", h);
        if (i != null) l.setProperty("bottom", i);
        if (j != null) l.setProperty("right", j);
        l.setProperty("backgroundPosition", (-f.getLeft()) + "px" + " " + (-f.getTop()) + "px");
        l.setProperty("backgroundRepeat", "no-repeat");
        return k;
    }

    /***/
    private static Element e(ImageResource f, RepeatStyle g, String h, String i, String j, String k) {
        Element l = d(f, h, i, j, k);
        Style m = l.getStyle();
        if (g == RepeatStyle.Horizontal) {
            m.clearWidth();
            m.setProperty("backgroundRepeat", "repeat-x");
        } else if (g == RepeatStyle.Vertical) {
            m.clearHeight();
            m.setProperty("backgroundRepeat", "repeat-y");
        }
        return l;
    }

    /**
	 * The interface specifying the images. Note that the top and bottom images
	 * must be bundled in such a manner that they can be tiled horizontally,
	 * and the left and right images, vertically. If you implement the interface
	 * through GWT's standard resource bundling mechanism, you can achieve that
	 * by annotating the image resource method with, for example, 
	 * <code>@ImageOptions(repeatStyle = RepeatStyle.Horizontal)</code>.
	 */
    public static interface BorderImages {

        ImageResource cornerTopLeft();

        ImageResource cornerTopRight();

        ImageResource cornerBottomRight();

        ImageResource cornerBottomLeft();

        ImageResource topLeft();

        ImageResource top();

        ImageResource topRight();

        ImageResource rightTop();

        ImageResource right();

        ImageResource rightBottom();

        ImageResource bottomRight();

        ImageResource bottom();

        ImageResource bottomLeft();

        ImageResource leftBottom();

        ImageResource left();

        ImageResource leftTop();
    }
}
