package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.MouseDownEvent;
import com.google.gwt.event.dom.client.MouseDownHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.AbstractImagePrototype;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.maryanovsky.gwtutils.client.BrowserUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.buttontap.ButtonTapRecognizer;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.mouse.TapClickEventsConverter;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.mouse.TouchMouseEventsConverter;

/**
 * An button widget with an image (with rollover and selection effects) and an
 * optional label. The primary style name of this widget is
 * <code>"gwtutils-ImageButton"</code>. The dependent style names of the widget
 * are <code>"rollover"</code>, <code>"selected"</code> and
 * <code>"disabled"</code>. The primary style name of the image is
 * <code>"gwtutils-ImageButtonImage"</code> and the primary style name of the
 * label is <code>"gwtutils-ImageButtonLabel"</code>.
 * 
 * @author Maryanovsky Alexander
 */
public class ImageButton extends Composite implements HasClickHandlers {

    /***/
    private final AbstractImagePrototype a;

    /***/
    private final AbstractImagePrototype b;

    /***/
    private final AbstractImagePrototype c;

    /***/
    private final AbstractImagePrototype d;

    /***/
    private final Image e;

    /***/
    private final Label f;

    /***/
    private boolean g = true;

    /**
	 * Creates an {@link ImageButton} with the specified image URL, which will
	 * be used for all button states (normal, rollover, selected, disabled).
	 */
    public ImageButton(String image) {
        this(new UrlImagePrototype(image));
    }

    /**
	 * Creates a new {@link ImageButton} with the specified image resource,
	 * which will be used for all button states (normal, rollover, selected,
	 * disabled).
	 */
    public ImageButton(ImageResource image) {
        this(image, image, image, image);
    }

    /**
	 * Creates an {@link ImageButton} with the specified image prototype, which
	 * will be used for all button states (normal, rollover, selected,
	 * disabled). 
	 */
    public ImageButton(AbstractImagePrototype image) {
        this(image, image, image, image);
    }

    /**
	 * Creates an <code>ImageButton</code> with images retrieved from the
	 * specified URLs.
	 * 
	 * @param normal The location of the normal image.
	 * @param rollover The location of the image to display when the mouse
	 * is over the widget. If <code>null</code>, defaults to the normal image.
	 * @param selected The location of the image to display when the button
	 * is pressed. If <code>null</code>, defaults to the rollover image.
	 * @param disabled The location of the image to display when disabled.
	 * If <code>null</code>, defaults to the normal image.
	 */
    public ImageButton(String normal, String rollover, String selected, String disabled) {
        this(new UrlImagePrototype(normal), rollover == null ? null : new UrlImagePrototype(rollover), selected == null ? null : new UrlImagePrototype(selected), disabled == null ? null : new UrlImagePrototype(disabled));
    }

    public ImageButton(ImageResource normal, ImageResource rollover, ImageResource selected, ImageResource disabled) {
        this(AbstractImagePrototype.create(normal), rollover == null ? null : AbstractImagePrototype.create(rollover), selected == null ? null : AbstractImagePrototype.create(selected), disabled == null ? null : AbstractImagePrototype.create(disabled));
    }

    /**
	 * Creates an {@link ImageButton} with images retrieved from the specified
	 * image prototypes.
	 * 
	 * @param normal The prototype of the normal image.
	 * @param rollover The prototype of the image to display when the mouse
	 * is over the widget. If <code>null</code>, defaults to the normal image.
	 * @param selected The prototype of the image to display when the button
	 * is pressed. If <code>null</code>, defaults to the rollover image.
	 * @param disabled The prototype of the image to display when disabled.
	 * If <code>null</code>, defaults to the normal image.
	 */
    public ImageButton(AbstractImagePrototype normal, AbstractImagePrototype rollover, AbstractImagePrototype selected, AbstractImagePrototype disabled) {
        if (normal == null) throw new IllegalArgumentException("normal may not be null");
        this.a = normal;
        this.b = rollover == null ? normal : rollover;
        this.c = selected == null ? this.b : selected;
        this.d = disabled == null ? this.a : disabled;
        this.e = normal.createImage();
        this.f = new Label("", false);
        e.setStyleName("gwtutils-ImageButtonImage");
        f.setStyleName("gwtutils-ImageButtonLabel");
        VerticalPanel h = new VerticalPanel();
        h.setHorizontalAlignment(VerticalPanel.ALIGN_CENTER);
        h.add(e);
        h.add(f);
        UiUtils.extraImageCellHeightWorkaround(DOM.getParent(e.getElement()));
        FocusPanel i = new FocusPanel(h);
        initWidget(i);
        i.addMouseDownHandler(new MouseDownHandler() {

            public void onMouseDown(MouseDownEvent evt) {
                if (!isEnabled()) return;
                ImageButton.this.c.applyTo(e);
                addStyleDependentName("selected");
            }
        });
        i.addMouseUpHandler(new MouseUpHandler() {

            public void onMouseUp(MouseUpEvent evt) {
                ImageButton.this.b.applyTo(e);
                removeStyleDependentName("selected");
            }
        });
        i.addMouseOverHandler(new MouseOverHandler() {

            public void onMouseOver(MouseOverEvent evt) {
                Element j = evt.getRelatedTarget().cast();
                if ((j == null) || !getElement().isOrHasChild(j)) {
                    ImageButton.this.b.applyTo(e);
                    addStyleDependentName("rollover");
                }
            }
        });
        i.addMouseOutHandler(new MouseOutHandler() {

            public void onMouseOut(MouseOutEvent evt) {
                Element j = evt.getRelatedTarget().cast();
                if ((j == null) || !getElement().isOrHasChild(j)) {
                    ImageButton.this.a.applyTo(e);
                    removeStyleDependentName("rollover");
                }
            }
        });
        if (BrowserUtils.hasTouchEventsSupport()) {
            new TouchMouseEventsConverter(i, i).setTouchEventsSource(i);
            new TapClickEventsConverter(this).setButtonTapEventsSource(new ButtonTapRecognizer(i, i));
        }
        setStylePrimaryName("gwtutils-ImageButton");
    }

    /**
	 * Sets the text of the button.
	 */
    public void setText(String text) {
        boolean h = (text == null) || (text.length() == 0);
        f.setText(text);
        f.setVisible(!h);
    }

    /**
	 * Returns the text of the button.
	 */
    public String getText() {
        return f.getText();
    }

    /**
	 * Sets the enabled state of the button.
	 */
    public void setEnabled(boolean isEnabled) {
        this.g = isEnabled;
        if (isEnabled()) {
            a.applyTo(e);
            removeStyleDependentName("disabled");
        } else {
            d.applyTo(e);
            addStyleDependentName("disabled");
        }
    }

    /**
	 * Returns the enabled state of the button.
	 */
    public boolean isEnabled() {
        return g;
    }

    /**
	 * Registers a handler for {@link ClickEvent}s.
	 */
    public HandlerRegistration addClickHandler(ClickHandler handler) {
        return addDomHandler(handler, ClickEvent.getType());
    }

    /**
	 * Prevent click events from being fired if the button is disabled.
	 */
    @Override
    public void onBrowserEvent(Event evt) {
        UiUtils.cancelEvent(evt);
        if ((evt.getTypeInt() == Event.ONCLICK) && !isEnabled()) return;
        super.onBrowserEvent(evt);
    }
}
