package com.maryanovsky.gwtutils.client.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.maryanovsky.gwtutils.client.Pair;
import com.maryanovsky.gwtutils.client.Utils;

/**
 * <p>Implements a queue of images to be loaded by the browser. This is needed
 * because some browsers (*cough* Internet Explorer *cough*) break when asked to
 * load many images and then cancel loading some (the breakage manifests in the
 * network hanging for a few minutes). This class carefully manages the process
 * of loading the images, never feeding the browser too many, and cancelling
 * downloads carefully so as to avoid the browser breaking.
 * <p>Note that the behaviour of this class might differ from that of a
 * perfectly behaving browser if you simply use an Image object for each image
 * you wish to download. For example, a browser knows whether a certain image
 * is already in the cache, and so, attempting to load it, will return the
 * result immediately. This class, however, if the loading slots are full, will
 * put the image into a queue regardless of its caching status, and will only
 * handle it when all the images in front of it have been handled. Because some
 * of them may not be cached, it may take a while until that occurs.
 * 
 * @author Maryanovsky Alexander
 */
public class ImageLoadingManager {

    /***/
    private static final int a = 2;

    /***/
    private static final String b = "gwt_sourceUrl";

    /***/
    private static final Map<String, Pair<List<Pair<String, ImageLoadListener>>, Counter>> c = new HashMap<String, Pair<List<Pair<String, ImageLoadListener>>, Counter>>();

    /***/
    private static final List<JavaScriptObject> d = new ArrayList<JavaScriptObject>();

    /***/
    private static String e(String n) {
        String o = Utils.getHostname(n);
        if (o == null) o = Utils.getHostname(GWT.getHostPageBaseURL());
        return (o == null) ? "" : o;
    }

    /**
	 * Adds an image to be loaded, and notify the specified
	 * <code>ImageLoadListener</code> of the loading status. Note that, for
	 * cached images, the listener may be called right from within the call to
	 * <code>load</code>.
	 */
    public static void load(String url, ImageLoadListener listener) {
        String n = e(url);
        Pair<List<Pair<String, ImageLoadListener>>, Counter> o = (Pair<List<Pair<String, ImageLoadListener>>, Counter>) c.get(n);
        if (o == null) {
            o = new Pair<List<Pair<String, ImageLoadListener>>, Counter>(new ArrayList<Pair<String, ImageLoadListener>>(), new Counter(0));
            c.put(n, o);
        }
        List<Pair<String, ImageLoadListener>> p = o.getFirst();
        p.add(new Pair<String, ImageLoadListener>(url, listener));
        f(n);
    }

    /**
	 * Removes the specified url from the loading queue. Note that if the image
	 * is already being downloaded, it will continue being downloaded, but
	 * transparently, to the caller of this method, that is, the listener will
	 * not be called when the image is done loading.
	 */
    public static void cancel(String url, ImageLoadListener listener) {
        String n = e(url);
        Pair<List<Pair<String, ImageLoadListener>>, Counter> o = (Pair<List<Pair<String, ImageLoadListener>>, Counter>) c.get(n);
        if (o == null) return;
        List<Pair<String, ImageLoadListener>> p = o.getFirst();
        Counter q = o.getSecond();
        int r = -1;
        for (int s = 0; s < p.size(); s++) {
            Pair<String, ImageLoadListener> t = p.get(s);
            if (url.equals(t.getFirst()) && (listener == t.getSecond())) {
                r = s;
                break;
            }
        }
        if (r != -1) {
            if (r < q.get()) {
                p.set(r, new Pair<String, ImageLoadListener>(url, null));
            } else {
                p.remove(r);
                if (p.isEmpty()) c.remove(n);
            }
        }
    }

    /***/
    private static void f(String n) {
        Pair<List<Pair<String, ImageLoadListener>>, Counter> o = (Pair<List<Pair<String, ImageLoadListener>>, Counter>) c.get(n);
        if (o == null) return;
        List<Pair<String, ImageLoadListener>> p = o.getFirst();
        Counter q = o.getSecond();
        if ((q.get() < a) && (q.get() < p.size())) {
            String r = p.get(q.get()).getFirst();
            JavaScriptObject s = g();
            q.increase();
            i(s, r);
        }
    }

    /***/
    private static JavaScriptObject g() {
        if (d.isEmpty()) return makeJsImage(); else return d.remove(d.size() - 1);
    }

    /***/
    private static void h(JavaScriptObject n) {
        d.add(n);
    }

    /***/
    private static native JavaScriptObject makeJsImage()/*-{
		var img = new Image();
		img.onload = function(){
			@com.maryanovsky.gwtutils.client.ui.ImageLoadingManager::onImageLoad(Lcom/google/gwt/core/client/JavaScriptObject;)(img);
		};
		img.onerror = function(){
			@com.maryanovsky.gwtutils.client.ui.ImageLoadingManager::onImageError(Lcom/google/gwt/core/client/JavaScriptObject;)(img);
		};
		img.onabort = function(){
			@com.maryanovsky.gwtutils.client.ui.ImageLoadingManager::onImageAbort(Lcom/google/gwt/core/client/JavaScriptObject;)(img);
		}
		return img;
	}-*/;

    /***/
    private static void i(JavaScriptObject n, String o) {
        Utils.setStringProperty(n, b, o);
        Utils.setStringProperty(n, "src", o);
    }

    /***/
    private static void j(String n, boolean o, JavaScriptObject p) {
        int q = o ? Utils.getIntProperty(p, "width") : -1;
        int r = o ? Utils.getIntProperty(p, "height") : -1;
        h(p);
        final String s = e(n);
        Pair<List<Pair<String, ImageLoadListener>>, Counter> t = (Pair<List<Pair<String, ImageLoadListener>>, Counter>) c.get(s);
        List<Pair<String, ImageLoadListener>> u = t.getFirst();
        Counter v = t.getSecond();
        int w = k(u, n);
        if (w == -1) {
            return;
        }
        Pair<String, ImageLoadListener> x = u.remove(w);
        if (u.isEmpty()) c.remove(s); else {
            v.decrease();
            Scheduler.get().scheduleDeferred(new ScheduledCommand() {

                public void execute() {
                    f(s);
                }
            });
        }
        ImageLoadListener y = x.getSecond();
        if (o) l(y, n, q, r); else m(y, n);
    }

    /***/
    private static int k(List<Pair<String, ImageLoadListener>> n, String o) {
        for (int p = 0; p < n.size(); p++) {
            Pair<String, ImageLoadListener> q = n.get(p);
            if (o.equals(q.getFirst())) return p;
        }
        return -1;
    }

    /***/
    private static void l(ImageLoadListener n, String o, int p, int q) {
        if (n == null) return;
        try {
            n.imageLoaded(o, p, q);
        } catch (Throwable r) {
            r.printStackTrace();
        }
    }

    /***/
    private static void m(ImageLoadListener n, String o) {
        if (n == null) return;
        try {
            n.imageFailed(o);
        } catch (Throwable p) {
            p.printStackTrace();
        }
    }

    /**
	 * Invoked when the image finishes loading normally.
	 */
    public static void onImageLoad(JavaScriptObject image) {
        String n = Utils.getStringProperty(image, b);
        j(n, true, image);
    }

    /**
	 * Invoked when the image errors while loading.
	 */
    public static void onImageError(JavaScriptObject image) {
        String n = Utils.getStringProperty(image, b);
        j(n, false, image);
    }

    /**
	 * Invoked when the loading of the image was aborted.
	 */
    public static void onImageAbort(JavaScriptObject image) {
        String n = Utils.getStringProperty(image, b);
        j(n, false, image);
    }

    /***/
    private static class Counter {

        /***/
        private int n;

        /**
		 * Creates a new <code>Counter</code> with the specified initial
		 * value.
		 */
        public Counter(int value) {
            this.n = value;
        }

        /**
		 * Returns the current value.
		 */
        public int get() {
            return n;
        }

        /**
		 * Decreases the current value by one.
		 */
        public void decrease() {
            n--;
        }

        /**
		 * Increases the current value by one.
		 */
        public void increase() {
            n++;
        }
    }
}
