package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.ComplexPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A panel which piles its children on top of each other, each placed at
 * <code>0,0</code> and fills the 100% of the width and height of the panel.
 * 
 * @author Maryanovsky Alexander
 */
public class PilePanel extends ComplexPanel {

    /**
	 * Creates a new <code>PilePanel</code>.
	 */
    public PilePanel() {
        setElement(DOM.createDiv());
        DOM.setStyleAttribute(getElement(), "position", "relative");
        DOM.setStyleAttribute(getElement(), "zIndex", "0");
        DOM.setStyleAttribute(getElement(), "overflow", "hidden");
    }

    /**
	 * Adds the specified widget to this panel at z-index 0. Widgets with same
	 * z-index value appear in the order they were added (later ones on top).
	 */
    public void add(Widget widget) {
        add(widget, 0);
    }

    /**
	 * Adds the specified widget at the specified z-index. Widgets with higher
	 * z-index values appear on top of widgets with lower z-index values.
	 */
    public void add(Widget widget, int zIndex) {
        UiUtils.setAbsoluteFullSize(widget);
        add(widget, getElement());
        DOM.setStyleAttribute(widget.getElement(), "zIndex", String.valueOf(zIndex));
    }

    /**
	 * Returns the z-index of the specified child widget.
	 */
    public int getZIndex(Widget widget) {
        return DOM.getIntStyleAttribute(widget.getElement(), "zIndex");
    }

    /**
	 * Returns the widget at the top (the one with the highest z-index).
	 */
    public Widget getTopWidget() {
        if (getWidgetCount() == 0) return null;
        Widget a = getWidget(0);
        int b = getZIndex(a);
        for (int c = 1; c < getWidgetCount(); c++) {
            Widget d = getWidget(c);
            int e = getZIndex(d);
            if (e >= b) {
                a = d;
                b = e;
            }
        }
        return a;
    }
}
