package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.dom.client.AnchorElement;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A plain HTML hyperlink, you know, the normal kind, which opens a URL when
 * clicked. 
 *  
 * @author Maryanovsky Alexander
 */
public class PlainHyperlink extends SimplePanel {

    /***/
    private final AnchorElement a;

    /**
	 * Creates a new {@link PlainHyperlink}, not yet pointing anywhere.
	 */
    public PlainHyperlink() {
        super(DOM.createAnchor());
        this.a = AnchorElement.as(getElement());
    }

    /**
	 * Creates a new {@link PlainHyperlink} pointing to the specified URL.
	 * 
	 * @param url The URL to open when the hyperlink is clicked. May be
	 * <code>null</code>, in which case it should be set later.
	 */
    public PlainHyperlink(String url) {
        this();
        setUrl(url);
    }

    /**
	 * A convenience constructor for creating a <em>really</em> plain link,
	 * whose content is just text.
	 * 
	 * @param url The URL to open when the hyperlink is clicked.
	 * @param content The linked text.
	 */
    public PlainHyperlink(String url, String content) {
        this(url);
        setContent(content);
    }

    /**
	 * Creates a new {@link PlainHyperlink} to point to the specified URL
	 * and contain the specified widget.
	 */
    public PlainHyperlink(String url, Widget content) {
        this(url);
        setContent(content);
    }

    /**
	 * Returns the {@link AnchorElement} this {@link PlainHyperlink} wraps.
	 */
    public AnchorElement getAnchor() {
        return a;
    }

    /**
	 * Sets the URL the link points to (the <code>href</code> attribute of the
	 * anchor). A <code>null</code> value removes the attribute.
	 */
    public void setUrl(String url) {
        a.setHref(url);
    }

    /**
	 * Returns the URL the link points to.
	 */
    public String getUrl() {
        return a.getHref();
    }

    /**
	 * Sets the content of the link - the widget which may be clicked to follow
	 * to the URL.
	 */
    public void setContent(Widget content) {
        setWidget(content);
    }

    /**
	 * A convenience method which sets the content of the link to be a simple
	 * {@linkplain Label} with the specified text.
	 */
    public void setContent(String content) {
        setContent(new Label(content));
    }

    /**
	 * Returns the content of the link - the widget which may be clicked to
	 * follow to the URL.
	 */
    public Widget getContent() {
        return getWidget();
    }

    /**
	 * Sets the <code>target</code> attribute of the hyperlink to the specified
	 * value. Specifying <code>null</code> removes the attribute.
	 */
    public void setTarget(String target) {
        a.setTarget(target);
    }
}
