package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.AbstractImagePrototype;

/**
 * A toggle button based on the {@link ImageButton} class.
 * The primary style name for this widget is 
 * <code>"gwtutils-ToggleImageButton"</code>. Toggled buttons
 * have a dependent style name of <code>"toggled"</code>.
 * 
 * @author Maryanovsky Alexander
 */
public class ToggleImageButton extends ImageButton {

    /***/
    private boolean a;

    /**
	 * Creates a {@link ToggleImageButton} with the specified image URL, which
	 * will be used for all button states (normal, rollover, selected, disabled).
	 */
    public ToggleImageButton(String image) {
        this(new UrlImagePrototype(image));
    }

    /**
	 * Creates a new {@link ToggleImageButton} with the specified image
	 * prototype, which will be used for all button states (normal, rollover,
	 * selected, disabled). 
	 */
    public ToggleImageButton(AbstractImagePrototype image) {
        this(image, image, image, image);
    }

    /**
	 * Creates a {@link ToggleImageButton} with images retrieved from the
	 * specified URLs.
	 * 
	 * @param normal The location of the normal image.
	 * @param rollover The location of the image to display when the mouse
	 * is over the widget. If <code>null</code>, defaults to the normal image.
	 * @param selected The location of the image to display when the button
	 * is pressed. If <code>null</code>, defaults to the rollover image.
	 * @param disabled The location of the image to display when disabled.
	 * If <code>null</code>, defaults to the normal image.
	 */
    public ToggleImageButton(String normal, String rollover, String selected, String disabled) {
        this(new UrlImagePrototype(normal), rollover == null ? null : new UrlImagePrototype(rollover), selected == null ? null : new UrlImagePrototype(selected), disabled == null ? null : new UrlImagePrototype(disabled));
    }

    /**
	 * Creates an {@link ImageButton} with images retrieved from the specified
	 * image prototypes.
	 * 
	 * @param normal The prototype of the normal image.
	 * @param rollover The prototype of the image to display when the mouse
	 * is over the widget. If <code>null</code>, defaults to the normal image.
	 * @param selected The prototype of the image to display when the button
	 * is pressed. If <code>null</code>, defaults to the rollover image.
	 * @param disabled The prototype of the image to display when disabled.
	 * If <code>null</code>, defaults to the normal image.
	 */
    public ToggleImageButton(AbstractImagePrototype normal, AbstractImagePrototype rollover, AbstractImagePrototype selected, AbstractImagePrototype disabled) {
        super(normal, rollover, selected, disabled);
        setStylePrimaryName("gwtutils-ToggleImageButton");
        addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent event) {
                if (isEnabled()) setToggled(!isToggled());
            }
        });
    }

    /**
	 * Sets this button's toggled state.
	 */
    public void setToggled(boolean isToggled) {
        this.a = isToggled;
        if (isToggled) addStyleDependentName("toggled"); else removeStyleDependentName("toggled");
    }

    /**
	 * Returns whether this button is currently toggled.
	 */
    public boolean isToggled() {
        return a;
    }
}
