package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.user.client.ui.AbstractImagePrototype;
import com.google.gwt.user.client.ui.Image;
import com.maryanovsky.gwtutils.client.BrowserUtils;
import com.maryanovsky.gwtutils.client.geom.Size;

/**
 * An image prototype which returns images from a certain URL.
 * 
 * @author Maryanovsky Alexander
 */
public class UrlImagePrototype extends AbstractImagePrototype {

    /***/
    private final String a;

    /***/
    private Size b;

    /***/
    private final boolean c;

    /**
	 * Creates a new {@link UrlImagePrototype} with the specified image URL.
	 */
    public UrlImagePrototype(String url) {
        this(url, null);
    }

    /**
	 * Creates a new {@link UrlImagePrototype} with the specified URL and an
	 * optional size.
	 */
    public UrlImagePrototype(String url, Size size) {
        this(url, size, url.endsWith(".png") || url.endsWith(".PNG"));
    }

    /**
	 * Creates a new {@link UrlImagePrototype} with the specified URL, an
	 * optional size and a flag specifying whether the image is a PNG image.
	 * PNG images will have the PNG hack applied to them under IE versions which
	 * don't support PNG transparency. If you are using a PNG image on such a
	 * browser it is advised to also pass the image's size.
	 * 
	 * @param url The URL of the image.
	 * @param imageSize The size of the image; may be <code>null</code> for the
	 * image's inherent size.
	 * @param isPng Whether the image is to be treated as a PNG image for the
	 * purpose of the IE PNG hack.
	 */
    public UrlImagePrototype(String url, Size imageSize, boolean isPng) {
        if (url == null) throw new IllegalArgumentException("url may not be null");
        this.a = url;
        this.b = imageSize;
        this.c = isPng;
    }

    /***/
    private boolean d(Image e) {
        if (c && BrowserUtils.isIE6()) {
            UiUtils.applyPngHack(e, getUrl(), b);
            return true;
        }
        return false;
    }

    /**
	 * Returns the image's URL.
	 */
    public String getUrl() {
        return a;
    }

    /**
	 * {@inheritDoc}
	 */
    public void applyTo(Image image) {
        if (!d(image)) {
            image.setUrl(getUrl());
            if (b != null) image.setPixelSize(b.getWidth(), b.getHeight());
        }
    }

    /**
	 * {@inheritDoc}
	 */
    public Image createImage() {
        Image e = new Image();
        applyTo(e);
        return e;
    }

    /**
	 * {@inheritDoc}
	 */
    public String getHTML() {
        if (c && BrowserUtils.isIE6()) return UiUtils.getPngHackedImageHtml(a, b); else if (b == null) return "<img src=\"" + a + "\"/>"; else return "<img src=\"" + a + "\" " + "style=\"" + "width: " + b.getWidth() + "px;" + "height: " + b.getWidth() + "px;" + "\" " + "/>";
    }
}
