package com.maryanovsky.gwtutils.client.ui;

import java.util.List;
import java.util.Vector;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A wizard is a multi-stage dialog.
 *
 * @author Maryanovsky Alexander
 */
public class Wizard extends Dialog {

    /***/
    private final SimplePanel a;

    /***/
    private final List<Widget> b = new Vector<Widget>();

    /***/
    private int c;

    /**
	 * Creates a new Wizard with the specified dialog callback.
	 */
    public Wizard(DialogCallback dialogCallback) {
        super(dialogCallback);
        this.a = new SimplePanel();
        initWidget(a);
    }

    /**
	 * Adds a stage to the (end of the) stages of this wizard. A wizard with no
	 * stages will confirm (with a <code>null</code> value) immediately when
	 * shown.
	 */
    protected void addStage(Widget stageWidget) {
        b.add(stageWidget);
    }

    /**
	 * Displays the first stage, or confirms (in a deferred command) if there
	 * are no stages.
	 */
    protected void onAttach() {
        super.onAttach();
        c = 0;
        if (b.size() == 0) {
            Scheduler.get().scheduleDeferred(new ScheduledCommand() {

                public void execute() {
                    dialogCallback.confirmed(null);
                }
            });
        } else a.setWidget(b.get(0));
    }

    /**
	 * Moves to the next stage. 
	 */
    protected void showNextStage() {
        if (c == b.size() - 1) throw new IllegalStateException("Already at the last stage");
        a.clear();
        c++;
        a.setWidget(b.get(c));
    }

    /**
	 * Returns dialog callback which can be used in intermediate dialogs to deny
	 * if the user denies, and continue to the next stage if he confirms.
	 */
    protected DialogCallback makeContinuationCallback() {
        return new DialogCallback() {

            public void confirmed(Object result) {
                showNextStage();
            }

            public void denied() {
                dialogCallback.denied();
            }
        };
    }

    /**
	 * Returns a dialog callback which can be used as the callback of the last
	 * stage - it denies to the global callback if the user denies, and confirms
	 * if the user confirms.
	 */
    protected DialogCallback makeFinishCallback() {
        return new DialogCallback() {

            public void confirmed(Object result) {
                dialogCallback.confirmed(result);
            }

            public void denied() {
                dialogCallback.denied();
            }
        };
    }
}
