package com.maryanovsky.gwtutils.client.ui;

import com.google.gwt.dom.client.Style;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.ProvidesResize;
import com.google.gwt.user.client.ui.RequiresResize;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A {@link SimplePanel} which makes any child widget fill it completely. Note
 * that it does so by setting the child's top, left, width and height CSS
 * properties and does not revert them when the child is removed.
 * 
 * @author Maryanovsky Alexander
 */
public class WrapperPanel extends SimplePanel implements RequiresResize, ProvidesResize {

    /**
	 * Creates a new {@link WrapperPanel}.
	 */
    public WrapperPanel() {
        DOM.setStyleAttribute(getElement(), "position", "relative");
    }

    /**
	 * Creates a new {@link WrapperPanel} with the specified widget as its
	 * child.
	 */
    public WrapperPanel(Widget widget) {
        this();
        setWidget(widget);
    }

    /**
	 * Overrides set<code>setWidget</code> to set the child's position and size.
	 */
    public void setWidget(Widget widget) {
        super.setWidget(widget);
        if (widget != null) {
            Style a = widget.getElement().getStyle();
            a.setTop(0, Unit.PX);
            a.setLeft(0, Unit.PX);
            a.setWidth(100, Unit.PCT);
            a.setHeight(100, Unit.PCT);
        }
    }

    /**
	 * If the child widget implements <code>RequiresResize</code>, passes the
	 * invocation to it.
	 */
    public void onResize() {
        Widget a = getWidget();
        if (a instanceof RequiresResize) ((RequiresResize) a).onResize();
    }
}
