package com.maryanovsky.gwtutils.client.ui.animations;

import com.google.gwt.dom.client.Element;
import com.maryanovsky.gwtutils.client.ui.Animation;

/**
 * An animation that changes the opacity of a specified element.
 * 
 * @author Maryanovsky Alexander
 */
public class FadeAnimation extends Animation {

    /***/
    private final Element a;

    /***/
    private final double b;

    /***/
    private final double c;

    /**
	 * Creates a new {@link FadeAnimation} with the specified duration, frame
	 * rate, element, and initial and final opacity.
	 */
    public FadeAnimation(int duration, int fps, Element element, double startOpacity, double endOpacity) {
        super(duration, fps);
        if (element == null) throw new IllegalArgumentException("element may not be null");
        if ((startOpacity < 0) || (startOpacity > 1)) throw new IllegalArgumentException("startOpacity must be between 0 and 1");
        if ((endOpacity < 0) || (endOpacity > 1)) throw new IllegalArgumentException("endOpacity must be between 0 and 1");
        this.a = element;
        this.b = startOpacity;
        this.c = endOpacity;
    }

    /**
	 * Creates a new {@link FadeAnimation} with the specified duration, element
	 * and initial and final opacity. 
	 */
    public FadeAnimation(int duration, Element element, double startOpacity, double endOpacity) {
        this(duration, Animation.DEFAULT_FPS, element, startOpacity, endOpacity);
    }

    /**
	 * Invoked to start the animation.
	 */
    @Override
    protected void animationStart() {
        a.getStyle().setOpacity(b);
    }

    /**
	 * Invoked for each frame of the animation.
	 */
    @Override
    protected void nextFrame() {
        a.getStyle().setOpacity(b + getLinearProgress() * (c - b));
    }

    /**
	 * Invoked to end the animation.
	 */
    @Override
    protected void animationEnd() {
        a.getStyle().setOpacity(c);
    }
}
