package com.maryanovsky.gwtutils.client.ui.dnd;

/**
 * A listener interface for receiving notifications of various events during a 
 * drag-and-drop gesture.
 * 
 * @author Maryanovsky Alexander
 */
public interface DndListener {

    /**
	 * Invoked when a drag-and-drop gesture starts, dragging the specified
	 * {@link Draggable}.
	 */
    void onDragStart(Draggable draggable);

    /**
	 * Invoked when the {@link Draggable} is dragged into the specified
	 * {@link DropTarget}.
	 */
    void onDragInto(Draggable draggable, DropTarget dropTarget);

    /**
	 * Invoked when the {@link Draggable} is dragged out of the specified
	 * {@link DropTarget}.
	 */
    void onDragOut(Draggable draggable, DropTarget dropTarget);

    /**
	 * Invoked when the {@link Draggable} is dropped into the specified
	 * {@link DropTarget}. Note that the drop target <strong>can</strong>
	 * be <code>null</code> if the user performed a drag gesture but not into a
	 * valid drop target.
	 */
    void onDrop(Draggable draggable, DropTarget dropTarget);

    /**
	 * Invoked when the drag-and-drop gesture has been canceled.
	 */
    void onDragCancel(Draggable draggable);
}
