package com.maryanovsky.gwtutils.client.ui.dnd;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import com.google.gwt.dom.client.Style;
import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.event.dom.client.HasAllMouseHandlers;
import com.google.gwt.event.dom.client.MouseMoveEvent;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.MathUtils;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.geom.Rectangle;
import com.maryanovsky.gwtutils.client.ui.GlassPane;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.drag.*;

/**
 * A utility class for drag-and-drop support.
 * 
 * @author Maryanovsky Alexander
 */
public class DragAndDrop {

    /**
	 * Makes the specified {@link Draggable} actually draggable into the
	 * specified set of {@link DropTarget}s. The specified set of 
	 * {@link DndListener}s will be notified of the various events ocurring
	 * during the drag-and-drop gesture. 
	 * The method returns a "drag manager" object which can then be passed to
	 * {@link #makeUndraggable(Object)} to disable the effect of this method
	 * on the draggable. 
	 */
    public static Object makeDraggable(Draggable draggable, Collection<? extends DropTarget> targets, Collection<? extends DndListener> listeners) {
        return new DragManager(draggable, targets, listeners);
    }

    /**
	 * Makes the Draggable
	 */
    public static void makeUndraggable(Object dragManager) {
        if (!(dragManager instanceof DragManager)) throw new IllegalArgumentException("The specified argument is not a \"drag manager\"");
        ((DragManager) dragManager).disable();
    }

    /***/
    private static class DragManager extends HandlesAllDragEvents {

        /***/
        private static final int a = 10;

        /***/
        private static final Widget b;

        static {
            b = new GlassPane();
            Style k = b.getElement().getStyle();
            k.setPosition(Position.FIXED);
            b.setVisible(false);
            RootPanel.get().add(b);
        }

        /***/
        private final Draggable c;

        /***/
        private final Collection<DropTarget> d;

        /***/
        private final List<DndListener> e;

        /***/
        private final DragRecognizer f;

        /***/
        private Widget g = null;

        /***/
        private DropTarget h = null;

        /***/
        private Point i = null;

        /**
		 * Creates a new <code>DragManager</code> for the specified draggable
		 * and collection of drop targets.
		 */
        public DragManager(Draggable draggable, Collection<? extends DropTarget> dropTargets, Collection<? extends DndListener> listeners) {
            if (draggable == null) throw new IllegalArgumentException("draggable may not be null");
            if (dropTargets == null) throw new IllegalArgumentException("dropTargets may not be null");
            if (dropTargets.isEmpty()) throw new IllegalArgumentException("dropTargets may not be empty");
            this.c = draggable;
            this.d = new ArrayList<DropTarget>(dropTargets);
            this.e = listeners == null ? Collections.<DndListener>emptyList() : new ArrayList<DndListener>(listeners);
            this.f = new DragRecognizer(this, draggable.getDragHandle(), DragRecognizer.STANDARD_DRAG_CONDITION);
            handle(f);
        }

        /**
		 * Disables this drag manager. It cannot be re-enabled.
		 */
        public void disable() {
            f.setMouseEventsSource((HasAllMouseHandlers) null);
        }

        /***/
        private DropTarget j() {
            Rectangle k = UiUtils.getBoundsClient(g);
            double l = 0;
            DropTarget m = null;
            Rectangle n = new Rectangle();
            for (DropTarget o : d) {
                Rectangle p = UiUtils.getBoundsClient(o.getDropWidget());
                Rectangle q = k.getIntersection(p, n);
                if (q != null) {
                    double r = MathUtils.sqr(q.getWidth()) + MathUtils.sqr(q.getHeight());
                    if (r > l) {
                        l = r;
                        m = o;
                    }
                }
            }
            return m;
        }

        /**
		 * {@inheritDoc} 
		 */
        public void onDragStart(DragStartEvent evt) {
            PointView k = evt.getLocation();
            i = k.getDistancePoint(c.getDragHandleOffset(), i);
        }

        /**
		 * {@inheritDoc}
		 */
        public void onDragCancel(DragCancelEvent evt) {
            if (g == null) return;
            for (DropTarget k : d) k.onDragCancel(c);
            c.onDragCancel();
            for (DndListener k : e) k.onDragCancel(c);
            RootPanel.get().remove(g);
            g = null;
            h = null;
            b.setVisible(false);
        }

        /**
		 * {@inheritDoc}
		 */
        public void onDragEnd(DragEndEvent evt) {
            if (g == null) return;
            for (DropTarget k : d) if (k != h) k.onDragCancel(c);
            if (h == null) {
                c.onDrop(null);
                for (DndListener k : e) k.onDrop(c, null);
            } else {
                Rectangle k = UiUtils.getBoundsClient(g);
                Rectangle l = UiUtils.getBoundsClient(h.getDropWidget());
                h.receive(c, l.getTopLeft().getDistancePoint(k.getCenter()));
                c.onDrop(h);
                for (DndListener m : e) m.onDrop(c, h);
            }
            RootPanel.get().remove(g);
            g = null;
            h = null;
            b.setVisible(false);
        }

        /**
		 * {@inheritDoc}
		 */
        public void onDragMove(DragMoveEvent evt) {
            if (g != null) {
                MouseMoveEvent k = evt.getSourceEvent();
                int l = k.getClientX();
                int m = k.getClientY();
                k.getNativeEvent().stopPropagation();
                k.getNativeEvent().preventDefault();
                RootPanel.get().setWidgetPosition(g, l + i.getX(), m + i.getY());
                DropTarget n = j();
                if ((h != null) && (h != n)) {
                    h.onDragOut(c);
                    for (DndListener o : e) o.onDragOut(c, h);
                }
                h = n;
                if (h != null) {
                    PointView o = UiUtils.getBoundsClient(g).getCenter();
                    Rectangle p = UiUtils.getBoundsClient(h.getDropWidget());
                    PointView q = o.getDistancePoint(p.getTopLeft());
                    h.onDragInto(c, q);
                    for (DndListener r : e) r.onDragInto(c, h);
                }
            } else if (evt.getOffset().getDistanceFromOrigin() >= a) {
                MouseMoveEvent k = evt.getSourceEvent();
                int l = k.getClientX();
                int m = k.getClientY();
                g = c.getDraggedWidget();
                RootPanel.get().add(g, l + i.getX(), m + i.getY());
                g.getElement().getStyle().setZIndex(Integer.MAX_VALUE - 1);
                c.onDragStart();
                for (DropTarget n : d) n.onDragStart(c);
                for (DndListener n : e) n.onDragStart(c);
                b.setVisible(true);
            }
        }
    }
}
