package com.maryanovsky.gwtutils.client.ui.dnd;

import com.google.gwt.event.dom.client.HasAllMouseHandlers;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.geom.PointView;

/**
 * The interface to be implemented by objects that can be dragged and dropped.
 * 
 * @author Maryanovsky Alexander
 */
public interface Draggable {

    /**
	 * Returns the widget to be dragged and dropped.
	 */
    Widget getDraggedWidget();

    /**
	 * Returns the handle by which the object can be dragged. Note that the
	 * handle cannot be part of the dragged widget and must remain attached to
	 * the DOM and unmoved for the entire duration of the drag-and-drop gesture.
	 * This is because during the drag-and-drop gesture, the dragged widget is
	 * removed from its parent (if any) and added to the root panel where it is
	 * made to follow the mouse cursor. If the handle were part of the dragged
	 * widget we would not be able to continue receiving mouse events from it
	 * during the entire gesture. 
	 */
    HasAllMouseHandlers getDragHandle();

    /**
	 * Returns the offset of the drag handle from the dragged widget when not
	 * in the middle of a drag-and-drop gesture.
	 */
    PointView getDragHandleOffset();

    /**
	 * Informs the draggable that it has started being dragged.
	 */
    void onDragStart();

    /**
	 * Informs the draggable that it has been dropped into the specified drop
	 * target. Note that the drop target <strong>can</strong> be
	 * <code>null</code> if the user performed a drag gesture but not into a
	 * valid drop target.
	 */
    void onDrop(DropTarget dropTarget);

    /**
	 * Informs the draggable that the drag-and-drop gesture has been canceled.
	 */
    void onDragCancel();
}
