package com.maryanovsky.gwtutils.client.ui.dnd;

import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.geom.PointView;

/**
 * The interface for objects into which draggables can be dropped. 
 * 
 * @author Maryanovsky Alexander
 */
public interface DropTarget {

    /**
	 * Returns the widget into which the draggable is dropped.
	 */
    Widget getDropWidget();

    /**
	 * Invoked when a drag gesture starts with this object one of its drop
	 * targets.
	 */
    void onDragStart(Draggable draggable);

    /**
	 * Invoked when a {@link Draggable} is dragged over the drop widget at the
	 * specified coordinates (in the coordinate system of the drop widget). Note
	 * that this doesn't mean that it has been <b>dropped</b>.
	 */
    void onDragInto(Draggable draggable, PointView location);

    /**
	 * Invoked when a {@link Draggable} that has been previously dragged into
	 * this drop target, has now been dragged out of it.
	 */
    void onDragOut(Draggable draggable);

    /**
	 * Invoked when a drag gesture is canceled.
	 */
    void onDragCancel(Draggable draggable);

    /**
	 * Drops the specified {@link Draggable} into this <code>DropTarget</code>
	 * at the specified location.
	 */
    void receive(Draggable draggable, PointView location);
}
