package com.maryanovsky.gwtutils.client.ui.mouse.gestures.click;

import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.SyntheticMouseGestureEvent;

/**
 * Represents a click event.
 * 
 * @author Maryanovsky Alexander
 */
public class ClickEvent extends SyntheticMouseGestureEvent<ClickHandler, MouseUpEvent> {

    /***/
    private static final Type<ClickHandler> a = new Type<ClickHandler>();

    /***/
    private final int b;

    /***/
    private final int c;

    /***/
    private final PointView d;

    /**
	 * Creates a new {@link ClickEvent} from the specified source
	 * {@link MouseUpEvent}, mouse button, click count and location.
	 * 
	 * @param evt The mouse up event that triggered this event.
	 * @param button The clicked mouse button.
	 * @param clickCount The number of clicks that have occurred.
	 * @param clickLocation The location of the click(s), in the target
	 * element's coordinate system.
	 */
    public ClickEvent(MouseUpEvent evt, int button, int clickCount, PointView clickLocation) {
        super(evt);
        if ((button != NativeEvent.BUTTON_LEFT) && (button != NativeEvent.BUTTON_MIDDLE) && (button != NativeEvent.BUTTON_RIGHT)) throw new IllegalArgumentException("Illegal value for mouse button: " + button);
        if (clickCount <= 0) throw new IllegalArgumentException("clickCount must be positive");
        if (clickLocation == null) throw new IllegalArgumentException("clickLocation may not be null");
        this.b = button;
        this.c = clickCount;
        this.d = clickLocation;
    }

    /**
	 * Returns the button that was clicked. Possible values are
	 * {@link NativeEvent#BUTTON_LEFT}, {@link NativeEvent#BUTTON_MIDDLE} and
	 * {@link NativeEvent#BUTTON_RIGHT}.
	 */
    public int getMouseButton() {
        return b;
    }

    /**
	 * Returns the number of clicks that have occurred.
	 */
    public int getClickCount() {
        return c;
    }

    /**
	 * Returns the location of the click(s), in the target element's coordinate
	 * system.
	 */
    public PointView getClickLocation() {
        return d;
    }

    /**
	 * Returns the event type associated with click events.
	 */
    public static Type<ClickHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with click events.
	 */
    public Type<ClickHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(ClickHandler handler) {
        handler.onClick(this);
    }
}
