package com.maryanovsky.gwtutils.client.ui.mouse.gestures.click;

import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.Condition;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.MouseGestureRecognizer;

/**
 * Recognizes and fires click events.
 * 
 * @author Maryanovsky Alexander
 */
public class ClickRecognizer extends MouseGestureRecognizer implements HasClickHandlers {

    /***/
    private static final int a = 1;

    /***/
    private static final int b = 300;

    /***/
    private static final int c = 1;

    /***/
    private static final int d = 1000;

    /***/
    private final Condition<? super ClickEvent> e;

    /***/
    private Point f = null;

    /***/
    private Point g = null;

    /***/
    private long h;

    /***/
    private boolean i = false;

    /***/
    private int j = 0;

    /***/
    private Point k = null;

    /***/
    private Point l = null;

    /***/
    private long m;

    /***/
    private int n;

    /**
	 * Creates a new {@link ClickRecognizer} which will fire click events with
	 * the specified source object.
	 */
    public ClickRecognizer(Object clickEventsSource) {
        this(clickEventsSource, (HasAllMouseHandlers) null, null);
    }

    /**
	 * Creates a new {@link ClickRecognizer} which will fire click events with
	 * the specified source object and use the specified source of mouse events.
	 */
    public <H extends HasMouseDownHandlers & HasMouseUpHandlers & HasMouseMoveHandlers & HasMouseOutHandlers & HasMouseOverHandlers & HasMouseWheelHandlers> ClickRecognizer(Object clickEventsSource, H mouseEventsSource) {
        this(clickEventsSource, mouseEventsSource, null);
    }

    /**
	 * Creates a new {@link ClickRecognizer} which will fire click events with
	 * the specified source object and consult the specified click condition
	 * before doing so.
	 */
    public ClickRecognizer(Object clickEventsSource, Condition<? super ClickEvent> clickCondition) {
        this(clickEventsSource, (HasAllMouseHandlers) null, clickCondition);
    }

    /**
	 * Creates a new {@link ClickRecognizer} which will fire events with the
	 * specified source object, use the specified source of mouse events and
	 * consult the specified click condition before firing click events. See the
	 * documentation of {@link #clickCondition(ClickEvent)} for more information
	 * on the click condition.
	 */
    public <H extends HasMouseDownHandlers & HasMouseUpHandlers & HasMouseMoveHandlers & HasMouseOutHandlers & HasMouseOverHandlers & HasMouseWheelHandlers> ClickRecognizer(Object clickEventsSource, H mouseEventsSource, Condition<? super ClickEvent> clickCondition) {
        super(clickEventsSource, mouseEventsSource);
        this.e = clickCondition;
    }

    /**
	 * Adds a click handler.
	 */
    public HandlerRegistration addClickHandler(ClickHandler handler) {
        return handlerManager.addHandler(ClickEvent.getType(), handler);
    }

    /**
	 * Returns whether the specified {@link ClickEvent} should be delivered to
	 * the handlers. This allows filtering click gestures to only those you
	 * really want. For example, if you only want to recognize right-clicks, you
	 * would override this method and return
	 * <code>evt.getButton() == NativeEvent.BUTTON_RIGHT</code>.
	 * The default implementation delegates the decision to the click condition
	 * passed in the constructor.
	 */
    protected boolean clickCondition(ClickEvent evt) {
        return e == null ? true : e.eval(evt);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onMouseDown(MouseDownEvent event) {
        if (i) return;
        f = Point.make(event.getX(), event.getY(), f);
        g = Point.make(event.getScreenX(), event.getScreenY(), g);
        h = System.currentTimeMillis();
        i = true;
        j = event.getNativeButton();
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onMouseUp(MouseUpEvent event) {
        if (o(event)) {
            n = p(event) ? n + 1 : 1;
            k = Point.make(event.getX(), event.getY(), k);
            l = Point.make(event.getScreenX(), event.getScreenY(), l);
            m = System.currentTimeMillis();
            ClickEvent q = new ClickEvent(event, j, n, k);
            if (clickCondition(q)) handlerManager.fireEvent(q);
        }
        i = false;
    }

    /***/
    private boolean o(MouseUpEvent q) {
        if (g == null) return false;
        int r = Math.abs(g.getX() - q.getScreenX());
        int s = Math.abs(g.getY() - q.getScreenY());
        long t = System.currentTimeMillis() - h;
        boolean u = (r == 0) && (s == 0);
        boolean v = (t <= b) && (r <= a) && (s <= a);
        return u || v;
    }

    /***/
    private boolean p(MouseUpEvent q) {
        if (l == null) return false;
        int r = Math.abs(l.getX() - q.getScreenX());
        int s = Math.abs(l.getY() - q.getScreenY());
        long t = System.currentTimeMillis() - m;
        return (t <= d) && (r <= c) && (s <= c);
    }
}
