package com.maryanovsky.gwtutils.client.ui.mouse.gestures.drag;

import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.SyntheticMouseGestureEvent;

/**
 * Represents a drag end event.
 * 
 * @author Maryanovsky Alexander
 */
public class DragEndEvent extends SyntheticMouseGestureEvent<DragEndHandler, MouseUpEvent> {

    /***/
    private static final Type<DragEndHandler> a = new Type<DragEndHandler>();

    /***/
    private final int b;

    /***/
    private final PointView c;

    /***/
    private final PointView d;

    /**
	 * Creates a new {@link DragEndEvent} from the specified source
	 * {@link MouseUpEvent}, mouse location and offset from the initial touch
	 * point.
	 * 
	 * @param evt The touch end event that triggered this event.
	 * @param button The pressed mouse button.
	 * @param location The location of the mouse in the target element's
	 * coordinate system.
	 * @param offset The offset of the mouse from the location where it was
	 * pressed.
	 */
    public DragEndEvent(MouseUpEvent evt, int button, PointView location, PointView offset) {
        super(evt);
        if (!UiUtils.isNativeMouseButton(button)) throw new IllegalArgumentException("Illegal value for mouse button: " + button);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        if (offset == null) throw new IllegalArgumentException("offset may not be null");
        this.b = button;
        this.c = location;
        this.d = offset;
    }

    /**
	 * Returns the button that was clicked. Possible values are
	 * {@link NativeEvent#BUTTON_LEFT}, {@link NativeEvent#BUTTON_MIDDLE} and
	 * {@link NativeEvent#BUTTON_RIGHT}.
	 */
    public int getButton() {
        return b;
    }

    /**
	 * Returns the location of the lifted finger, in the target element's
	 * coordinate system.
	 */
    public PointView getLocation() {
        return c;
    }

    /**
	 * Returns the offset of the lifted finger from the initial touching
	 * location.
	 */
    public PointView getOffset() {
        return d;
    }

    /**
	 * Returns the event type associated with drag end events.
	 */
    public static Type<DragEndHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with drag end events.
	 */
    public Type<DragEndHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(DragEndHandler handler) {
        handler.onDragEnd(this);
    }
}
