package com.maryanovsky.gwtutils.client.ui.mouse.gestures.drag;

import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseMoveEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.SyntheticMouseGestureEvent;

/**
 * Represents a mouse-drag event.
 * 
 * @author Maryanovsky Alexander
 */
public class DragMoveEvent extends SyntheticMouseGestureEvent<DragMoveHandler, MouseMoveEvent> {

    /***/
    private static final Type<DragMoveHandler> a = new Type<DragMoveHandler>();

    /***/
    private final int b;

    /***/
    private final PointView c;

    /***/
    private final PointView d;

    /**
	 * Creates a new {@link DragMoveEvent} from the specified source
	 * {@link MouseMoveEvent}, mouse button, location and offset.
	 * 
	 * @param evt The mouse up event that triggered this event.
	 * @param button The pressed mouse button.
	 * @param location The location of the mouse, in the target element's
	 * coordinate system.
	 * @param offset The offset of the mouse position from the location where it
	 * was initially pressed.
	 */
    public DragMoveEvent(MouseMoveEvent evt, int button, PointView location, PointView offset) {
        super(evt);
        if (!UiUtils.isNativeMouseButton(button)) throw new IllegalArgumentException("Illegal value for mouse button: " + button);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        if (offset == null) throw new IllegalArgumentException("offset may not be null");
        this.b = button;
        this.c = location;
        this.d = offset;
    }

    /**
	 * Returns the button that was clicked. Possible values are
	 * {@link NativeEvent#BUTTON_LEFT}, {@link NativeEvent#BUTTON_MIDDLE} and
	 * {@link NativeEvent#BUTTON_RIGHT}.
	 */
    public int getButton() {
        return b;
    }

    /**
	 * Returns the location of the mouse, in the target element's coordinate
	 * system.
	 */
    public PointView getLocation() {
        return c;
    }

    /**
	 * Returns the offset of the mouse position from the location where it
	 * was initially pressed.
	 */
    public PointView getOffset() {
        return d;
    }

    /**
	 * Returns the event type associated with mouse-drag events.
	 */
    public static Type<DragMoveHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with mouse-drag events.
	 */
    public Type<DragMoveHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(DragMoveHandler handler) {
        handler.onDragMove(this);
    }
}
