package com.maryanovsky.gwtutils.client.ui.mouse.gestures.drag;

import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.MouseDownEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.SyntheticMouseGestureEvent;

/**
 * Represents a mouse drag start event. Note that until a drag-move event occurs
 * a mouse-drag may be cancelled (such as when the user was in fact just
 * clicking).
 * 
 * @author Maryanovsky Alexander
 */
public class DragStartEvent extends SyntheticMouseGestureEvent<DragStartHandler, MouseDownEvent> {

    /***/
    private static final Type<DragStartHandler> a = new Type<DragStartHandler>();

    /***/
    private final int b;

    /***/
    private final PointView c;

    /**
	 * Creates a new {@link DragStartEvent} from the specified source
	 * {@link MouseDownEvent} and the specified mouse location.
	 * 
	 * @param evt The mouse down event that triggered this event.
	 * @param button The pressed mouse button.
	 * @param location The location of the mouse in the target element's
	 * coordinate system.
	 */
    public DragStartEvent(MouseDownEvent evt, int button, PointView location) {
        super(evt);
        if (!UiUtils.isNativeMouseButton(button)) throw new IllegalArgumentException("Illegal value for mouse button: " + button);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        this.b = button;
        this.c = location;
    }

    /**
	 * Returns the button that was clicked. Possible values are
	 * {@link NativeEvent#BUTTON_LEFT}, {@link NativeEvent#BUTTON_MIDDLE} and
	 * {@link NativeEvent#BUTTON_RIGHT}.
	 */
    public int getButton() {
        return b;
    }

    /**
	 * Returns the location of the mouse, in the target element's coordinate
	 * system.
	 */
    public PointView getLocation() {
        return c;
    }

    /**
	 * Returns the event type associated with drag start events.
	 */
    public static Type<DragStartHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with drag start events.
	 */
    public Type<DragStartHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(DragStartHandler handler) {
        handler.onDragStart(this);
    }
}
