package com.maryanovsky.gwtutils.client.ui.mouse.gestures.wheelscroll;

import com.google.gwt.event.dom.client.MouseWheelEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.SyntheticMouseGestureEvent;

/**
 * Represents a mouse wheel scroll-up event. 
 * 
 * @author Maryanovsky Alexander
 */
public class WheelScrollUpEvent extends SyntheticMouseGestureEvent<WheelScrollUpHandler, MouseWheelEvent> {

    /***/
    private final PointView a;

    /***/
    private static final Type<WheelScrollUpHandler> b = new Type<WheelScrollUpHandler>();

    /**
	 * Creates a new {@link WheelScrollUpEvent} from the source
	 * {@link MouseWheelEvent} and location of the mouse.
	 * 
	 * @param evt The {@link MouseWheelEvent} that triggered this event.
	 * @param location The location of the mouse, in the target element's
	 * coordinate system.
	 */
    public WheelScrollUpEvent(MouseWheelEvent evt, PointView location) {
        super(evt);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        this.a = location;
    }

    /**
	 * Returns the location of the mouse, in the target element's coordinate
	 * system.
	 */
    public PointView getLocation() {
        return a;
    }

    /**
	 * Returns the event type associated with mouse wheel scroll-up events.
	 */
    public static Type<WheelScrollUpHandler> getType() {
        return b;
    }

    /**
	 * Returns the event type associated with mouse wheel scroll-up events.
	 */
    public Type<WheelScrollUpHandler> getAssociatedType() {
        return b;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(WheelScrollUpHandler handler) {
        handler.onWheelScrollUp(this);
    }
}
