package com.maryanovsky.gwtutils.client.ui.touch.gestures.drag;

import com.google.gwt.event.dom.client.TouchEndEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a drag end event.
 * 
 * @author Maryanovsky Alexander
 */
public class DragEndEvent extends SyntheticTouchGestureEvent<DragEndHandler, TouchEndEvent> {

    /***/
    private static final Type<DragEndHandler> a = new Type<DragEndHandler>();

    /***/
    private final PointView b;

    /***/
    private final PointView c;

    /**
	 * Creates a new {@link DragEndEvent} from the specified source
	 * {@link TouchEndEvent}, touch location and offset from the initial touch
	 * point.
	 * 
	 * @param evt The touch end event that triggered this event.
	 * @param location The location of the finger in the target element's
	 * coordinate system.
	 * @param offset The offset of the finger from the location where it was
	 * pressed.
	 */
    public DragEndEvent(TouchEndEvent evt, PointView location, PointView offset) {
        super(evt);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        if (offset == null) throw new IllegalArgumentException("offset may not be null");
        this.b = location;
        this.c = offset;
    }

    /**
	 * Returns the location of the lifted finger, in the target element's
	 * coordinate system.
	 */
    public PointView getLocation() {
        return b;
    }

    /**
	 * Returns the offset of the lifted finger from the initial touching
	 * location.
	 */
    public PointView getOffset() {
        return c;
    }

    /**
	 * Returns the event type associated with drag end events.
	 */
    public static Type<DragEndHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with drag end events.
	 */
    public Type<DragEndHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(DragEndHandler handler) {
        handler.onDragEnd(this);
    }
}
