package com.maryanovsky.gwtutils.client.ui.touch.gestures.drag;

import com.google.gwt.event.dom.client.TouchMoveEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a drag move event.
 * 
 * @author Maryanovsky Alexander
 */
public class DragMoveEvent extends SyntheticTouchGestureEvent<DragMoveHandler, TouchMoveEvent> {

    /***/
    private static final Type<DragMoveHandler> a = new Type<DragMoveHandler>();

    /***/
    private final PointView b;

    /***/
    private final PointView c;

    /**`
	 * Creates a new {@link DragMoveEvent} from the specified source
	 * {@link TouchMoveEvent}, touch location and offset from the initial touch
	 * point.
	 * 
	 * @param evt The touch move event that triggered this event.
	 * @param location The location of the finger in the target element's
	 * coordinate system.
	 * @param offset The offset of the finger from the location where it was
	 * pressed.
	 */
    public DragMoveEvent(TouchMoveEvent evt, PointView location, PointView offset) {
        super(evt);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        if (offset == null) throw new IllegalArgumentException("offset may not be null");
        this.b = location;
        this.c = offset;
    }

    /**
	 * Returns the location of the touching finger, in the target element's
	 * coordinate system.
	 */
    public PointView getLocation() {
        return b;
    }

    /**
	 * Returns the offset of the touching finger from the initial touching
	 * location.
	 */
    public PointView getOffset() {
        return c;
    }

    /**
	 * Returns the event type associated with drag move events.
	 */
    public static Type<DragMoveHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with drag move events.
	 */
    public Type<DragMoveHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(DragMoveHandler handler) {
        handler.onDragMove(this);
    }
}
