package com.maryanovsky.gwtutils.client.ui.touch.gestures.drag;

import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.Condition;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.TouchGestureRecognizer;

/**
 * Recognizes and fires drag events.
 * 
 * @author Maryanovsky Alexander
 */
public class DragRecognizer extends TouchGestureRecognizer implements HasAllDragHandlers {

    /***/
    private final Condition<? super DragStartEvent> a;

    /***/
    private boolean b = false;

    /***/
    private boolean c = false;

    /***/
    private final Point d = new Point();

    /***/
    private boolean e = false;

    /**
	 * Creates a new {@link DragRecognizer} which will fire drag events with the
	 * specified source object.
	 */
    public DragRecognizer(Object dragEventsSource) {
        this(dragEventsSource, (HasAllTouchHandlers) null, null);
    }

    /**
	 * Creates a new {@link DragRecognizer} which will fire events with the
	 * specified source object and use the specified source of touch events.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> DragRecognizer(Object dragEventsSource, H touchEventsSource) {
        this(dragEventsSource, touchEventsSource, null);
    }

    /**
	 * Creates a new {@link DragRecognizer} which will fire drag events with
	 * the specified source object and consult the specified drag condition
	 * before doing so.
	 */
    public DragRecognizer(Object dragEventsSource, Condition<? super DragStartEvent> dragCondition) {
        this(dragEventsSource, (HasAllTouchHandlers) null, dragCondition);
    }

    /**
	 * Creates a new {@link DragRecognizer} which will fire events with the
	 * specified source object, use the specified source of touch events and
	 * consult the specified drag condition before firing the drag event(s).
	 * See the documentation of {@link #dragCondition(DragStartEvent)} for more
	 * information on the drag condition.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> DragRecognizer(Object dragEventsSource, H touchEventsSource, Condition<? super DragStartEvent> dragCondition) {
        super(dragEventsSource, touchEventsSource);
        this.a = dragCondition;
    }

    /**
	 * Adds a drag start handler. 
	 */
    public HandlerRegistration addDragStartHandler(DragStartHandler handler) {
        return handlerManager.addHandler(DragStartEvent.getType(), handler);
    }

    /**
	 * Adds a drag move handler.
	 */
    public HandlerRegistration addDragMoveHandler(DragMoveHandler handler) {
        return handlerManager.addHandler(DragMoveEvent.getType(), handler);
    }

    /**
	 * Adds a drag end handler.
	 */
    public HandlerRegistration addDragEndHandler(DragEndHandler handler) {
        return handlerManager.addHandler(DragEndEvent.getType(), handler);
    }

    /**
	 * Adds a drag cancel handler.
	 */
    public HandlerRegistration addDragCancelHandler(DragCancelHandler handler) {
        return handlerManager.addHandler(DragCancelEvent.getType(), handler);
    }

    /**
	 * Returns whether the specified {@link DragStartEvent} and the drag events
	 * that follow it in the drag gesture should be delivered to the handlers.
	 * This allows filtering drag gestures to only those you really want.
	 * The default implementation delegates the decision to the drag condition
	 * passed in the constructor. 
	 */
    protected boolean dragCondition(DragStartEvent evt) {
        return a == null ? true : a.eval(evt);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchStart(TouchStartEvent evt) {
        if (evt.getTouches().length() == 1) {
            b = true;
            c = false;
            d.set(UiUtils.getCurrentTargetCoordinates(evt.getTouches().get(0), evt));
            DragStartEvent f = new DragStartEvent(evt, d);
            e = dragCondition(f);
            if (e) fireEvent(f);
        } else if (b) {
            b = false;
            if (e) fireEvent(new DragCancelEvent(evt));
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchMove(TouchMoveEvent evt) {
        if (b) {
            c = true;
            PointView f = UiUtils.getCurrentTargetCoordinates(evt.getTouches().get(0), evt);
            PointView g = d.getDistancePoint(f);
            if (e) fireEvent(new DragMoveEvent(evt, f, g));
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchEnd(TouchEndEvent evt) {
        if (b) {
            if (c) {
                PointView f = UiUtils.getCurrentTargetCoordinates(evt.getChangedTouches().get(0), evt);
                PointView g = d.getDistancePoint(f);
                if (e) fireEvent(new DragEndEvent(evt, f, g));
                c = false;
            } else {
                if (e) fireEvent(new DragCancelEvent(evt));
            }
            b = false;
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchCancel(TouchCancelEvent evt) {
        if (b) {
            b = false;
            if (e) fireEvent(new DragCancelEvent(evt));
        }
    }
}
