package com.maryanovsky.gwtutils.client.ui.touch.gestures.drag;

import com.google.gwt.event.dom.client.TouchStartEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a drag start event.
 * 
 * @author Maryanovsky Alexander
 */
public class DragStartEvent extends SyntheticTouchGestureEvent<DragStartHandler, TouchStartEvent> {

    /***/
    private static final Type<DragStartHandler> a = new Type<DragStartHandler>();

    /***/
    private final PointView b;

    /**
	 * Creates a new {@link DragStartEvent} from the specified source
	 * {@link TouchStartEvent} and the specified touch location.
	 * 
	 * @param evt The touch start event that triggered this event.
	 * @param location The location of the finger in the target element's
	 * coordinate system.
	 */
    public DragStartEvent(TouchStartEvent evt, PointView location) {
        super(evt);
        if (location == null) throw new IllegalArgumentException("location may not be null");
        this.b = location;
    }

    /**
	 * Returns the location of the touch that caused the drag gesture to start,
	 * in the target element's coordinate system.
	 */
    public PointView getLocation() {
        return b;
    }

    /**
	 * Returns the event type associated with drag start events.
	 */
    public static Type<DragStartHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with drag start events.
	 */
    public Type<DragStartHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(DragStartHandler handler) {
        handler.onDragStart(this);
    }
}
