package com.maryanovsky.gwtutils.client.ui.touch.gestures.mftap;

import java.util.Collections;
import java.util.List;
import com.google.gwt.event.dom.client.TouchEndEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a multi-finger tap event.
 * 
 * @author Maryanovsky Alexander
 */
public class MultiFingerTapEvent extends SyntheticTouchGestureEvent<MultiFingerTapHandler, TouchEndEvent> {

    /***/
    private static final Type<MultiFingerTapHandler> a = new Type<MultiFingerTapHandler>();

    /***/
    private final List<PointView> b;

    /**
	 * Creates a new {@link MultiFingerTapEvent} from the specified source
	 * {@link TouchEndEvent} and finger locations.
	 * 
	 * @param evt The touch end event that triggered this event.
	 * @param fingerLocations The locations of the fingers, in the target
	 * element's coordinate system.
	 */
    public MultiFingerTapEvent(TouchEndEvent evt, List<PointView> fingerLocations) {
        super(evt);
        this.b = fingerLocations;
    }

    /**
	 * Returns the number of fingers involved in the tap.
	 */
    public int getFingerCount() {
        return b.size();
    }

    /**
	 * Returns the location of the finger(s), in the target element's coordinate
	 * system. The returned value is an immutable list.
	 */
    public List<PointView> getFingerLocations() {
        return Collections.unmodifiableList(b);
    }

    /**
	 * Returns the event type associated with multi-finger tap events.
	 */
    public static Type<MultiFingerTapHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with multi-finger events.
	 */
    public Type<MultiFingerTapHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(MultiFingerTapHandler handler) {
        handler.onMultiFingerTap(this);
    }
}
