package com.maryanovsky.gwtutils.client.ui.touch.gestures.mftap;

import java.util.ArrayList;
import java.util.List;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.dom.client.Touch;
import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.Condition;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.TouchGestureRecognizer;

/**
 * Recognizes and fires multi-finger, single-tap events.
 * 
 * @author Maryanovsky Alexander
 */
public class MultiFingerTapRecognizer extends TouchGestureRecognizer implements HasMultiFingerTapHandlers {

    /***/
    private static final int a = 1000;

    /***/
    private final Condition<? super MultiFingerTapEvent> b;

    /***/
    private boolean c = true;

    /***/
    private boolean d = false;

    /***/
    private final List<Point> e = new ArrayList<Point>();

    /***/
    private long f;

    /***/
    private int g = 0;

    /**
	 * Creates a new {@link MultiFingerTapRecognizer} which will fire
	 * multi-finger tap events with the specified source object.
	 */
    public MultiFingerTapRecognizer(Object multiFingerTapEventSource) {
        this(multiFingerTapEventSource, (HasAllTouchHandlers) null, null);
    }

    /**
	 * Creates a new {@link MultiFingerTapRecognizer} which will fire
	 * multi-finger tap events with the specified source object and use the
	 * specified source of touch events.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> MultiFingerTapRecognizer(Object multiFingerTapEventSource, H touchEventsSource) {
        this(multiFingerTapEventSource, touchEventsSource, null);
    }

    /**
	 * Creates a new {@link MultiFingerTapRecognizer} which will fire
	 * multi-finger tap events with the specified source object and consult the
	 * specified tap condition before doing so.
	 */
    public MultiFingerTapRecognizer(Object multiFingerTapEventSource, Condition<? super MultiFingerTapEvent> multiFingerTapCondition) {
        this(multiFingerTapEventSource, (HasAllTouchHandlers) null, multiFingerTapCondition);
    }

    /**
	 * Creates a new {@link MultiFingerTapRecognizer} which will fire events
	 * with the specified source object, use the specified source of touch
	 * events and consult the specified tap condition before firing the events.
	 * See the documentation of
	 * {@link #multiFingerTapCondition(MultiFingerTapEvent)} for more
	 * information on the tap condition.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> MultiFingerTapRecognizer(Object multiFingerTapEventSource, H touchEventsSource, Condition<? super MultiFingerTapEvent> multiFingerTapCondition) {
        super(multiFingerTapEventSource, touchEventsSource);
        this.b = multiFingerTapCondition;
    }

    /**
	 * Adds a multi-finger tap handler.
	 */
    public HandlerRegistration addMultiFingerTapHandler(MultiFingerTapHandler handler) {
        return handlerManager.addHandler(MultiFingerTapEvent.getType(), handler);
    }

    /**
	 * Returns whether the specified {@link MultiFingerTapEvent} should be
	 * delivered to the handlers. This allows filtering multi-finger tap
	 * gestures to only those you really want. For example, if you only want to
	 * recognize two-finger taps, you would override this method and return
	 * <code>evt.getFingerCount() == 2</code>.
	 * The default implementation delegates the decision to the tap condition
	 * passed in the constructor. 
	 */
    protected boolean multiFingerTapCondition(MultiFingerTapEvent evt) {
        return b == null ? true : b.eval(evt);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchStart(TouchStartEvent evt) {
        JsArray<Touch> h = evt.getTargetTouches();
        long i = System.currentTimeMillis();
        if (d) {
            d = false;
            c = false;
        }
        for (int j = 0; j < h.length(); j++) {
            PointView k = UiUtils.getCurrentTargetCoordinates(h.get(j), evt);
            if (j >= e.size()) e.add(new Point(k)); else e.get(j).set(k);
        }
        f = i;
        g = 0;
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchMove(TouchMoveEvent evt) {
        g++;
        if (g > 4 * (e.size() - 1)) {
            c = false;
            d = false;
            e.clear();
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchEnd(TouchEndEvent evt) {
        if (c) {
            c = false;
            d = true;
        }
        JsArray<Touch> h = evt.getTargetTouches();
        if (d && (h.length() == 0)) {
            if (System.currentTimeMillis() - f < a) {
                MultiFingerTapEvent i = new MultiFingerTapEvent(evt, new ArrayList<PointView>(e));
                if (multiFingerTapCondition(i)) fireEvent(i);
            }
            e.clear();
        }
        if (h.length() == 0) {
            d = false;
            c = true;
        }
        g = 0;
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchCancel(TouchCancelEvent evt) {
        e.clear();
        c = true;
        d = false;
    }
}
