package com.maryanovsky.gwtutils.client.ui.touch.gestures.pinch;

import com.google.gwt.event.dom.client.TouchMoveEvent;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a pinch change event.
 * 
 * @author Maryanovsky Alexander
 */
public class PinchChangeEvent extends SyntheticTouchGestureEvent<PinchChangeHandler, TouchMoveEvent> {

    /***/
    private static final Type<PinchChangeHandler> a = new Type<PinchChangeHandler>();

    /***/
    private final double b;

    /**
	 * Creates a new {@link PinchChangeEvent} from the specified source
	 * {@link TouchMoveEvent} and scale.
	 * 
	 * @param evt The touch move event that triggered this event.
	 * @param scale The factor by which the user changed the distance between
	 * his fingers. Values greater than 1 imply he moved his fingers further
	 * apart; values smaller than 1, closer together.
	 */
    public PinchChangeEvent(TouchMoveEvent evt, double scale) {
        super(evt);
        if (scale <= 0) throw new IllegalArgumentException("scale must be positive");
        this.b = scale;
    }

    /**
	 * Returns the factor by which the user changed the distance between
	 * his fingers. Values greater than 1 imply he moved his fingers further
	 * apart; values smaller than 1, closer together.
	 */
    public double getScale() {
        return b;
    }

    /**
	 * Returns the event type associated with pinch change events.
	 */
    public static Type<PinchChangeHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with pinch change events.
	 */
    public Type<PinchChangeHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(PinchChangeHandler handler) {
        handler.onPinchChange(this);
    }
}
