package com.maryanovsky.gwtutils.client.ui.touch.gestures.pinch;

import com.google.gwt.event.dom.client.TouchEndEvent;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a pinch end event.
 * 
 * @author Maryanovsky Alexander
 */
public class PinchEndEvent extends SyntheticTouchGestureEvent<PinchEndHandler, TouchEndEvent> {

    /***/
    private static final Type<PinchEndHandler> a = new Type<PinchEndHandler>();

    /***/
    private final double b;

    /**
	 * Creates a new {@link PinchEndEvent} from the specified source
	 * {@link TouchEndEvent} and scale.
	 * 
	 * @param evt The touch end event that triggered this event.
	 * @param scale The factor by which the user changed the distance between
	 * his fingers. Values greater than 1 imply he moved his fingers further
	 * apart; values smaller than 1, closer together.
	 */
    public PinchEndEvent(TouchEndEvent evt, double scale) {
        super(evt);
        if (scale <= 0) throw new IllegalArgumentException("scale must be positive");
        this.b = scale;
    }

    /**
	 * Returns the factor by which the user changed the distance between
	 * his fingers. Values greater than 1 imply he moved his fingers further
	 * apart; values smaller than 1, closer together.
	 */
    public double getScale() {
        return b;
    }

    /**
	 * Returns the event type associated with pinch end events.
	 */
    public static Type<PinchEndHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with pinch end events.
	 */
    public Type<PinchEndHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(PinchEndHandler handler) {
        handler.onPinchEnd(this);
    }
}
