package com.maryanovsky.gwtutils.client.ui.touch.gestures.pinch;

import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.Condition;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.TouchGestureRecognizer;

/**
 * Recognizes and fires pinch events.
 * 
 * @author Maryanovsky Alexander
 */
public class PinchRecognizer extends TouchGestureRecognizer implements HasAllPinchHandlers {

    /***/
    private final Condition<? super PinchStartEvent> a;

    /***/
    private boolean b = false;

    /***/
    private boolean c = false;

    /***/
    private double d;

    /***/
    private double e;

    /***/
    private boolean f = false;

    /**
	 * Creates a new {@link PinchRecognizer} which will fire pinch events with
	 * the specified source object.
	 */
    public PinchRecognizer(Object eventSource) {
        this(eventSource, (HasAllTouchHandlers) null, null);
    }

    /**
	 * Creates a new {@link PinchRecognizer} which will fire pinch events with
	 * the specified source object and use the specified source of touch events.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> PinchRecognizer(Object eventSource, H touchEventsSource) {
        this(eventSource, touchEventsSource, null);
    }

    /**
	 * Creates a new {@link PinchRecognizer} which will fire pinch events with
	 * the specified source object and consult the specified pinch condition
	 * before doing so.
	 */
    public PinchRecognizer(Object pinchEventsSource, Condition<? super PinchStartEvent> pinchCondition) {
        this(pinchEventsSource, (HasAllTouchHandlers) null, pinchCondition);
    }

    /**
	 * Creates a new {@link PinchRecognizer} which will fire events with the
	 * specified source object, use the specified source of touch events and
	 * consult the specified pinch condition before firing the pinch event(s).
	 * See the documentation of {@link #pinchCondition(PinchStartEvent)} for
	 * more information on the pinch condition.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> PinchRecognizer(Object dragEventsSource, H touchEventsSource, Condition<? super PinchStartEvent> pinchCondition) {
        super(dragEventsSource, touchEventsSource);
        this.a = pinchCondition;
    }

    /**
	 * Adds a pinch start handler.
	 */
    public HandlerRegistration addPinchStartHandler(PinchStartHandler handler) {
        return handlerManager.addHandler(PinchStartEvent.getType(), handler);
    }

    /**
	 * Adds a pinch change handler.
	 */
    public HandlerRegistration addPinchChangeHandler(PinchChangeHandler handler) {
        return handlerManager.addHandler(PinchChangeEvent.getType(), handler);
    }

    /**
	 * Adds a pinch end handler.
	 */
    public HandlerRegistration addPinchEndHandler(PinchEndHandler handler) {
        return handlerManager.addHandler(PinchEndEvent.getType(), handler);
    }

    /**
	 * Adds a pinch cancel handler.
	 */
    public HandlerRegistration addPinchCancelHandler(PinchCancelHandler handler) {
        return handlerManager.addHandler(PinchCancelEvent.getType(), handler);
    }

    /**
	 * Returns whether the specified {@link PinchStartEvent} and the pinch
	 * events that follow it in the pinch gesture should be delivered to the
	 * handlers. This allows filtering pinch gestures to only those you really
	 * want.
	 * The default implementation delegates the decision to the pinch condition
	 * passed in the constructor. 
	 */
    protected boolean pinchCondition(PinchStartEvent evt) {
        return a == null ? true : a.eval(evt);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchStart(TouchStartEvent evt) {
        int g = evt.getTouches().length();
        if (g == 2) {
            b = true;
            c = false;
            PointView h = UiUtils.getCurrentTargetCoordinates(evt.getTouches().get(0), evt);
            PointView i = UiUtils.getCurrentTargetCoordinates(evt.getTouches().get(1), evt);
            d = h.getDistanceFrom(i);
            PinchStartEvent j = new PinchStartEvent(evt, h, i);
            f = pinchCondition(j);
            if (f) fireEvent(j);
        } else if (g > 2) {
            if (f) fireEvent(new PinchCancelEvent(evt));
            b = false;
            c = false;
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchMove(TouchMoveEvent evt) {
        if (b) {
            c = true;
            PointView g = UiUtils.getCurrentTargetCoordinates(evt.getTouches().get(0), evt);
            PointView h = UiUtils.getCurrentTargetCoordinates(evt.getTouches().get(1), evt);
            e = g.getDistanceFrom(h);
            if (f) fireEvent(new PinchChangeEvent(evt, e / d));
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchEnd(TouchEndEvent evt) {
        if (b) {
            if (c) {
                if (f) fireEvent(new PinchEndEvent(evt, e / d));
            } else {
                if (f) fireEvent(new PinchCancelEvent(evt));
            }
            c = false;
            b = false;
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchCancel(TouchCancelEvent evt) {
        if (b) {
            if (f) fireEvent(new PinchCancelEvent(evt));
            b = false;
            c = false;
        }
    }
}
