package com.maryanovsky.gwtutils.client.ui.touch.gestures.pinch;

import com.google.gwt.event.dom.client.TouchStartEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a pinch start event.
 * 
 * @author Maryanovsky Alexander
 */
public class PinchStartEvent extends SyntheticTouchGestureEvent<PinchStartHandler, TouchStartEvent> {

    /***/
    private static final Type<PinchStartHandler> a = new Type<PinchStartHandler>();

    /***/
    private final PointView b;

    /***/
    private final PointView c;

    /**
	 * Creates a new {@link PinchStartEvent} from the specified source
	 * {@link TouchStartEvent} and the specified finger locations.
	 * 
	 * @param evt The touch start event that triggered this event.
	 * @param firstFingerLocation The location of the first finger, in the
	 * target element's coordinate system.
	 * @param secondFingerLocation The location of the second finger, in the target element's
	 * coordinate system.
	 */
    public PinchStartEvent(TouchStartEvent evt, PointView firstFingerLocation, PointView secondFingerLocation) {
        super(evt);
        if (firstFingerLocation == null) throw new IllegalArgumentException("firstFingerLocation may not be null");
        if (secondFingerLocation == null) throw new IllegalArgumentException("secondFingerLocation may not be null");
        this.b = firstFingerLocation;
        this.c = secondFingerLocation;
    }

    /**
	 * Returns the location of the first finger, in the target element's
	 * coordinate system.
	 */
    public PointView getFirstFingerLocation() {
        return b;
    }

    /**
	 * Returns the location of the second finger, in the target element's
	 * coordinate system.
	 */
    public PointView getSecondFingerLocation() {
        return c;
    }

    /**
	 * Returns the event type associated with pinch start events.
	 */
    public static Type<PinchStartHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with pinch start events.
	 */
    public Type<PinchStartHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(PinchStartHandler handler) {
        handler.onPinchStart(this);
    }
}
