package com.maryanovsky.gwtutils.client.ui.touch.gestures.tap;

import com.google.gwt.event.dom.client.TouchEndEvent;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.SyntheticTouchGestureEvent;

/**
 * Represents a tap event.
 * 
 * @author Maryanovsky Alexander
 */
public class TapEvent extends SyntheticTouchGestureEvent<TapHandler, TouchEndEvent> {

    /***/
    private static final Type<TapHandler> a = new Type<TapHandler>();

    /***/
    private final int b;

    /***/
    private final PointView c;

    /**
	 * Creates a new {@link TapEvent} from the specified source
	 * {@link TouchEndEvent}, tap count and tap location.
	 * 
	 * @param evt The touch end event that triggered this event.
	 * @param tapCount The number of taps that have occurred.
	 * @param tapLocation The location of the tap(s), in the target element's
	 * coordinate system.
	 */
    public TapEvent(TouchEndEvent evt, int tapCount, PointView tapLocation) {
        super(evt);
        if (tapCount <= 0) throw new IllegalArgumentException("tapCount must be positive");
        if (tapLocation == null) throw new IllegalArgumentException("tapLocation may not be null");
        this.b = tapCount;
        this.c = tapLocation;
    }

    /**
	 * Returns the number of taps that have occurred.
	 */
    public int getTapCount() {
        return b;
    }

    /**
	 * Returns the location of the tap(s), in the target element's coordinate
	 * system.
	 */
    public PointView getTapLocation() {
        return c;
    }

    /**
	 * Returns the event type associated with tap events.
	 */
    public static Type<TapHandler> getType() {
        return a;
    }

    /**
	 * Returns the event type associated with tap events.
	 */
    public Type<TapHandler> getAssociatedType() {
        return a;
    }

    /**
	 * Dispatches the event to the handler.
	 */
    protected void dispatch(TapHandler handler) {
        handler.onTap(this);
    }
}
