package com.maryanovsky.gwtutils.client.ui.touch.gestures.tap;

import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.Condition;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.TouchGestureRecognizer;

/**
 * Recognizes and fires single-finger, multi-tap events.
 * 
 * @author Maryanovsky Alexander
 */
public class TapRecognizer extends TouchGestureRecognizer implements HasTapHandlers {

    /***/
    private static final int a = 1000;

    /***/
    private static final int b = 500;

    /***/
    private static final int c = 20;

    /***/
    private final Condition<? super TapEvent> d;

    /***/
    private boolean e = false;

    /***/
    private int f = 0;

    /***/
    private final Point g = new Point();

    /***/
    private long h;

    /***/
    private long i;

    /**
	 * Creates a new {@link TapRecognizer} which will fire events with the
	 * specified source object.
	 */
    public TapRecognizer(Object tapEventsSource) {
        this(tapEventsSource, (HasAllTouchHandlers) null, null);
    }

    /**
	 * Creates a new {@link TapRecognizer} which will fire events with the
	 * specified source object and use the specified source of touch events.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> TapRecognizer(Object tapEventsSource, H touchEventsSource) {
        this(tapEventsSource, touchEventsSource, null);
    }

    /**
	 * Creates a new {@link TapRecognizer} which will fire events with the
	 * specified source object and consult the specified tap condition before
	 * firing tap events. 
	 */
    public TapRecognizer(Object tapEventsSource, Condition<? super TapEvent> tapCondition) {
        this(tapEventsSource, (HasAllTouchHandlers) null, tapCondition);
    }

    /**
	 * Creates a new {@link TapRecognizer} which will fire events with the
	 * specified source object, use the specified source of touch events and
	 * consult the specified tap condition before firing tap events.
	 * See the documentation of {@link #tapCondition(TapEvent)} for more
	 * information on the tap condition.
	 */
    public <H extends HasTouchStartHandlers & HasTouchMoveHandlers & HasTouchEndHandlers & HasTouchCancelHandlers> TapRecognizer(Object tapEventsSource, H touchEventsSource, Condition<? super TapEvent> tapCondition) {
        super(tapEventsSource, touchEventsSource);
        this.d = tapCondition;
    }

    /**
	 * Adds a tap handler.
	 */
    public HandlerRegistration addTapHandler(TapHandler handler) {
        return handlerManager.addHandler(TapEvent.getType(), handler);
    }

    /**
	 * Returns whether the specified {@link TapEvent} should be delivered to the
	 * handlers. This allows filtering tap gestures to only those you really
	 * want. For example, if you only want to recognize double taps, you would
	 * override this method and return <code>evt.getTapCount() == 2</code>.
	 * The default implementation delegates the decision to the tap condition
	 * passed in the constructor. 
	 */
    protected boolean tapCondition(TapEvent evt) {
        return d == null ? true : d.eval(evt);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchStart(TouchStartEvent evt) {
        e = (evt.getTargetTouches().length() == 1);
        if (e) {
            h = System.currentTimeMillis();
            PointView j = UiUtils.getCurrentTargetCoordinates(evt.getTargetTouches().get(0), evt);
            int k = j.getX() - g.getX();
            int l = j.getY() - g.getY();
            long m = h - i;
            if ((k * k + l * l < c * c) && (m < b)) f++; else {
                f = 1;
                g.set(j);
            }
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchMove(TouchMoveEvent evt) {
        e = false;
        f = 0;
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchEnd(TouchEndEvent evt) {
        if (e) {
            e = false;
            i = System.currentTimeMillis();
            long j = i - h;
            if (j < a) {
                TapEvent k = new TapEvent(evt, f, g);
                if (tapCondition(k)) fireEvent(k);
            } else f = 0;
        }
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    protected void onTouchCancel(TouchCancelEvent evt) {
        e = false;
    }
}
