package com.maryanovsky.map.client;

import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;

/**
 * Various map related utilities.
 * 
 * @author Maryanovsky Alexander
 */
final class MapUtils {

    /**
	 * Returns the new center point when magnifying by the specified value from
	 * the specified <code>center</code>, while keeping the specified point
	 * fixed on the screen.
	 */
    public static Point getTargetPointForFixPointMagnification(PointView center, PointView fixed, double magnification) {
        return a(fixed, center, 1 / magnification);
    }

    /**
	 * Returns the point which will be kept unmoved when magnifying by the
	 * specified value and recentering from to the specified <code>center</code>
	 * to the specified <code>target</code>.
	 */
    public static Point getFixedPointForTargetPointMagnification(PointView center, PointView target, double magnification) {
        if (magnification == 1) return new Point(center); else return a(center, target, magnification / (magnification - 1));
    }

    /***/
    private static Point a(PointView b, PointView c, double d) {
        Point e = new Point(b.getDistancePoint(c));
        e.multiplyBy(d);
        return new Point(b.getX() + e.getX(), b.getY() + e.getY());
    }
}
