package com.maryanovsky.map.client;

import java.util.*;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.event.dom.client.HasAllMouseHandlers;
import com.google.gwt.event.dom.client.HasAllTouchHandlers;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.BrowserUtils;
import com.maryanovsky.gwtutils.client.MathUtils;
import com.maryanovsky.gwtutils.client.Pair;
import com.maryanovsky.gwtutils.client.event.*;
import com.maryanovsky.gwtutils.client.event.ChangeListener;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.gwtutils.client.geom.SizeView;
import com.maryanovsky.gwtutils.client.ui.Animation;
import com.maryanovsky.gwtutils.client.ui.EventCancellingPanel;
import com.maryanovsky.gwtutils.client.ui.PilePanel;
import com.maryanovsky.gwtutils.client.ui.UiUtils;

/**
 * A widget displaying a map and possibly other elements on it, and allows
 * interaction with them. The style name of this widget is
 * <code>"map-MapWidget"</code>.
 * 
 * @author Maryanovsky Alexander
 */
public class MapWidget extends Composite {

    /**
	 * The property name for the map this widget is displaying.
	 */
    public static final String MAP_PROPERTY_NAME = "map";

    /**
	 * The property name for the size of the map widget.
	 */
    public static final String SIZE_PROPERTY_NAME = "size";

    /**
	 * The property name for the temporary offset of the map widget.
	 */
    public static final String TEMPORARY_OFFSET_PROPERTY_NAME = "temporaryOffset";

    /**
	 * The property name for magnification of the map widget.
	 */
    public static final String MAGNIFICATION_PROPERTY_NAME = "magnification";

    /***/
    private final PropertyChangeListenerList a;

    /***/
    private final MapLocationModel b;

    /***/
    private final AbsolutePanel c = new AbsolutePanel();

    /***/
    private final AbsolutePanel d = new AbsolutePanel();

    /***/
    private final MagnifiedTileLayersPanelsCleaner e = new MagnifiedTileLayersPanelsCleaner();

    /***/
    private TileLayersPanel f;

    /***/
    private PilePanel g = new PilePanel();

    /***/
    private final List<TileLayersPanel> h = new LinkedList<TileLayersPanel>();

    /***/
    private final InteractionPanel i;

    /***/
    private HandlerRegistration j = null;

    /***/
    private final Size k = new Size(0, 0);

    /***/
    private Map l = null;

    /***/
    private final List<Overlay> m = new ArrayList<Overlay>();

    /***/
    private final Point n = new Point(0, 0);

    /***/
    private double o = 1.0;

    /***/
    private final Point p = new Point();

    /***/
    private final Point q = new Point();

    /***/
    private boolean r = false;

    /***/
    private double s = 1.3;

    /***/
    private boolean t = !(BrowserUtils.isMobileAppleWebKit() || BrowserUtils.isAndroid());

    /***/
    private MapAnimation u = null;

    /***/
    private int v = 300;

    /***/
    private int w = 300;

    /***/
    private final List<ReadyListener<MapWidget>> x = new LinkedList<ReadyListener<MapWidget>>();

    /**
	 * Creates a new <code>MapWidget</code> with the specified location model.
	 */
    public MapWidget(MapLocationModel mapLocationModel) {
        if (mapLocationModel == null) throw new IllegalArgumentException("mapLocationModel may not be null");
        this.a = new PropertyChangeListenerList(new String[] { MAP_PROPERTY_NAME, SIZE_PROPERTY_NAME, TEMPORARY_OFFSET_PROPERTY_NAME, MAGNIFICATION_PROPERTY_NAME });
        this.b = mapLocationModel;
        this.f = y();
        this.i = new InteractionPanel();
        initWidget(z());
        DOM.setStyleAttribute(getElement(), "cursor", "default");
        DOM.setStyleAttribute(getElement(), "backgroundColor", "white");
        DOM.setStyleAttribute(getElement(), "position", "relative");
        mapLocationModel.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent evt) {
                ag();
            }
        });
        ag();
        setStyleName("map-MapWidget");
    }

    /***/
    private TileLayersPanel y() {
        TileLayersPanel aj = new TileLayersPanel(new ReadyListener<TileLayersPanel>() {

            public void becameReady(TileLayersPanel source) {
                e.becameReady(source);
                if (source == f) aa();
            }

            public void becameUnready(TileLayersPanel source) {
                e.becameUnready(source);
                if (source == f) ab();
            }
        });
        aj.setTileCoverSize(getTileCoverSize());
        aj.setLoadTilesDuringTemporaryChanges(isLoadTilesDuringTemporaryChanges());
        return aj;
    }

    /***/
    private Widget z() {
        Panel aj = new PilePanel();
        UiUtils.addFullSize(g, f);
        aj.add(g);
        aj.add(c);
        UiUtils.setZOrder(f, c);
        UiUtils.addFullSize(c, d);
        DOM.setStyleAttribute(d.getElement(), "zIndex", String.valueOf(Integer.MIN_VALUE));
        UiUtils.addFullSize(i, aj);
        int ak = Event.MOUSEEVENTS | Event.ONMOUSEWHEEL | Event.ONCONTEXTMENU | Event.TOUCHEVENTS;
        EventCancellingPanel al = new EventCancellingPanel(null, ak, ak);
        UiUtils.addFullSize(al, i);
        return al;
    }

    /**
	 * Adds the specified <code>PropertyChangeListener</code> to be notified
	 * when the property with the specified name changes. Currently supported
	 * properties are:
	 * <ul>
	 *   <li>{@linkplain #MAP_PROPERTY_NAME}</li>
	 *   <li>{@linkplain #SIZE_PROPERTY_NAME}</li>
	 *   <li>{@linkplain #TEMPORARY_OFFSET_PROPERTY_NAME}</li>
	 *   <li>{@linkplain #MAGNIFICATION_PROPERTY_NAME}</li>
	 * </ul>
	 */
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        a.add(propertyName, listener);
    }

    /**
	 * Removes the specified <code>PropertyChangeListener</code> from being
	 * notified when the property with the specified name changes.
	 */
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        a.remove(propertyName, listener);
    }

    /**
	 * Adds the specified <code>ReadyListener</code> to be notified when the map
	 * widget becomes ready or unready. The map widget is considered ready when
	 * all of the tiles needed to display the current location at the current
	 * size have finished loading.
	 */
    public void addReadyListener(ReadyListener<MapWidget> listener) {
        x.add(listener);
    }

    /**
	 * Removes the specified ReadyListener from being notified when the ready
	 * state of the map widget changes. 
	 */
    public void removeReadyListener(ReadyListener<MapWidget> listener) {
        x.remove(listener);
    }

    /***/
    private void aa() {
        for (ReadyListener<MapWidget> aj : x) aj.becameReady(this);
    }

    /***/
    private void ab() {
        for (ReadyListener<MapWidget> aj : x) aj.becameUnready(this);
    }

    /**
	 * Sets the currently displayed map.
	 */
    public void setMap(Map map) {
        if (map == null) throw new IllegalArgumentException("map may not be null");
        if ((map.getMinZoom() > b.getMinZoom()) || (map.getMaxZoom() < b.getMaxZoom())) throw new IllegalArgumentException("The specified map does not support the entire zoom range of the MapLocationModel");
        Map aj = this.l;
        Map ak = map;
        this.l = map;
        af();
        a.firePropertyChanged(this, MAP_PROPERTY_NAME, aj, ak);
    }

    /**
	 * Returns the currently displayed map.
	 */
    public Map getMap() {
        return l;
    }

    /**
	 * Returns the location model of this <code>MapWidget</code>.
	 */
    public MapLocationModel getMapLocationModel() {
        return b;
    }

    /**
	 * Returns the widget with which event listeners should be registered in
	 * order to receive user interaction events from the map widget. Note that
	 * usually you should be using the event sources returned by
	 * {@link #getMouseEventsSource()} and/or {@link #getTouchEventsSource()}
	 * to register listeners. This method exposes the widget to allow
	 * registering for events not (yet) supported explicitly.
	 */
    public Widget getUserEventWidget() {
        return i;
    }

    /**
	 * Returns the source of mouse events for the map widget. The coordinates of
	 * the fired mouse events are in this map widget's local coordinate system.
	 */
    public HasAllMouseHandlers getMouseEventsSource() {
        return i;
    }

    /**
	 * Returns the source of touch events for the map widget. The coordinates of
	 * the fired touch events are in this map widget's local coordinate system.
	 */
    public HasAllTouchHandlers getTouchEventsSource() {
        return i;
    }

    /**
	 * Returns the panel into which widgets can be added over the map. Note that
	 * the widget panel is not guaranteed to be initially clean, and may contain
	 * widgets without which the <code>MapWidget</code> will not function
	 * properly. Basically, do not touch anything in it which you have not added
	 * yourself. 
	 */
    public AbsolutePanel getWidgetPanel() {
        return c;
    }

    /**
	 * Adds an overlay to this <code>MapWidget</code>. Overlays, unlike widgets,
	 * are (logically) positioned relative to the map, and they normally move
	 * and change when the map moves or changes zooms.
	 */
    public void addOverlay(Overlay overlay) {
        m.add(overlay);
        overlay.added(this);
    }

    /**
	 * A convenience method for adding a collection of overlays. See
	 * {@linkplain #addOverlay(Overlay)} for more information.
	 */
    public void addOverlays(Collection<? extends Overlay> overlays) {
        for (Overlay aj : overlays) addOverlay(aj);
    }

    /**
	 * Removes an overlay from this <code>MapWidget</code>.
	 */
    public void removeOverlay(Overlay overlay) {
        if (m.remove(overlay)) overlay.removed(this);
    }

    /**
	 * A convenience method for removing a collection of overlays. See
	 * {@linkplain #removeOverlay(Overlay)} for more information. 
	 */
    public void removeOverlays(Collection<? extends Overlay> overlays) {
        for (Overlay aj : overlays) removeOverlay(aj);
    }

    /**
	 * Returns all the overlays. The returned collection is unmodifiable.
	 */
    public Collection<Overlay> getOverlays() {
        return Collections.unmodifiableCollection(m);
    }

    /**
	 * Returns the panel into which overlays should be added.
	 */
    public AbsolutePanel getOverlayPanel() {
        return d;
    }

    /**
	 * Returns the center of the displayed map. Note that this may differ from
	 * the {@linkplain #getMapLocationModel()}'s center due to
	 * {@linkplain #getTemporaryOffset()}.
	 */
    public LatLng getCenter() {
        SizeView aj = getMapSize();
        return localToMap(new Point(aj.getWidth() / 2, aj.getHeight() / 2));
    }

    /**
	 * Returns the size of the displayed map, in pixels.
	 */
    public SizeView getMapSize() {
        return k;
    }

    /**
	 * Returns the bounds of the currently displayed area, in geographical
	 * coordinates.
	 */
    public LatLngBounds getBounds() {
        SizeView aj = getMapSize();
        PointView ak = mapToLocal(new LatLng(0, -90));
        PointView al = mapToLocal(new LatLng(0, 90));
        int am = 2 * Math.abs(al.getX() - ak.getX());
        LatLng an = getCenter();
        LatLng ao = localToMap(new Point(0, aj.getHeight()));
        LatLng ap = localToMap(new Point(aj.getWidth(), 0));
        if (aj.getWidth() > am) {
            ao = new LatLng(ao.getLatitude(), -180);
            ap = new LatLng(ap.getLatitude(), 180);
            return new LatLngBounds(ao, ap);
        } else return LatLngBounds.create(ao, ap, an);
    }

    /**
	 * Sets whether the map widget will use the grab cursor. Note that this may
	 * not be supported under all browsers. See
	 * {@link UiUtils#setGrabCursor(Widget)} and
	 * {@link UiUtils#setGrabbingCursor(Widget)} for more information.
	 */
    public void setGrabCursor(boolean isGrabCursor) {
        this.r = isGrabCursor;
        if (isGrabCursor) UiUtils.setGrabCursor(this); else DOM.setStyleAttribute(getElement(), "cursor", "default");
    }

    /**
	 * Returns whether the map widget will have the grab cursor.
	 */
    public boolean isGrabCursor() {
        return r;
    }

    /**
	 * Sets the size of the area covered by tiles, relative to the the size of
	 * the widget itself. For example, <code>setTileCoverSize(1)</code> would
	 * cause the tiles to cover at least the entire area of the widget
	 * (actually, slightly more, due to rounding to the tile grid). Normally,
	 * however, the tile cover should be <strong>larger</strong> than the
	 * widget, to allow the prefetching of some tiles around the visible ones. 
	 */
    public void setTileCoverSize(double tileCoverSize) {
        if (tileCoverSize <= 0) throw new IllegalArgumentException("tileCoverSize must be positive");
        f.setTileCoverSize(tileCoverSize);
        this.s = tileCoverSize;
    }

    /**
	 * Returns the current size of the tile cover, as a percentage of the size
	 * of the widget itself. Note that the actual cover may be slightly larger,
	 * because of rounding to the tile grid.
	 */
    public double getTileCoverSize() {
        return s;
    }

    /**
	 * Sets whether tiles are to be loaded during temporary changes in the map's
	 * location. Setting this to <code>false</code> can speed up panning and
	 * zooming animations on slow devices.
	 */
    public void setLoadTilesDuringTemporaryChanges(boolean isLoadTilesDuringTemporaryChanges) {
        this.t = isLoadTilesDuringTemporaryChanges;
        f.setLoadTilesDuringTemporaryChanges(isLoadTilesDuringTemporaryChanges);
    }

    /**
	 * Returns whether tiles are loaded during temporary changes in the map's
	 * location.
	 */
    public boolean isLoadTilesDuringTemporaryChanges() {
        return t;
    }

    /**
	 * Converts from local (in the widget's coordinate system), pixel
	 * coordinates to map coordinates.
	 */
    public LatLng localToMap(PointView point) {
        Projection aj = l.getProjection();
        int ak = b.getZoom();
        SizeView al = getMapSize();
        Point am = new Point(al.getWidth() / 2, al.getHeight() / 2);
        Point an = MapUtils.getTargetPointForFixPointMagnification(am, p, o);
        Point ao = an.moveBy(am.getOpposite());
        Point ap = new Point(q).moveBy(ao).moveBy(n.multiplyBy(1 / o));
        Point aq = ap.moveBy((int) ((point.getX() - al.getWidth() / 2) / o), (int) ((point.getY() - al.getHeight() / 2) / o));
        return aj.fromPixelToLatLng(aq, ak);
    }

    /**
	 * Converts from map coordinates to local (in the widget's coordinate
	 * system), pixel coordinates.
	 */
    public Point mapToLocal(LatLng location) {
        Projection aj = l.getProjection();
        int ak = b.getZoom();
        SizeView al = getMapSize();
        Point am = new Point(al.getWidth() / 2, al.getHeight() / 2);
        Point an = MapUtils.getTargetPointForFixPointMagnification(am, p, o);
        Point ao = an.moveBy(am.getOpposite());
        Point ap = new Point(q).moveBy(ao).moveBy(n.multiplyBy(1 / o));
        Point aq = new Point(aj.fromLatLngToPixel(location, ak));
        SizeView ar = aj.getWrapSize(ak);
        if ((ar.getWidth() != 0) || (ar.getHeight() != 0)) {
            int as = ap.getX() - aq.getX();
            int at = ap.getY() - aq.getY();
            int au = MathUtils.sgn(as) * ar.getWidth();
            int av = MathUtils.sgn(at) * ar.getHeight();
            aq.moveBy(au, av);
            if (Math.abs(as) < Math.abs(ap.getX() - aq.getX())) aq.moveBy(-au, 0);
            if (Math.abs(at) < Math.abs(ap.getY() - aq.getY())) aq.moveBy(0, -av);
        }
        return am.moveBy(aq.moveBy(ap.getOpposite()).multiplyBy(o));
    }

    /**
	 * Sets the length of zoom animations, in milliseconds.
	 */
    public void setZoomAnimationLength(int zoomAnimationLength) {
        if (zoomAnimationLength <= 0) throw new IllegalArgumentException("zoomAnimationLength must be positive");
        this.v = zoomAnimationLength;
    }

    /**
	 * Returns the length of zoom animations, in milliseconds.
	 */
    public int getZoomAnimationLength() {
        return v;
    }

    /**
	 * Sets the length of pan animations, in milliseconds.
	 */
    public void setPanAnimationLength(int panAnimationLength) {
        if (panAnimationLength <= 0) throw new IllegalArgumentException("panAnimationLength must be positive");
        this.w = panAnimationLength;
    }

    /**
	 * Returns the length of pan animations, in milliseconds.
	 */
    public int getPanAnimationLength() {
        return w;
    }

    /**
	 * Returns whether panning to the specified location should use animation
	 * or not. The current implementation returns whether the current bounds and
	 * the resulting bounds intersect.  
	 */
    public boolean shouldAnimateTo(LatLng newCenter) {
        if (l == null) return false;
        Projection aj = l.getProjection();
        int ak = b.getZoom();
        Point al = new Point(aj.fromLatLngToPixel(b.getCenter(), ak)).moveBy(n);
        PointView am = aj.fromLatLngToPixel(newCenter, ak);
        Point an = al.getDistancePoint(am, null);
        SizeView ao = getMapSize();
        return (Math.abs(an.getX()) < ao.getWidth()) && (Math.abs(an.getY()) < ao.getHeight());
    }

    /**
	 * Ends the currently running animation, if any.
	 */
    public void endCurrentAnimation() {
        if ((u != null) && u.isRunning()) {
            u.end();
            u = null;
        }
    }

    /**
	 * Ends the current animation, if any, and starts a new one.
	 */
    public void startAnimation(MapAnimation animation) {
        endCurrentAnimation();
        u = animation;
        u.start();
    }

    /***/
    private void panTo(PointView aj, boolean ak, LatLng al) {
        SizeView am = getMapSize();
        Point an = new Point(aj.getX() - am.getWidth() / 2, aj.getY() - am.getHeight() / 2);
        if (ak && !an.equals(0, 0)) startAnimation(new PanAnimation(getPanAnimationLength(), an, al)); else b.setCenter(al);
    }

    /**
	 * Pans the map to center on the specified point, in widget coordinates,
	 * possibly animating the transition.
	 */
    public void panTo(PointView location, boolean animate) {
        panTo(location, animate, localToMap(location));
    }

    /**
	 * Pans the map by the specified offset, in pixels, possibly animating the
	 * transition.
	 */
    public void panBy(PointView offset, boolean animate) {
        SizeView aj = getMapSize();
        panTo(new Point(aj.getWidth() / 2 + offset.getX(), aj.getHeight() / 2 + offset.getY()), animate);
    }

    /**
	 * Moves the map to center on the specified coordinate, possibly animating
	 * the transition. 
	 */
    public void moveTo(LatLng location, boolean animate) {
        panTo(mapToLocal(location), animate, location);
    }

    /**
	 * Returns whether animated zoom is supported by the current environment.
	 * Since animated zoom is now supported on all browsers, this method is
	 * deprecated and always returns <code>true</code>.
	 */
    @Deprecated
    public static boolean isAnimatedZoomSupported() {
        return TileLayerWidget.isMagnificationSupported();
    }

    /***/
    private Point ac(int aj, PointView ak) {
        SizeView al = getMapSize();
        Point am = new Point(al.getWidth() / 2, al.getHeight() / 2);
        int an = b.getZoom();
        double ao = l.getProjection().getZoomMagnification(an, an + aj) / this.o;
        return MapUtils.getTargetPointForFixPointMagnification(am, ak, ao);
    }

    /***/
    private Point ad(int aj, PointView ak) {
        SizeView al = getMapSize();
        Point am = new Point(al.getWidth() / 2, al.getHeight() / 2);
        int an = b.getZoom();
        double ao = l.getProjection().getZoomMagnification(an, an + aj) / this.o;
        return MapUtils.getFixedPointForTargetPointMagnification(am, ak, ao);
    }

    /**
	 * Zooms in or out by specified number of zooms, while moving the map to
	 * the specified location, optionally with animation (if supported)
	 * 
	 * @param zoomChange The number of zooms to change by (use negative values
	 * to zoom out). For convenience, if the resulting zoom is illegal, the
	 * method zooms to the nearest legal zoom instead.
	 * @param targetLocation The location which will be centered at the end of
	 * the zoom change.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomByTargetLocation(int zoomChange, LatLng targetLocation, boolean animate) {
        endCurrentAnimation();
        int aj = b.getZoom();
        zoomChange = b.clampToLegal(aj + zoomChange) - aj;
        if ((zoomChange == 0) && (o == 1.0)) moveTo(targetLocation, animate); else if (animate) startAnimation(new ZoomAnimation(getZoomAnimationLength(), zoomChange, targetLocation)); else b.set(targetLocation, aj + zoomChange);
    }

    /**
	 * Zooms to the specified zoom, while moving the map to the specified
	 * location, optionally with animation (if supported).
	 * 
	 * @param zoom The new zoom.
	 * @param targetLocation The location which will be centered at the end of
	 * the zoom change.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomToTargetLocation(int zoom, LatLng targetLocation, boolean animate) {
        b.zoomCheck(zoom);
        endCurrentAnimation();
        zoomByTargetLocation(zoom - b.getZoom(), targetLocation, animate);
    }

    /**
	 * Zooms in or out by specified number of zooms, while moving the map to
	 * the specified pixel location (in widget coordinates), optionally with
	 * animation (if supported)
	 * 
	 * @param zoomChange The number of zooms to change by (use negative values
	 * to zoom out). For convenience, if the resulting zoom is illegal, the
	 * method zooms to the nearest legal zoom instead.
	 * @param targetPoint The point on the widget (in widget coordinates) which
	 * will be centered at the end of the zoom change.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomByTargetPoint(int zoomChange, PointView targetPoint, boolean animate) {
        endCurrentAnimation();
        zoomByTargetLocation(zoomChange, localToMap(targetPoint), animate);
    }

    /**
	 * Zooms to the specified zoom, while moving the map to the specified
	 * pixel location (in widget coordinates), optionally with animation (if
	 * supported).
	 * 
	 * @param zoom The new zoom.
	 * @param targetPoint The point on the widget (in widget coordinates) which
	 * will be centered at the end of the zoom change.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomToTargetPoint(int zoom, PointView targetPoint, boolean animate) {
        endCurrentAnimation();
        zoomToTargetLocation(zoom, localToMap(targetPoint), animate);
    }

    /**
	 * Zooms in or out by specified number of zooms, while keeping the specified
	 * pixel point (in widget coordinates) fixed on the screen, optionally with
	 * animation (if supported).
	 * 
	 * @param zoomChange The number of zooms to change by (use negative values
	 * to zoom out). For convenience, if the resulting zoom is illegal, the
	 * method zooms to the nearest legal zoom instead. 
	 * @param fixedPoint The point on the widget (in widget coordinates) which
	 * will remain unmoved on the screen during and after the zoom change.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomByFixPoint(int zoomChange, PointView fixedPoint, boolean animate) {
        endCurrentAnimation();
        int aj = b.getZoom();
        zoomChange = b.clampToLegal(aj + zoomChange) - aj;
        Point ak = ac(zoomChange, fixedPoint);
        zoomByTargetPoint(zoomChange, ak, animate);
    }

    /**
	 * Zooms to the specified zoom, while keeping the specified pixel point (in
	 * widget coordinates) fixed on the screen, optionally with animation (if
	 * supported).
	 * 
	 * @param zoom The new zoom.
	 * @param fixedPoint The point on the widget (in widget coordinates) which
	 * will remain unmoved on the screen during and after the zoom change.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomToFixPoint(int zoom, PointView fixedPoint, boolean animate) {
        endCurrentAnimation();
        int aj = b.getZoom();
        int ak = zoom - aj;
        Point al = ac(ak, fixedPoint);
        zoomToTargetPoint(zoom, al, animate);
    }

    /**
	 * Zooms in or out by the specified number of zooms, optionally with
	 * animation (if supported).
	 * 
	 * @param zoomChange The number of zooms to change by (use negative values
	 * to zoom out). For convenience, if the resulting zoom is illegal, the
	 * method zooms to the nearest legal zoom instead.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomBy(int zoomChange, boolean animate) {
        endCurrentAnimation();
        zoomByTargetLocation(zoomChange, b.getCenter(), animate);
    }

    /**
	 * Zooms to the specified zoom, optionally with animation (if supported).
	 * 
	 * @param zoom The new zoom.
	 * @param animate If <code>true</code>, the zoom will change with a short
	 * animation.
	 */
    public void zoomTo(int zoom, boolean animate) {
        endCurrentAnimation();
        zoomToTargetLocation(zoom, b.getCenter(), animate);
    }

    /**
	 * Displays the specified area, centered at the center of the bounds, using
	 * the maximal zoom at which the entire area is visible.
	 * 
	 *  @param bounds The bounds of the area to display.
	 *  @param animate Whether the transition from the current location should
	 *  be animated.
	 */
    public void show(LatLngBounds bounds, boolean animate) {
        LatLng aj = bounds.getCenter();
        int ak = b.getMaxZoom();
        int al = b.getMinZoom();
        Projection am = l.getProjection();
        SizeView an = getMapSize();
        while (ak > al) {
            PointView ao = am.fromLatLngToPixel(bounds.getSouthWest(), ak);
            PointView ap = am.fromLatLngToPixel(bounds.getNorthEast(), ak);
            SizeView aq = ao.getDistanceSize(ap);
            if ((aq.getWidth() <= an.getWidth()) && (aq.getHeight() <= an.getHeight())) break;
            ak--;
        }
        zoomToTargetLocation(ak, aj, animate);
    }

    /***/
    private void ae(boolean aj) {
        for (Overlay ak : m) ak.updated(this, aj);
    }

    /***/
    private void af() {
        for (Widget aj : h) g.remove(aj);
        h.clear();
        ah(0, 0);
        int aj = b.getZoom();
        q.set(l.getProjection().fromLatLngToPixel(b.getCenter(), aj));
        f.setMap(l.getLayers(), q, aj);
    }

    /***/
    private void ag() {
        if (l == null) return;
        ah(0, 0);
        ai(1.0, new Point(0, 0));
        for (int aj = h.size() - 1; aj >= 0; aj--) {
            TileLayersPanel ak = h.get(aj);
            if (ak.getReadyTileCount() == 0) {
                h.remove(aj);
                g.remove(ak);
            } else ak.cancelLoadingUnreadyTiles();
        }
        int aj = b.getZoom();
        LatLng ak = b.getCenter();
        Projection al = l.getProjection();
        q.set(al.fromLatLngToPixel(ak, aj));
        f.setView(q, aj);
        f.setMagnification(1.0, mapToLocal(ak), q, 0);
        ae(false);
        for (TileLayersPanel am : h) {
            int an = am.getZoom();
            PointView ao = al.fromLatLngToPixel(ak, an);
            am.setCenter(ao, false);
            double ap = al.getZoomMagnification(an, aj);
            am.setMagnification(ap, mapToLocal(ak), ao, 0);
        }
    }

    /**
	 * Call this method whenever you manually adjust the size of the widget
	 * after it has been displayed. This is necessary because there is currently
	 * no way to detect such size changes. Note that this method should also be
	 * called when the border, margin or padding change, because they may affect
	 * the size of the map.
	 */
    public void resized() {
        if (l == null) return;
        Size aj = new Size(k);
        k.set(g.getOffsetWidth(), g.getOffsetHeight());
        f.resized();
        for (TileLayersPanel ak : h) ak.resized();
        ae(false);
        a.firePropertyChanged(this, SIZE_PROPERTY_NAME, aj, new Size(k));
    }

    /**
	 * Adds a temporary offset, in pixels, to the "official", permanent,
	 * location of the map.
	 */
    public void setTemporaryOffset(PointView offset) {
        if (n.equals(offset)) return;
        if ((u != null) && u.isRunning()) u.beforeTemporaryOffsetChange(offset);
        Projection aj = l.getProjection();
        LatLng ak = b.getCenter();
        Point al = new Point();
        int am = b.getZoom();
        double an = f.getMagnification();
        al.set(aj.fromLatLngToPixel(ak, am)).moveBy((int) (offset.getX() / an), (int) (offset.getY() / an));
        f.setCenter(al, true);
        for (TileLayersPanel ao : h) {
            int ap = ao.getZoom();
            an = ao.getMagnification();
            al.set(aj.fromLatLngToPixel(ak, ap)).moveBy((int) (offset.getX() / an), (int) (offset.getY() / an));
            ao.setCenter(al, true);
        }
        ah(offset.getX(), offset.getY());
        ae(true);
    }

    /**
	 * Returns the temporary offset of the map, in pixels, from its "official",
	 * permanent, location.
	 */
    public PointView getTemporaryOffset() {
        return n;
    }

    /**
	 * Makes the temporary offset permanent and clears it.
	 */
    public void applyTemporaryOffset() {
        Projection aj = l.getProjection();
        int ak = b.getZoom();
        Point al = new Point(q).moveBy(n.multiplyBy(1 / o));
        LatLng am = aj.fromPixelToLatLng(al, ak);
        b.setCenter(am);
        if (!n.equals(0, 0)) setTemporaryOffset(new Point(0, 0));
    }

    /***/
    private void ah(int aj, int ak) {
        Point al = new Point(n);
        Point am = new Point(aj, ak);
        n.set(aj, ak);
        a.firePropertyChanged(this, TEMPORARY_OFFSET_PROPERTY_NAME, al, am);
    }

    /**
	 * Sets the magnification of the map while keeping the specified point (in
	 * pixels, relative to the view) fixed on the screen. Note that this is just
	 * a visual effect; it does not affect the current zoom or location.
	 */
    public void setMagnification(double magnification, PointView fixedPoint) {
        if (magnification <= 0) throw new IllegalArgumentException("magnification must be positive");
        if (fixedPoint == null) throw new IllegalArgumentException("fixedPoint may not be null");
        if ((u != null) && u.isRunning()) u.beforeMagnificationChange(magnification, fixedPoint);
        Projection aj = l.getProjection();
        int ak = b.getZoom();
        f.setMagnification(magnification, fixedPoint, f.getCenter(), 0);
        for (TileLayersPanel al : h) {
            double am = aj.getZoomMagnification(al.getZoom(), ak);
            double an = am * magnification;
            al.setMagnification(an, fixedPoint, al.getCenter(), 0);
        }
        ai(magnification, fixedPoint);
        ae(true);
    }

    /**
	 * Returns the current magnification.
	 */
    public Pair<Double, PointView> getMagnification() {
        return new Pair<Double, PointView>(o, p);
    }

    /***/
    private void ai(double aj, PointView ak) {
        Pair<Double, PointView> al = new Pair<Double, PointView>(this.o, new Point(this.p));
        Pair<Double, PointView> am = new Pair<Double, PointView>(aj, ak);
        this.o = aj;
        this.p.set(ak);
        a.firePropertyChanged(this, MAGNIFICATION_PROPERTY_NAME, al, am);
    }

    /**
	 * Applies the current magnification by setting the zoom and location of the
	 * map to values closest to those seen by the user. The direction in which
	 * magnification is rounded to the closest supported zoom is determined by
	 * the <code>roundingDirection</code> parameter. A negative value specifies
	 * that the zoom should be selected such that its magnification is smaller
	 * than the current magnification. A positive value specifies that it should
	 * be larger than the current magnification. A value of 0 specifies that it
	 * should be rounded to the closest supported zoom, whether up or down.
	 */
    public void applyMagnification(boolean animate, int roundingDirection) {
        Projection aj = l.getProjection();
        double ak = Math.log(o);
        int al = b.getZoom();
        int am = al;
        double an = 0;
        for (int ao = b.getMinZoom(); ao <= b.getMaxZoom(); ao++) {
            double ap = Math.log(aj.getZoomMagnification(al, ao));
            if (Math.abs(ap - ak) < Math.abs(an - ak)) {
                am = ao;
                an = ap;
            }
        }
        int ao;
        if (roundingDirection == 0) ao = am; else if (roundingDirection > 0) ao = an >= ak ? am : Math.min(am + 1, b.getMaxZoom()); else ao = an <= ak ? am : Math.max(am - 1, b.getMinZoom());
        SizeView ap = getMapSize();
        Point aq = new Point(ap.getWidth() / 2, ap.getHeight() / 2);
        zoomToTargetPoint(ao, aq, animate);
    }

    /**
	 * Applies the current magnification by setting the zoom and location of the
	 * map to values closest to those seen by the user.
	 */
    public void applyMagnification(boolean animate) {
        applyMagnification(animate, 0);
    }

    /**
	 * Invoked when the widget is attached to the DOM. Causes the tiles to be
	 * displayed.
	 */
    protected void onLoad() {
        super.onLoad();
        j = UiUtils.addSizeChangeHandler(this, new ResizeHandler() {

            public void onResize(ResizeEvent evt) {
                resized();
            }
        });
        resized();
    }

    /**
	 * Invoked when the widget is removed from the DOM.
	 */
    protected void onUnload() {
        j.removeHandler();
        j = null;
        super.onUnload();
    }

    /***/
    private class MagnifiedTileLayersPanelsCleaner implements ReadyListener<TileLayersPanel> {

        /***/
        private boolean aj = false;

        /***/
        private boolean ak = false;

        /**
		 * Sets whether we ignore the main tile layers panel becoming ready, and
		 * don't remove all the magnified tile layers panels when it does. This
		 * is useful when the main tile layers panel is temporarily not
		 * displayed, so although it may be ready, it doesn't actually obscure
		 * the magnified tile layers panels. It occurs, for example, during a
		 * zoom animation.
		 * Note that we can't simply rely on the visibility of the main
		 * tile layers panel, for two reasons. One is that to make a tile layers
		 * panel start loading its tiles it needs to be visible (otherwise its
		 * offset size is 0x0), and if its tiles happen to be already cached,
		 * it will become ready from within the call setting its new view, so
		 * if it is made invisible first, it won't load the tiles, and if its
		 * new view is set first, it will become ready before it is invisible
		 * (causing this class to remove all the magnified tile layers panels).
		 * The second reason is that either way, we need someone to notify us
		 * when the main tile layers panel becomes visible again (since there is
		 * no listener for that), so why not have the same someone notify us
		 * when it becomes invisible in the first place?
		 */
        public void setIgnoreMainTileLayersPanel(boolean ignore) {
            if (aj && !ignore && ak) al();
            this.aj = ignore;
            ak = false;
        }

        /**
		 * {@inheritDoc}
		 */
        public void becameReady(TileLayersPanel source) {
            if (source == f) {
                if (aj) ak = true; else al();
            } else am(source);
        }

        /**
		 * {@inheritDoc}
		 */
        public void becameUnready(TileLayersPanel source) {
            if (aj) ak = false;
        }

        /***/
        private void al() {
            for (TileLayersPanel an : h) g.remove(an);
            h.clear();
        }

        /***/
        private void am(TileLayersPanel an) {
            int ao = h.indexOf(an);
            if (ao <= 0) return;
            for (int ap = 0; ap < ao; ap++) g.remove(h.get(ap));
            for (int ap = 0; ap < ao; ap++) h.remove(0);
        }
    }

    /**
	 * The superclass for map-related animations. 
	 */
    public abstract static class MapAnimation extends Animation {

        /**
		 * Creates a new {@link MapAnimation} with the specified duration and
		 * desired frames-per-second.
		 */
        public MapAnimation(int duration, int fps) {
            super(duration, fps);
        }

        /**
		 * Creates a new {@link MapAnimation} with the specified duration.
		 */
        public MapAnimation(int duration) {
            super(duration);
        }

        /**
		 * Invoked when this animation is a {@link MapWidget}'s current running
		 * animation and the temporary offset is about to be changed. The
		 * default implementation does nothing.
		 */
        public void beforeTemporaryOffsetChange(PointView newTemporaryOffset) {
        }

        /**
		 * Invoked when this animation is a {@link MapWidget}'s current running
		 * animation and the magnification is about to be changed.
		 */
        public void beforeMagnificationChange(double magnification, PointView fixedPoint) {
        }
    }

    /***/
    private class PanAnimation extends MapAnimation {

        /***/
        private final PointView aj;

        /***/
        private final LatLng ak;

        /***/
        private Point al;

        /**
		 * Creates a new <code>PanAnimation</code> which will smoothly pan the
		 * map by the specified offset and afterwards set its location to the
		 * specified coordinate.
		 */
        public PanAnimation(int animationLength, PointView offset, LatLng endLocation) {
            super(animationLength);
            if (offset == null) throw new IllegalArgumentException("offset may not be null");
            if (endLocation == null) throw new IllegalArgumentException("endLocation may not be null");
            this.aj = offset;
            this.ak = endLocation;
        }

        /**
		 * Remembers the initial offset.
		 */
        protected void animationStart() {
            al = new Point(getTemporaryOffset());
        }

        /**
		 * Displays the next frame of the animation.
		 */
        protected void nextFrame() {
            Point am = new Point(aj).multiplyBy(getSinusoidalProgress()).moveBy(al);
            setTemporaryOffset(am);
        }

        /**
		 * Sets the location to the final one.
		 */
        protected void animationEnd() {
            getMapLocationModel().setCenter(ak);
        }

        /**
		 * This animation is incompatible with magnification, so we cancel it
		 * whenever magnification changes.
		 */
        public void beforeMagnificationChange(double magnification, PointView fixedPoint) {
            if (isRunning()) end();
        }
    }

    /***/
    private class ZoomAnimation extends MapAnimation {

        /***/
        private final int aj;

        /***/
        private final LatLng ak;

        /***/
        private final PointView al;

        /***/
        private double am;

        /**
		 * Creates a new <code>ZoomAnimation</code> with the specified change in
		 * zoom at the end of the magnification and target center point at the
		 * end of the zoom animation.
		 */
        public ZoomAnimation(int animationLength, int zoomChange, LatLng targetLocation) {
            super(animationLength);
            if (targetLocation == null) throw new IllegalArgumentException("targetLocation may not be null");
            this.aj = zoomChange;
            this.ak = targetLocation;
            this.al = ad(zoomChange, mapToLocal(targetLocation));
        }

        /**
		 * Initializes the animation.
		 */
        protected void animationStart() {
            Projection an = l.getProjection();
            int ao = b.getZoom();
            final int ap = ao + aj;
            am = an.getZoomMagnification(ao, ap) / o;
            h.add(f);
            if (h.size() > 3) {
                g.remove(h.get(0));
                h.remove(0);
            }
            e.setIgnoreMainTileLayersPanel(true);
            f = y();
            UiUtils.setVisibility(f, false);
            UiUtils.addFullSize(g, f);
            Scheduler.ScheduledCommand aq = new Scheduler.ScheduledCommand() {

                public void execute() {
                    f.setMap(l.getLayers(), l.getProjection().fromLatLngToPixel(ak, ap), ap);
                }
            };
            if (BrowserUtils.isAppleWebKitOnAndroid()) Scheduler.get().scheduleDeferred(aq); else aq.execute();
            UiUtils.setVisibility(d, false);
            if (TileLayerWidget.isAnimatedMagnificationSupported()) {
                for (TileLayersPanel ar : h) {
                    double as = an.getZoomMagnification(ar.getZoom(), ap);
                    ar.setMagnification(as, al, ar.getCenter(), getDuration());
                }
            }
        }

        /**
		 * Displays the next frame of the animation.
		 */
        protected void nextFrame() {
            if (TileLayerWidget.isAnimatedMagnificationSupported()) return;
            double an = getSinusoidalProgress();
            double ao = 1 * (1 - an) + am * an;
            Projection ap = l.getProjection();
            int aq = b.getZoom();
            for (TileLayersPanel ar : h) {
                double as = ap.getZoomMagnification(ar.getZoom(), aq) * o;
                double at = as * ao;
                ar.setMagnification(at, al, ar.getCenter(), 0);
            }
        }

        /**
		 * Sets the end zoom and finishes the animation.
		 */
        protected void animationEnd() {
            Projection an = l.getProjection();
            int ao = b.getZoom() + aj;
            for (TileLayersPanel ap : h) {
                double aq = an.getZoomMagnification(ap.getZoom(), ao);
                ap.setMagnification(aq, al, l.getProjection().fromLatLngToPixel(ak, ap.getZoom()), 0);
            }
            UiUtils.setVisibility(f, true);
            UiUtils.setVisibility(d, true);
            e.setIgnoreMainTileLayersPanel(false);
            b.set(ak, ao);
        }

        /**
		 * This animation is incompatible with temporary offset changes, so we
		 * end it when one is about to occur.
		 */
        public void beforeTemporaryOffsetChange(PointView offset) {
            if (isRunning()) end();
        }
    }

    /***/
    private static class InteractionPanel extends FocusPanel implements HasAllMouseHandlers, HasAllTouchHandlers {

        /**
		 * Creates a new {@link InteractionPanel}.
		 */
        public InteractionPanel() {
            sinkEvents(Event.MOUSEEVENTS | Event.ONMOUSEWHEEL | Event.TOUCHEVENTS);
        }
    }
}
