package com.maryanovsky.map.client;

/**
 * The interface for overlays added to the map. Usually, you will not need to
 * implement this interface directly, but use {@linkplain WidgetOverlay}
 * instead.
 * 
 * @author Maryanovsky Alexander
 */
public interface Overlay {

    /**
	 * Invoked when the overlay is added to the <code>MapWidget</code>.
	 * The implementation is supposed to actually add something to the
	 * <code>mapWidget</code>'s overlay panel.
	 */
    void added(MapWidget mapWidget);

    /**
	 * Invoked when the map view changes. <code>isTemporary</code> specifies
	 * whether the change is temporary (such as during a drag of the map or a
	 * pan animation) or permanent.
	 */
    void updated(MapWidget mapWidget, boolean isTemporary);

    /**
	 * Invoked when the overlay is removed from the <code>MapWidget</code>.
	 * The implementtion is supposed to actually remove whatever it added from
	 * the <code>mapWidget</code>'s overlay panel.
	 */
    void removed(MapWidget mapWidget);
}
