package com.maryanovsky.map.client;

import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.geom.SizeView;

/**
 * A projection maps between latitude-longitude coordinates and pixel
 * coordinates (at a certain zoom).
 * 
 * @author Maryanovsky Alexander
 */
public interface Projection {

    /**
	 * Returns the pixel coordinates corresponding to the specified
	 * latitude-longitude coordinates at the specified zoom.
	 */
    PointView fromLatLngToPixel(LatLng latLng, int zoom);

    /**
	 * Returns the latitude-longitude coordinates corresponding to the specified
	 * pixel coordinates at the specified zoom. This method should always return
	 * a legal value, even if the pixel coordinates fall outside the map. In
	 * such a case, the projection may, for example, choose to return the
	 * closest legal location or wrap around.
	 */
    LatLng fromPixelToLatLng(PointView pixel, int zoom);

    /**
	 * Returns the number of pixels after which the map wraps around the earth
	 * on each axis at the specified zoom. A value of 0 indicates that the map
	 * does not repeat itself.
	 */
    SizeView getWrapSize(int zoom);

    /**
	 * Returns the magnification coefficient between the two specified zooms,
	 * i.e. how much the map is magnified when the zoom is changed from
	 * <code>startZoom</code> to <code>endZoom</code>. This would normally be a
	 * value larger than 1 if <code>endZoom</code> is larger than
	 * <code>startZoom</code> and vice versa.
	 */
    double getZoomMagnification(int startZoom, int endZoom);
}
