package com.maryanovsky.map.client;

import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.event.ReadyListener;
import com.maryanovsky.gwtutils.client.geom.SizeView;

/**
 * The interface for implementing map tiles. This is an abstract class rather
 * than an interface because tiles must be {@link Widget}s, and by subclassing
 * {@link Composite}, we allow a subclass to implement a tile using any actual
 * widget it chooses.  
 * 
 * @author Maryanovsky Alexander
 */
public abstract class Tile extends Composite {

    /***/
    private ReadyListener<Tile> a;

    /***/
    private boolean b = true;

    /**
	 * Sets the specified <code>ReadyListener</code> to be notified when the
	 * tile starts and finishes loading.
	 */
    public void setReadyListener(ReadyListener<Tile> readyListener) {
        this.a = readyListener;
    }

    /**
	 * Returns whether this tile is currently ready. For image-based tiles, for
	 * example, this means that all the images comprising the tile have been
	 * loaded.
	 */
    public boolean isReady() {
        return b;
    }

    /**
	 * Marks the tile as unready and notifies our ready listener, if any.
	 * This method is expected to be called by the concrete subclass when the
	 * tile becomes unready or just before it starts loading.
	 */
    protected void markUnready() {
        if (b) {
            b = false;
            if (a != null) a.becameUnready(this);
        }
    }

    /**
	 * Marks the tile as ready and notifies our ready listener, if any.
	 * This method is expected to be called by the concrete subclass when the
	 * tile becomes ready.
	 */
    protected void markReady() {
        if (!b) {
            b = true;
            if (a != null) a.becameReady(this);
        }
    }

    /**
	 * Marks the tile as unready and starts loading it. When done, marks the
	 * tile as ready.
	 */
    public abstract void startLoading();

    /**
	 * If the tile is currently being loaded, cancels loading it.
	 */
    public abstract void cancelLoading();

    /**
	 * Sets the size of the tile, in pixels. This should stretch/compress the
	 * content of the tile to the specified size.  A <code>null</code> value
	 * resets it to the default size, if applicable.
	 */
    public abstract void setTileSizePixels(SizeView tileSize);
}
