package com.maryanovsky.map.client;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.BrowserUtils;
import com.maryanovsky.gwtutils.client.event.GroupReadyListener;
import com.maryanovsky.gwtutils.client.event.ReadyListener;
import com.maryanovsky.gwtutils.client.geom.*;
import com.maryanovsky.gwtutils.client.ui.UiUtils;

/**
 * Displays a single {@linkplain TileLayer}.
 * 
 * @author Maryanovsky Alexander
 */
final class TileLayerWidget extends Composite {

    /***/
    private static final Magnifier a;

    static {
        if (BrowserUtils.isAppleWebKitOnAndroid()) a = new WebkitTransformMagnifier(); else if (BrowserUtils.isMobileAppleWebKit()) {
            if ((BrowserUtils.compareVersions("526", BrowserUtils.getAppleWebKitVersion()) <= 0) && WebkitTransformMagnifier.isSupported()) a = new IOSWebkitTransformMagnifier(); else a = new TileSizeMagnifier();
        } else if (BrowserUtils.isMacOS()) a = new TileSizeMagnifier(); else if (Css3Magnifier.isSupported()) a = new Css3Magnifier(); else if (WebkitTransformMagnifier.isSupported()) a = new WebkitTransformMagnifier(); else if (MozTransformMagnifier.isSupported()) a = new MozTransformMagnifier(); else if (IeMagnifier.isSupported()) a = new IeMagnifier(); else a = new TileSizeMagnifier();
    }

    /***/
    private final TileLayer tileLayer;

    /***/
    private final AbsolutePanel b;

    /***/
    private final AbsolutePanel tilePanel;

    /***/
    private final GroupReadyListener<Tile, TileLayerWidget> c;

    /***/
    private final TilePanelMover d;

    /***/
    private final Point e = new Point();

    /***/
    private int f;

    /***/
    private double g = 1.3;

    /***/
    private final Rectangle h = new Rectangle();

    /***/
    private boolean i = true;

    /***/
    private Size j;

    /***/
    private double k = 1.0;

    /***/
    private final Transform tilePanelTransform = BrowserUtils.isMobileAppleWebKit() || BrowserUtils.isAppleWebKitOnAndroid() ? new MobileWebkitTransform() : new Transform();

    /**
	 * Creates a new <code>TileLayerWidget</code> to display the specified tile
	 * layer at the specified coordinates and zoom (initially). The specified
	 * <code>ReadyListener</code> (which may be <code>null</code>) will be
	 * notified whenever the <code>TileLayerWidget</code> changes its ready
	 * state). A <code>TileLayerWidget</code> is ready iff all of its tiles are
	 * ready.
	 */
    public TileLayerWidget(TileLayer tileLayer, PointView center, int zoom, ReadyListener<TileLayerWidget> readyListener) {
        if (tileLayer == null) throw new IllegalArgumentException("tileLayer may not be null");
        this.tileLayer = tileLayer;
        this.b = new AbsolutePanel();
        this.tilePanel = new AbsolutePanel();
        this.c = readyListener == null ? null : new GroupReadyListener<Tile, TileLayerWidget>(readyListener, this);
        this.d = l();
        DOM.setStyleAttribute(tilePanel.getElement(), "position", "absolute");
        DOM.setStyleAttribute(tilePanel.getElement(), "overflow", "visible");
        tilePanel.setPixelSize(10, 10);
        setView(center, zoom);
        initWidget(m());
        a.init(this);
        d.init();
    }

    /***/
    private TilePanelMover l() {
        if (BrowserUtils.isAppleWebKitOnAndroid() && (BrowserUtils.compareVersions("526", BrowserUtils.getAppleWebKitVersion()) <= 0)) return new WebkitTransformTilePanelMover(); else if (BrowserUtils.isMobileAppleWebKit() && (BrowserUtils.compareVersions("526", BrowserUtils.getAppleWebKitVersion()) <= 0)) return new IOSWebkitTransformTilePanelMover(); else return new DefaultTilePanelMover();
    }

    /***/
    private Widget m() {
        UiUtils.addFullSize(b, tilePanel);
        return b;
    }

    /**
	 * Sets the size of the area covered by tiles, relative to the the size of
	 * the tile layer widget itself. For example,
	 * <code>setTileCoverSize(1)</code> would cause the tiles to cover at least
	 * the entire area of the widget (actually, slightly more, due to rounding
	 * to the tile grid). Normally, however, the tile cover should be
	 * <strong>larger</strong> than the widget, to allow the prefetching of some
	 * tiles around the visible ones. 
	 */
    public void setTileCoverSize(double tileCoverSize) {
        if (tileCoverSize <= 0) throw new IllegalArgumentException("tileCoverSize must be positive");
        this.g = tileCoverSize;
        resized();
    }

    /**
	 * Returns the current size of the tile cover, as a percentage of the size
	 * of the widget itself. Note that the actual cover may be slightly larger,
	 * because of rounding to the tile grid.
	 */
    public double getTileCoverSize() {
        return g;
    }

    /**
	 * Sets whether tiles are loaded during temporary changes in the map's
	 * location.
	 */
    public void setLoadTilesDuringTemporaryChanges(boolean isLoadTilesDuringTemporaryChanges) {
        this.i = isLoadTilesDuringTemporaryChanges;
    }

    /**
	 * Sets the point displayed at the center of the widget.
	 */
    public void setCenter(PointView center, boolean isTemporary) {
        PointView ab = new Point(this.e);
        n(center);
        y(ab, center, isTemporary);
    }

    /***/
    private void n(PointView ab) {
        if (ab == null) throw new IllegalArgumentException("center may not be null");
        this.e.set(ab);
    }

    /**
	 * Sets the zoom at which we're displaying the map.
	 */
    public void setZoom(int zoom) {
        if (this.f == zoom) return;
        o(zoom);
        z();
    }

    /***/
    private void o(int ab) {
        this.f = ab;
    }

    /**
	 * Sets both the center point and the zoom in one call.
	 */
    public void setView(PointView center, int zoom) {
        if (this.f == zoom) {
            setCenter(center, false);
            return;
        }
        n(center);
        o(zoom);
        aa();
    }

    /**
	 * Returns whether magnification other than <code>1.0</code> is supported.
	 * Since animated zoom is now supported on all browsers, this method is
	 * deprecated and always returns <code>true</code>.
	 */
    @Deprecated
    public static boolean isMagnificationSupported() {
        return true;
    }

    /**
	 * Returns whether animated magnification transitions are supported. When
	 * this method returns true, you may pass a non-zero
	 * <code>animationLength</code> argument to the
	 * {@link #setMagnification(double, PointView, PointView, int)} method to
	 * cause the magnification change to be animated.
	 */
    public static boolean isAnimatedMagnificationSupported() {
        return a.isAnimatedMagnificationSupported();
    }

    /**
	 * Sets the magnification at which the tiles are displayed, keeping the
	 * specified point (in pixels, relative to the widget) fixed on the screen.
	 * If <code>fixedPoint</code> is not the center of the widget, the point
	 * displayed at the center changes, and, to avoid accumulating rounding
	 * errors, the caller is required to specify the new center point (although
	 * this is not necessary during temporary magnification changes, if no other
	 * methods are called while the center is incorrect).
	 */
    public void setMagnification(double magnification, PointView fixedPoint, PointView targetPoint, int animationLength) {
        if (magnification <= 0) throw new IllegalArgumentException("magnification must be a positive value");
        if (fixedPoint == null) throw new IllegalArgumentException("fixedPoint may not be null");
        if (!isAnimatedMagnificationSupported() && (animationLength != 0)) throw new IllegalArgumentException("animated magnification is not supported");
        t(fixedPoint);
        n(targetPoint);
        a.setMagnification(this, magnification, animationLength);
        this.k = magnification;
    }

    /**
	 * Returns the current magnification.
	 */
    public double getMagnification() {
        return k;
    }

    /**
	 * Returns the number of tiles which are displayed and ready.
	 */
    public int getReadyTileCount() {
        int ab = 0;
        for (Widget ac : tilePanel) {
            Tile ad = (Tile) ac;
            if (ad.isReady()) ab++;
        }
        return ab;
    }

    /**
	 * Cancels loading any unready tiles.
	 */
    public void cancelLoadingUnreadyTiles() {
        for (Widget ab : tilePanel) {
            Tile ac = (Tile) ab;
            if (!ac.isReady()) ac.cancelLoading();
        }
    }

    /***/
    private Point p(PointView ab, PointView ac) {
        return new Point(ab).moveBy(ac.getX(), ac.getY());
    }

    /***/
    private Rectangle p(RectangleView ab, PointView ac) {
        return new Rectangle(ab).moveBy(ac.getX(), ac.getY());
    }

    /***/
    private Point q(PointView ab, PointView ac) {
        return new Point(ab).moveBy(-ac.getX(), -ac.getY());
    }

    /***/
    private Rectangle q(RectangleView ab, PointView ac) {
        return new Rectangle(ab).moveBy(-ac.getX(), -ac.getY());
    }

    /***/
    private Rectangle r(PointView ab) {
        int ac = (int) (j.getWidth() * g);
        int ad = (int) (j.getHeight() * g);
        Rectangle ae = new Rectangle(-ac / 2, -ad / 2, ac, ad);
        return p(ae, ab);
    }

    /***/
    private Rectangle s(PointView ab) {
        Rectangle ac = r(ab);
        SizeView ad = tileLayer.getTileSize();
        ac.setTopLeft(ac.getTopLeft(null).roundDownToGrid(ad));
        ac.growBy(ad);
        ac.expandToGrid(ad);
        return ac;
    }

    /***/
    private void t(PointView ab) {
        int ac = DOM.getIntStyleAttribute(tilePanel.getElement(), "left") - ab.getX();
        int ad = DOM.getIntStyleAttribute(tilePanel.getElement(), "top") - ab.getY();
        if ((ac == 0) && (ad == 0)) return;
        b.setWidgetPosition(tilePanel, ab.getX(), ab.getY());
        boolean ae = (getMagnification() != 1.0);
        for (Widget af : tilePanel) {
            if (ae) a.moveTile(this, (Tile) af, ac, ad); else {
                int ag = DOM.getIntStyleAttribute(af.getElement(), "left");
                int ah = DOM.getIntStyleAttribute(af.getElement(), "top");
                tilePanel.setWidgetPosition(af, ag + ac, ah + ad);
            }
        }
    }

    /***/
    private void u() {
        t(new Point(b.getOffsetWidth() / 2, b.getOffsetHeight() / 2));
    }

    /***/
    private void v() {
        if (j == null) return;
        u();
        List<Tile> ab = new ArrayList<Tile>();
        for (Widget ac : tilePanel) ab.add((Tile) ac);
        tilePanel.clear();
        for (Tile ac : ab) releaseTile(ac);
        SizeView ac = tileLayer.getTileSize();
        int ad = ac.getWidth();
        int ae = ac.getHeight();
        Rectangle af = s(e);
        Rectangle ag = q(af, e);
        PointView ah = ag.getBottomRight();
        List<Tile> ai = new ArrayList<Tile>();
        Point aj = new Point(ag.getCenter()).roundDownToGrid(ac);
        int ak = ad / 2;
        int al = ae / 2;
        int am = ag.getX();
        int an = af.getX();
        while (am < ah.getX()) {
            int ao = Math.abs(am + ak - aj.getX()) / ad;
            int ap = ag.getY();
            int aq = af.getY();
            while (ap < ah.getY()) {
                int ar = Math.abs(ap + al - aj.getY()) / ae;
                Tile as = x(an, aq, f);
                as.getElement().setPropertyInt("sashaMapsDistToCenterSq", ao * ao + ar * ar);
                tilePanel.add(as, am, ap);
                ai.add(as);
                ap += ae;
                aq += ae;
            }
            am += ad;
            an += ad;
        }
        Collections.sort(ai, new Comparator<Tile>() {

            public int compare(Tile t1, Tile t2) {
                int ao = t1.getElement().getPropertyInt("sashaMapsDistToCenterSq");
                int ap = t2.getElement().getPropertyInt("sashaMapsDistToCenterSq");
                return ao - ap;
            }
        });
        for (Tile ao : ai) ao.startLoading();
        h.set(af);
    }

    /***/
    private void moveTilePanel(PointView ab, PointView ac, boolean ad) {
        d.moveTilePanel(ab, ac, ad);
    }

    /***/
    private void w(PointView ab, PointView ac) {
        moveTilePanel(ab, ac, false);
        Rectangle ad = s(ac);
        Rectangle ae = q(ad, ac);
        Point af = new Point(UiUtils.getTopLeft(tilePanel));
        af.moveBy(-b.getOffsetWidth() / 2, -b.getOffsetHeight() / 2);
        Point ag = af.getOpposite(null);
        Rectangle ah = new Rectangle(ae).moveBy(ag);
        SizeView ai = tileLayer.getTileSize();
        Rectangle aj = new Rectangle().setSize(ai);
        Point ak = Point.getMidPoint(h.getTopRight(null).moveBy(-ai.getWidth(), 0), ad.getBottomLeft(null).moveBy(0, -ai.getHeight()), null);
        ak = q(ak, ac);
        ak.moveBy(ag);
        Point al = new Point();
        List<Tile> am = new ArrayList<Tile>();
        List<Tile> an = new ArrayList<Tile>();
        for (Widget ao : tilePanel) {
            Tile ap = (Tile) ao;
            int aq = DOM.getIntStyleAttribute(ap.getElement(), "left");
            int ar = DOM.getIntStyleAttribute(ap.getElement(), "top");
            aj.setTopLeft(aq, ar);
            if (aj.intersects(ah)) continue;
            am.add(ap);
            Point as = aj.getTopLeft(al);
            as.reflect(ak);
            int at = as.getY();
            int au = as.getX();
            as.moveBy(af);
            as = p(as, ac);
            Tile av = x(as.getX(), as.getY(), f);
            Element aw = av.getElement();
            DOM.setStyleAttribute(aw, "top", at + "px");
            DOM.setStyleAttribute(aw, "left", au + "px");
            an.add(av);
        }
        for (Tile ao : am) {
            tilePanel.remove(ao);
            releaseTile(ao);
        }
        for (Tile ao : an) {
            tilePanel.add(ao);
            ao.startLoading();
        }
        h.set(ad);
    }

    /***/
    private Tile x(int ab, int ac, int ad) {
        SizeView ae = tileLayer.getTileSize();
        Tile af = tileLayer.getTile(ab + ae.getWidth() / 2, ac + ae.getHeight() / 2, ad);
        if (af == null) {
            af = new DummyTile(ae);
        } else {
            af.setReadyListener(c);
        }
        d.init(af);
        a.init(af);
        return af;
    }

    /***/
    private void releaseTile(Tile ab) {
        tileLayer.releaseTile(ab);
        c.becameReady(ab);
    }

    /**
	 * Call this method whenever you manually adjust the size of the widget
	 * after it has been displayed. This is necessary because there is currently
	 * no way to detect such size changes automatically.
	 */
    public void resized() {
        if (j == null) return;
        int ab = getOffsetWidth();
        int ac = getOffsetHeight();
        if (j.equals(ab, ac)) return;
        j.set(ab, ac);
        v();
    }

    /***/
    private void y(PointView ab, PointView ac, boolean ad) {
        if (j == null) return;
        Rectangle ae = r(ac);
        if (h.contains(ae) || (ad && !i) || (getMagnification() != 1.0)) moveTilePanel(ab, ac, ad); else if (h.intersects(ae)) w(ab, ac); else v();
    }

    /***/
    private void z() {
        if (j == null) return;
        v();
    }

    /***/
    private void aa() {
        if (j == null) return;
        v();
    }

    /**
	 * Invoked when the widget is attached to the DOM.
	 */
    protected void onLoad() {
        super.onLoad();
        j = new Size(getOffsetWidth(), getOffsetHeight());
        v();
    }

    /**
	 * Invoked when the widget is detached from the DOM.
	 */
    protected void onUnload() {
        super.onUnload();
        j = null;
        for (Widget ab : tilePanel) releaseTile((Tile) ab);
        tilePanel.clear();
        h.set(0, 0, 0, 0);
    }

    /***/
    private static class Transform {

        /***/
        private double ab = 1;

        /***/
        private final Point ac = new Point(0, 0);

        /**
		 * Returns the current scale.
		 */
        public double getScale() {
            return ab;
        }

        /**
		 * Sets the scale.
		 */
        public void setScale(double scale) {
            this.ab = scale;
        }

        /**
		 * Returns the CSS value to apply the scale.
		 */
        protected String getScaleCss() {
            return "scale(" + getScale() + ")";
        }

        /**
		 * Returns the current translate.
		 */
        public PointView getTranslate() {
            return ac;
        }

        /**
		 * Set the translate.
		 */
        public void setTranslate(PointView translate) {
            this.ac.set(translate);
        }

        /**
		 * Returns the CSS value to apply the translate.
		 */
        protected String getTranslateCss() {
            return "translate(" + getTranslate().getX() + "px," + getTranslate().getY() + "px)";
        }

        /**
		 * Returns the CSS value of the transform.
		 */
        public String toCss() {
            return getScaleCss() + getTranslateCss();
        }
    }

    /***/
    private static class MobileWebkitTransform extends Transform {

        @Override
        protected String getTranslateCss() {
            return "translate3d(" + getTranslate().getX() + "px," + getTranslate().getY() + "px,0)";
        }

        @Override
        protected String getScaleCss() {
            double ab = getScale();
            return "scale3d(" + ab + "," + ab + ",1)";
        }
    }

    /***/
    private static interface Magnifier {

        /**
		 * Performs whatever initialization the magnifier needs on the specified
		 * {@link TileLayerWidget}.
		 */
        void init(TileLayerWidget tileLayerWidget);

        /**
		 * Performs whatever initialization the magnifier needs with the
		 * {@link Tile}.
		 */
        void init(Tile tile);

        /**
		 * Returns whether the magnifier supports animated magnification
		 * transitions.
		 */
        boolean isAnimatedMagnificationSupported();

        /**
		 * Sets the magnification. If the magnifier supports animated
		 * magnification, <code>animationLength</code> may be non-zero.
		 */
        void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength);

        /**
		 * Moves the specified tile by the specified offset in screen
		 * coordinates. This method is needed because the coordinate system of
		 * the tile's parent may be scaled, depending on the implementation of
		 * the magnifier.
		 */
        void moveTile(TileLayerWidget tileLayerWidget, Tile tile, int offsetX, int offsetY);
    }

    /***/
    private abstract static class AbstractMagnifier implements Magnifier {

        /**
		 * {@inheritDoc}
		 */
        public void init(TileLayerWidget tileLayerWidget) {
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(Tile tile) {
        }
    }

    /***/
    private abstract static class ScaledCoordinateSystemMagnifier extends AbstractMagnifier {

        /**
		 * Moves the tile. In the <code>tilePanel</code>'s coordinate system,
		 * the offset needs to be scaled.
		 */
        public void moveTile(TileLayerWidget tileLayerWidget, Tile tile, int offsetX, int offsetY) {
            int ab = DOM.getIntStyleAttribute(tile.getElement(), "left");
            int ac = DOM.getIntStyleAttribute(tile.getElement(), "top");
            double ad = tileLayerWidget.getMagnification();
            offsetX = (int) (offsetX / ad);
            offsetY = (int) (offsetY / ad);
            tileLayerWidget.tilePanel.setWidgetPosition(tile, ab + offsetX, ac + offsetY);
        }
    }

    /***/
    private static class Css3Magnifier extends ScaledCoordinateSystemMagnifier {

        /**
		 * Returns whether supported.
		 */
        public static native boolean isSupported()/*-{
			return 'transform' in document.body.style;
		}-*/;

        /**
		 * {@inheritDoc}
		 */
        public boolean isAnimatedMagnificationSupported() {
            return false;
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(TileLayerWidget tileLayerWidget) {
            DOM.setStyleAttribute(tileLayerWidget.tilePanel.getElement(), "transformOrigin", "0 0");
        }

        /**
		 * {@inheritDoc}
		 */
        public void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength) {
            Transform ab = tileLayerWidget.tilePanelTransform;
            ab.setScale(magnification);
            DOM.setStyleAttribute(tileLayerWidget.tilePanel.getElement(), "transform", ab.toCss());
        }
    }

    /***/
    private static class IeMagnifier extends ScaledCoordinateSystemMagnifier {

        /**
		 * Returns whether supported.
		 */
        public static native boolean isSupported()/*-{
			var ua = navigator.userAgent.toLowerCase();
			if (('zoom' in document.body.style) && (ua.indexOf("msie") != -1)){
				var version;
				var re  = new RegExp("msie ([0-9]{1,}[\.0-9]{0,})");
    			if (re.exec(ua) != null)
      				version = parseFloat( RegExp.$1 );
      			else
      				return false;
      			
      			return version < 8.0; // IE8 completely changed what the zoom property does.
      		}
      		else
      			return false;
		}-*/;

        /**
		 * {@inheritDoc}
		 */
        public boolean isAnimatedMagnificationSupported() {
            return false;
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(TileLayerWidget tileLayerWidget) {
        }

        /**
		 * Sets the magnification.
		 */
        public void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength) {
            DOM.setStyleAttribute(tileLayerWidget.tilePanel.getElement(), "zoom", String.valueOf(magnification));
        }
    }

    /***/
    private static class WebkitTransformMagnifier extends ScaledCoordinateSystemMagnifier {

        /**
		 * Returns whether supported.
		 */
        public static native boolean isSupported()/*-{
			return 'WebkitTransform' in document.body.style;
		}-*/;

        /**
		 * {@inheritDoc}
		 */
        public boolean isAnimatedMagnificationSupported() {
            return true;
        }

        /**
		 * {@inheritDoc}
		 */
        public void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength) {
            Transform ab = tileLayerWidget.tilePanelTransform;
            Element ac = tileLayerWidget.tilePanel.getElement();
            DOM.setStyleAttribute(ac, "WebkitTransition", "-webkit-transform " + animationLength + "ms ease-in-out 0");
            DOM.setStyleAttribute(ac, "WebkitTransformOrigin", "0px 0px");
            if (ab.getScale() != magnification) {
                ab.setScale(magnification);
                DOM.setStyleAttribute(ac, "WebkitTransform", ab.toCss());
            }
        }
    }

    /***/
    private static class IOSWebkitTransformMagnifier extends ScaledCoordinateSystemMagnifier {

        /**
		 * Returns whether supported.
		 */
        public static native boolean isSupported()/*-{
			return 'WebkitTransform' in document.body.style;
		}-*/;

        /**
		 * {@inheritDoc}
		 */
        public boolean isAnimatedMagnificationSupported() {
            return true;
        }

        /**
		 * {@inheritDoc}
		 */
        public void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength) {
            Transform ab = tileLayerWidget.tilePanelTransform;
            AbsolutePanel ac = tileLayerWidget.tilePanel;
            for (Widget ad : ac) {
                DOM.setStyleAttribute(ad.getElement(), "WebkitTransition", "-webkit-transform " + animationLength + "ms ease-in-out 0");
            }
            if (ab.getScale() != magnification) {
                ab.setScale(magnification);
                for (Widget ad : ac) {
                    DOM.setStyleAttribute(ad.getElement(), "WebkitTransition", "-webkit-transform " + animationLength + "ms ease-in-out 0");
                    int ae = DOM.getIntStyleAttribute(ad.getElement(), "left");
                    int af = DOM.getIntStyleAttribute(ad.getElement(), "top");
                    String ag = (-ae) + "px " + (-af) + "px";
                    DOM.setStyleAttribute(ad.getElement(), "WebkitTransformOrigin", ag);
                    ad.getElement().getStyle().setProperty("WebkitTransform", ab.toCss());
                }
            }
        }

        /**
		 * {@inheritDoc}
		 */
        public void moveTile(TileLayerWidget tileLayerWidget, Tile tile, int offsetX, int offsetY) {
            int ab = DOM.getIntStyleAttribute(tile.getElement(), "left");
            int ac = DOM.getIntStyleAttribute(tile.getElement(), "top");
            double ad = tileLayerWidget.tilePanelTransform.getScale();
            offsetX = (int) (offsetX / ad);
            offsetY = (int) (offsetY / ad);
            int ae = ab + offsetX;
            int af = ac + offsetY;
            tileLayerWidget.tilePanel.setWidgetPosition(tile, ae, af);
            String ag = (-ae) + "px " + (-af) + "px";
            DOM.setStyleAttribute(tile.getElement(), "WebkitTransformOrigin", ag);
        }
    }

    /***/
    private static class MozTransformMagnifier extends ScaledCoordinateSystemMagnifier {

        /**
		 * Returns whether supported.
		 */
        public static native boolean isSupported()/*-{
			return 'MozTransform' in document.body.style;
		}-*/;

        /**
		 * {@inheritDoc}
		 */
        public boolean isAnimatedMagnificationSupported() {
            return false;
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(TileLayerWidget tileLayerWidget) {
            DOM.setStyleAttribute(tileLayerWidget.tilePanel.getElement(), "MozTransformOrigin", "0 0");
        }

        /**
		 * {@inheritDoc}
		 */
        public void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength) {
            Transform ab = tileLayerWidget.tilePanelTransform;
            ab.setScale(magnification);
            DOM.setStyleAttribute(tileLayerWidget.tilePanel.getElement(), "MozTransform", ab.toCss());
        }
    }

    /***/
    private static class TileSizeMagnifier extends AbstractMagnifier {

        /**
		 * {@inheritDoc}
		 */
        public boolean isAnimatedMagnificationSupported() {
            return false;
        }

        /**
		 * Sets the magnification.
		 */
        public void setMagnification(TileLayerWidget tileLayerWidget, double magnification, int animationLength) {
            SizeView ab = tileLayerWidget.tileLayer.getTileSize();
            SizeView ac = new Size((int) Math.round(ab.getWidth() * magnification), (int) Math.round(ab.getHeight() * magnification));
            double ad = ((double) ac.getWidth()) / ab.getWidth();
            double ae = ((double) ac.getHeight()) / ab.getHeight();
            AbsolutePanel af = tileLayerWidget.tilePanel;
            for (Widget ag : af) {
                Tile ah = (Tile) ag;
                int ai = DOM.getIntStyleAttribute(ah.getElement(), "left");
                int aj = DOM.getIntStyleAttribute(ah.getElement(), "top");
                int ak = (int) Math.round(ai * ad);
                int al = (int) Math.round(aj * ae);
                ah.setTileSizePixels(ac);
                DOM.setStyleAttribute(ah.getElement(), "marginLeft", (ak - ai) + "px");
                DOM.setStyleAttribute(ah.getElement(), "marginTop", (al - aj) + "px");
            }
        }

        /**
		 * Moves the tile.
		 */
        public void moveTile(TileLayerWidget tileLayerWidget, Tile tile, int offsetX, int offsetY) {
            int ab = DOM.getIntStyleAttribute(tile.getElement(), "left");
            int ac = DOM.getIntStyleAttribute(tile.getElement(), "top");
            int ad = DOM.getIntStyleAttribute(tile.getElement(), "marginLeft");
            int ae = DOM.getIntStyleAttribute(tile.getElement(), "marginTop");
            double af = tileLayerWidget.getMagnification();
            int ag = (int) Math.round(offsetX / af);
            int ah = (int) Math.round(offsetY / af);
            int ai = offsetX - ag;
            int aj = offsetY - ah;
            DOM.setStyleAttribute(tile.getElement(), "left", (ab + ag) + "px");
            DOM.setStyleAttribute(tile.getElement(), "top", (ac + ah) + "px");
            DOM.setStyleAttribute(tile.getElement(), "marginLeft", (ad + ai) + "px");
            DOM.setStyleAttribute(tile.getElement(), "marginTop", (ae + aj) + "px");
        }
    }

    /***/
    private static interface TilePanelMover {

        /**
		 * Performs any required initialization.
		 */
        void init();

        /**
		 * Performs any required initialization on the specified {@link Tile}.
		 */
        void init(Tile tile);

        /**
		 * Moves the tile panel from its old center to its new center.
		 */
        void moveTilePanel(PointView oldCenter, PointView newCenter, boolean isTemporary);
    }

    /***/
    private class DefaultTilePanelMover implements TilePanelMover {

        /**
		 * {@inheritDoc}
		 */
        public void init() {
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(Tile tile) {
        }

        /***/
        private final Point ab = new Point();

        /**
		 * {@inheritDoc}
		 */
        public void moveTilePanel(PointView oldCenter, PointView newCenter, boolean isTemporary) {
            newCenter.getDistancePoint(oldCenter, ab);
            if (k != 1.0) ab.multiplyBy(k);
            Element ac = tilePanel.getElement();
            DOM.setStyleAttribute(ac, "left", (DOM.getIntStyleAttribute(ac, "left") + ab.getX() + "px"));
            DOM.setStyleAttribute(ac, "top", (DOM.getIntStyleAttribute(ac, "top") + ab.getY() + "px"));
        }
    }

    /***/
    private class WebkitTransformTilePanelMover implements TilePanelMover {

        /**
		 * {@inheritDoc} 
		 */
        public void init() {
            DOM.setStyleAttribute(tilePanel.getElement(), "WebkitTransform", tilePanelTransform.toCss());
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(Tile tile) {
        }

        /**
		 * {@inheritDoc}
		 */
        public void moveTilePanel(PointView oldCenter, PointView newCenter, boolean isTemporary) {
            Point ab = newCenter.getDistancePoint(oldCenter, null);
            if (k != 1.0) ab.multiplyBy(k);
            double ac = tilePanelTransform.getScale();
            if (ac != 1.0) ab.multiplyBy(1 / ac);
            Element ad = tilePanel.getElement();
            Point ae = new Point(tilePanelTransform.getTranslate());
            if (isTemporary) {
                tilePanelTransform.setTranslate(ae.moveBy(ab));
                DOM.setStyleAttribute(ad, "WebkitTransform", tilePanelTransform.toCss());
            } else {
                ae.multiplyBy(ac);
                int af = (DOM.getIntStyleAttribute(ad, "left") + ab.getX() + ae.getX());
                int ag = (DOM.getIntStyleAttribute(ad, "top") + ab.getY() + ae.getY());
                DOM.setStyleAttribute(ad, "left", af + "px");
                DOM.setStyleAttribute(ad, "top", ag + "px");
                if (!ae.equals(0, 0)) {
                    ae.set(0, 0);
                    tilePanelTransform.setTranslate(ae);
                    DOM.setStyleAttribute(ad, "WebkitTransform", tilePanelTransform.toCss());
                }
            }
        }
    }

    /***/
    private class IOSWebkitTransformTilePanelMover implements TilePanelMover {

        /**
		 * {@inheritDoc} 
		 */
        public void init() {
        }

        /**
		 * {@inheritDoc}
		 */
        public void init(Tile tile) {
            DOM.setStyleAttribute(tile.getElement(), "WebkitTransform", tilePanelTransform.toCss());
        }

        /**
		 * {@inheritDoc}
		 */
        public void moveTilePanel(PointView oldCenter, PointView newCenter, boolean isTemporary) {
            Point ab = newCenter.getDistancePoint(oldCenter, null);
            if (k != 1.0) ab.multiplyBy(k);
            double ac = tilePanelTransform.getScale();
            if (ac != 1.0) ab.multiplyBy(1 / ac);
            Element ad = tilePanel.getElement();
            Point ae = new Point(tilePanelTransform.getTranslate());
            if (isTemporary) {
                tilePanelTransform.setTranslate(ae.moveBy(ab));
                String af = tilePanelTransform.toCss();
                for (Widget ag : tilePanel) ag.getElement().getStyle().setProperty("WebkitTransform", af);
            } else {
                ae.multiplyBy(ac);
                int af = (DOM.getIntStyleAttribute(ad, "left") + ab.getX() + ae.getX());
                int ag = (DOM.getIntStyleAttribute(ad, "top") + ab.getY() + ae.getY());
                DOM.setStyleAttribute(ad, "left", af + "px");
                DOM.setStyleAttribute(ad, "top", ag + "px");
                if (!ae.equals(0, 0)) {
                    ae.set(0, 0);
                    tilePanelTransform.setTranslate(ae);
                    String ah = tilePanelTransform.toCss();
                    for (Widget ai : tilePanel) ai.getElement().getStyle().setProperty("WebkitTransform", ah);
                }
            }
        }
    }

    /***/
    private static class DummyTile extends Tile {

        /**
		 * Creates a new <code>DummyTile</code> with the specified size. 
		 */
        public DummyTile(SizeView size) {
            initWidget(new SimplePanel());
            UiUtils.setSize(this, size);
            getElement().getStyle().setBackgroundColor("transparent");
        }

        /**
		 * Does nothing.
		 */
        @Override
        public void startLoading() {
        }

        /**
		 * Does nothing.
		 */
        @Override
        public void cancelLoading() {
        }

        /**
		 * {@inheritDoc}
		 */
        @Override
        public void setTileSizePixels(SizeView tileSize) {
            UiUtils.setSize(this, tileSize);
        }
    }
}
