package com.maryanovsky.map.client;

import java.util.List;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.event.GroupReadyListener;
import com.maryanovsky.gwtutils.client.event.ReadyListener;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.PilePanel;

/**
 * A panel which displays a stack of {@linkplain TileLayerWidget}s - one for
 * each layer of a {@linkplain Map}.
 * 
 * @author Maryanovsky Alexander
 */
final class TileLayersPanel extends Composite {

    /***/
    private final PilePanel a = new PilePanel();

    /***/
    private final GroupReadyListener<TileLayerWidget, TileLayersPanel> b;

    /***/
    private final Point c = new Point();

    /***/
    private int d;

    /***/
    private double e = 1;

    /***/
    private double f = -1;

    /***/
    private boolean g;

    /**
	 * Creates a new <code>TileLayersPanel</code>. The specified
	 * <code>ReadyListener</code> will be notified of the ready state of all
	 * the contained {@linkplain TileLayerWidget}s, as a group. 
	 */
    public TileLayersPanel(ReadyListener<TileLayersPanel> readyListener) {
        this.b = readyListener == null ? null : new GroupReadyListener<TileLayerWidget, TileLayersPanel>(readyListener, this);
        initWidget(h());
    }

    /***/
    private Widget h() {
        return a;
    }

    /**
	 * Sets this <code>TileLayersPanel</code> to display the specified
	 * {@linkplain TileLayer}s, centered at the specified point (in pixels) and
	 * zoom. 
	 */
    public void setMap(List<TileLayer> tileLayers, PointView center, int zoom) {
        a.clear();
        if (b != null) b.reset();
        this.c.set(center);
        this.d = zoom;
        this.e = 1;
        for (TileLayer j : tileLayers) {
            TileLayerWidget k = i(j);
            a.add(k);
        }
    }

    /***/
    private TileLayerWidget i(TileLayer j) {
        TileLayerWidget k = new TileLayerWidget(j, c, d, b);
        if (f > 0) k.setTileCoverSize(f);
        k.setLoadTilesDuringTemporaryChanges(g);
        return k;
    }

    /**
	 * Sets the center point and zoom at which we display the tile layers.
	 */
    public void setView(PointView center, int zoom) {
        this.c.set(center);
        this.d = zoom;
        for (Widget j : a) ((TileLayerWidget) j).setView(center, zoom);
    }

    /**
	 * Sets the center point at which we display the tile layers. 
	 */
    public void setCenter(PointView center, boolean isTemporary) {
        this.c.set(center);
        for (Widget j : a) ((TileLayerWidget) j).setCenter(center, isTemporary);
    }

    /**
	 * Sets the magnification of the displayed tile layers.
	 */
    public void setMagnification(double magnification, PointView fixedPoint, PointView targetPoint, int animationLength) {
        this.e = magnification;
        this.c.set(targetPoint);
        for (Widget j : a) ((TileLayerWidget) j).setMagnification(magnification, fixedPoint, targetPoint, animationLength);
    }

    /**
	 * Returns the currently displayed location, in pixels.
	 */
    public PointView getCenter() {
        return c;
    }

    /**
	 * Returns the currently displayed zoom.
	 */
    public int getZoom() {
        return d;
    }

    /**
	 * Returns the current magnification.
	 */
    public double getMagnification() {
        return e;
    }

    /**
	 * Sets the tile cover size on the tile layer widgets inside this panel.
	 * See {@link TileLayerWidget#setTileCoverSize(double)} for information. 
	 */
    public void setTileCoverSize(double tileCoverSize) {
        if (tileCoverSize <= 0) throw new IllegalArgumentException("tileCoverSize must be positive");
        for (Widget j : a) ((TileLayerWidget) j).setTileCoverSize(tileCoverSize);
        this.f = tileCoverSize;
    }

    /**
	 * Sets whether tiles are loaded during temporary changes in the map's
	 * location.
	 */
    public void setLoadTilesDuringTemporaryChanges(boolean isLoadTilesDuringTemporaryChanges) {
        for (Widget j : a) ((TileLayerWidget) j).setLoadTilesDuringTemporaryChanges(isLoadTilesDuringTemporaryChanges);
        this.g = isLoadTilesDuringTemporaryChanges;
    }

    /**
	 * Returns the number of tiles which are displayed and ready.
	 */
    public int getReadyTileCount() {
        int j = 0;
        for (Widget k : a) j += ((TileLayerWidget) k).getReadyTileCount();
        return j;
    }

    /**
	 * Cancels loading any unready tiles.
	 */
    public void cancelLoadingUnreadyTiles() {
        for (Widget j : a) ((TileLayerWidget) j).cancelLoadingUnreadyTiles();
    }

    /**
	 * Call this method whenever you manually adjust the size of the panel
	 * after it has been displayed. This is necessary because there is currently
	 * no way to detect such size changes.
	 */
    public void resized() {
        for (Widget j : a) ((TileLayerWidget) j).resized();
    }
}
