package com.maryanovsky.map.client.actions;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.map.client.MapWidget;

/**
 * A convenience base class for all map actions. The class is parameterized with
 * the type of gesture event source for the gesture the action is triggered by.
 * 
 * @author Maryanovsky Alexander
 */
public abstract class MapAction<T> {

    /**
	 * The map widget this action affects.
	 */
    protected final MapWidget mapWidget;

    /**
	 * Whether whatever transition the action applies should be animated.
	 */
    protected final boolean isAnimated;

    /**
	 * Creates a new {@link MapAction} that will act on the specified map
	 * widget. 
	 */
    public MapAction(MapWidget mapWidget) {
        this(mapWidget, false);
    }

    /**
	 * Creates a new {@link MapAction} that will act on the specified map
	 * widget with possibly animated transitions.
	 */
    public MapAction(MapWidget mapWidget, boolean isAnimated) {
        if (mapWidget == null) throw new IllegalArgumentException("mapWidget may not be null");
        this.mapWidget = mapWidget;
        this.isAnimated = isAnimated;
    }

    /**
	 * Registers this action as a handler for the gesture it is triggered by.
	 */
    public abstract HandlerRegistration registerWith(T gestureRecognizer);
}
