package com.maryanovsky.map.client.actions.mouse;

import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.MathUtils;
import com.maryanovsky.gwtutils.client.geom.*;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.drag.*;
import com.maryanovsky.map.client.MapLocationModel;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.Projection;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * A drag action which allows the user to select the area he would like to be
 * displayed on the map.
 * 
 * @author Maryanovsky Alexander
 */
public class AreaSelectionAction extends MapAction<HasAllDragHandlers> implements DragStartHandler, DragMoveHandler, DragEndHandler, DragCancelHandler {

    /***/
    private final boolean a;

    /***/
    private Rectangle b = null;

    /***/
    private Widget c = null;

    /**
	 * Creates a new {@link AreaSelectionAction} for the specified map widget,
	 * with optional ratio-keeping and animation of the transition.
	 * If <code>keepRatio</code> is <code>true</code>, the displayed selection
	 * will be forced into the same width-to-height ratio as the map widget.
	 * 
	 * @param mapWidget The target map widget.
	 * @param keepRatio Whether the width-to-height ratio of the selection
	 * rectangle should be restricted to match that of the map widget.
	 * @param isAnimated Whether the zoom-in transition is animated.
	 */
    public AreaSelectionAction(MapWidget mapWidget, boolean keepRatio, boolean isAnimated) {
        super(mapWidget, isAnimated);
        this.a = keepRatio;
        this.c = createSelectionWidget();
    }

    /**
	 * Registers for handling drag events from the specified source.
	 */
    public HandlerRegistration registerWith(HasAllDragHandlers dragSource) {
        return HandlesAllDragEvents.handle(dragSource, this);
    }

    /**
	 * Creates the widget that will display the selected area. The default
	 * implementation returns a {@link SimplePanel} with another one embedded in
	 * it. The style names are <code>map-AreaSelectionActionRectOuter</code> and
	 * <code>map-AreaSelectionActionRectInner</code>.
	 */
    protected Widget createSelectionWidget() {
        SimplePanel j = new SimplePanel();
        j.setStyleName("map-AreaSelectionActionRectOuter");
        SimplePanel k = new SimplePanel();
        k.setStyleName("map-AreaSelectionActionRectInner");
        DOM.setStyleAttribute(k.getElement(), "fontSize", "0px");
        UiUtils.addFullSize(j, k);
        return j;
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragStart(DragStartEvent evt) {
        d(evt.getLocation());
    }

    /**
	 * Invoked when the mouse button is released without creating a drag event.
	 */
    public void onDragCancel(DragCancelEvent evt) {
        i();
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragMove(DragMoveEvent evt) {
        e(evt.getLocation(), evt.getOffset());
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragEnd(DragEndEvent evt) {
        e(evt.getLocation(), evt.getOffset());
        g();
        i();
    }

    /***/
    private void d(PointView j) {
        b = new Rectangle(j, new Size(0, 0));
        UiUtils.setAbsoluteBounds(c, j.getX(), j.getY(), 0, 0);
        mapWidget.getWidgetPanel().add(c);
    }

    /***/
    private void e(PointView j, PointView k) {
        PointView l = k.getDistancePoint(j);
        if (a) {
            PointView m = f(k.getX(), k.getY());
            b.set(l.getX(), l.getY(), m.getX(), m.getY());
        } else {
            b.setCover(l, j);
        }
        UiUtils.setBounds(c, b);
    }

    /***/
    private PointView f(int j, int k) {
        SizeView l = mapWidget.getMapSize();
        if (Math.abs(l.getWidth() * k) < Math.abs(j * l.getHeight())) k = MathUtils.sgn(k * j) * (j * l.getHeight() / l.getWidth()); else j = MathUtils.sgn(j * k) * (l.getWidth() * k / l.getHeight());
        return new Point(j, k);
    }

    /***/
    private void g() {
        PointView j = b.getCenter();
        int k = h();
        mapWidget.zoomToTargetPoint(k, j, isAnimated);
    }

    /***/
    private int h() {
        SizeView j = mapWidget.getMapSize();
        PointView k = f(b.getWidth(), b.getHeight());
        double l = ((double) j.getWidth()) / k.getX();
        Projection m = mapWidget.getMap().getProjection();
        MapLocationModel n = mapWidget.getMapLocationModel();
        int o = n.getZoom();
        int p = n.getMinZoom();
        int q = n.getMaxZoom();
        for (int r = p; r <= q; r++) {
            if (m.getZoomMagnification(o, r) > l) return Math.min(q, MathUtils.max(p, r - 1, o + 1));
        }
        return q;
    }

    /***/
    private void i() {
        b = null;
        mapWidget.getWidgetPanel().remove(c);
    }
}
