package com.maryanovsky.map.client.actions.mouse;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.click.ClickEvent;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.click.ClickHandler;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.click.HasClickHandlers;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Recenters (pans) the map to the clicked location.
 * 
 * @author Maryanovsky Alexander
 */
public class ClickPanAction extends MapAction<HasClickHandlers> implements ClickHandler {

    /**
	 * Creates a new {@link ClickPanAction} for panning the specified map
	 * widget.
	 * 
	 * @param mapWidget The target map widget.
	 * @param isAnimated Whether the panning transition is animated.
	 */
    public ClickPanAction(MapWidget mapWidget, boolean isAnimated) {
        super(mapWidget, isAnimated);
    }

    /**
	 * Registers for handling click events from the specified source.
	 */
    public HandlerRegistration registerWith(HasClickHandlers clickSource) {
        return clickSource.addClickHandler(this);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onClick(ClickEvent evt) {
        mapWidget.panTo(evt.getClickLocation(), isAnimated);
    }
}
