package com.maryanovsky.map.client.actions.mouse;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.drag.*;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Pans the map as it is dragged by the mouse.
 */
public class DragPanAction extends MapAction<HasAllDragHandlers> implements DragStartHandler, DragMoveHandler, DragEndHandler, DragCancelHandler {

    /**
	 * Creates a new {@link DragPanAction} for panning the specified map widget. 
	 */
    public DragPanAction(MapWidget mapWidget) {
        super(mapWidget);
    }

    /**
	 * Registers for handling drag events from the specified source.
	 */
    public HandlerRegistration registerWith(HasAllDragHandlers dragSource) {
        return HandlesAllDragEvents.handle(dragSource, this);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragStart(DragStartEvent evt) {
        if (mapWidget.isGrabCursor()) UiUtils.setGrabbingCursor(mapWidget);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragMove(DragMoveEvent evt) {
        mapWidget.setTemporaryOffset(evt.getOffset().getOpposite());
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragEnd(DragEndEvent evt) {
        if (mapWidget.isGrabCursor()) UiUtils.setGrabCursor(mapWidget);
        mapWidget.setTemporaryOffset(evt.getOffset().getOpposite());
        mapWidget.applyTemporaryOffset();
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragCancel(DragCancelEvent evt) {
        if (mapWidget.isGrabCursor()) UiUtils.setGrabCursor(mapWidget);
    }
}
