package com.maryanovsky.map.client.actions.mouse;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.ui.mouse.gestures.wheelscroll.*;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Zooms the map in when the user scrolls the mouse-wheel up and zooms it out
 * when he scrolls the mouse-wheel down. 
 * 
 * @author Maryanovsky Alexander
 */
public class WheelScrollZoomAction extends MapAction<HasAllWheelScrollHandlers> implements WheelScrollUpHandler, WheelScrollDownHandler {

    /**
	 * Creates a new {@link WheelScrollZoomAction} for panning the specified map
	 * widget.
	 * 
	 * @param mapWidget The target map widget.
	 * @param isAnimated Whether the zoom transition is animated.
	 */
    public WheelScrollZoomAction(MapWidget mapWidget, boolean isAnimated) {
        super(mapWidget, isAnimated);
    }

    /**
	 * Registers for handling wheel-scroll events from the specified source.
	 */
    public HandlerRegistration registerWith(HasAllWheelScrollHandlers wheelScrollSource) {
        return HandlesAllWheelScrollEvents.handle(wheelScrollSource, this);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onWheelScrollUp(WheelScrollUpEvent evt) {
        mapWidget.zoomByFixPoint(1, evt.getLocation(), isAnimated);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onWheelScrollDown(WheelScrollDownEvent evt) {
        mapWidget.zoomByFixPoint(-1, evt.getLocation(), isAnimated);
    }
}
