package com.maryanovsky.map.client.actions.touch;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.event.MultipleHandlerRegistration;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.drag.*;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Pans the map as the user drags his finger across the map widget.
 * 
 * @author Maryanovsky Alexander
 */
public class DragPanAction extends MapAction<HasAllDragHandlers> implements DragMoveHandler, DragEndHandler, DragCancelHandler {

    /**
	 * Creates a new {@link DragPanAction} for the specified map widget.
	 * 
	 * @param mapWidget The target map widget.
	 */
    public DragPanAction(MapWidget mapWidget) {
        super(mapWidget);
    }

    /**
	 * Registers for handling drag events from the specified source.
	 */
    public HandlerRegistration registerWith(HasAllDragHandlers dragSource) {
        return new MultipleHandlerRegistration(dragSource.addDragMoveHandler(this), dragSource.addDragEndHandler(this), dragSource.addDragCancelHandler(this));
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragMove(DragMoveEvent evt) {
        if (UiUtils.getVisibility(mapWidget.getOverlayPanel())) UiUtils.setVisibility(mapWidget.getOverlayPanel(), false);
        mapWidget.setTemporaryOffset(evt.getOffset().getOpposite());
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragEnd(DragEndEvent evt) {
        mapWidget.setTemporaryOffset(evt.getOffset().getOpposite());
        UiUtils.setVisibility(mapWidget.getOverlayPanel(), true);
        mapWidget.applyTemporaryOffset();
    }

    /**
	 * {@inheritDoc}
	 */
    public void onDragCancel(DragCancelEvent evt) {
        mapWidget.setTemporaryOffset(new Point(0, 0));
        UiUtils.setVisibility(mapWidget.getOverlayPanel(), true);
    }
}
