package com.maryanovsky.map.client.actions.touch;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.mftap.HasMultiFingerTapHandlers;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.mftap.MultiFingerTapEvent;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.mftap.MultiFingerTapHandler;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Zooms the map out when the user taps the map widget.
 * 
 * @author Maryanovsky Alexander
 */
public class MultiFingerTapZoomOutAction extends MapAction<HasMultiFingerTapHandlers> implements MultiFingerTapHandler {

    /**
	 * Creates a new {@link MultiFingerTapZoomOutAction} for the specified map
	 * widget, and with optionally animated zoom-out transition.
	 * 
	 * @param mapWidget The target map widget.
	 * @param isAnimated Whether the zoom-out transition is animated.
	 */
    public MultiFingerTapZoomOutAction(MapWidget mapWidget, boolean isAnimated) {
        super(mapWidget, isAnimated);
    }

    /**
	 * Registers for handling multi-finger tap events from the specified source.
	 */
    public HandlerRegistration registerWith(HasMultiFingerTapHandlers tapSource) {
        return tapSource.addMultiFingerTapHandler(this);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onMultiFingerTap(MultiFingerTapEvent evt) {
        mapWidget.zoomBy(-1, true);
    }
}
