package com.maryanovsky.map.client.actions.touch;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.pinch.*;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Magnifies and zooms in/out the map as the user pinches the map widget.
 * 
 * @author Maryanovsky Alexander
 */
public class PinchMagnifyAction extends MapAction<HasAllPinchHandlers> implements PinchStartHandler, PinchChangeHandler, PinchEndHandler, PinchCancelHandler {

    /***/
    private static final double a = 0.03;

    /***/
    private final Point b = new Point();

    /**
	 * Creates a new {@link PinchMagnifyAction} for the specified map widget,
	 * and with optionally animated zoom transition.
	 * 
	 * @param mapWidget The target map widget.
	 * @param isAnimated Whether the final zoom-adjusting transition is
	 * animated.
	 */
    public PinchMagnifyAction(MapWidget mapWidget, boolean isAnimated) {
        super(mapWidget, isAnimated);
    }

    /**
	 * Registers for handling pinch events from the specified source.
	 */
    public HandlerRegistration registerWith(HasAllPinchHandlers pinchSource) {
        return HandlesAllPinchEvents.handle(pinchSource, this);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onPinchStart(PinchStartEvent evt) {
        PointView c = evt.getFirstFingerLocation();
        PointView d = evt.getSecondFingerLocation();
        b.set((c.getX() + d.getX()) / 2, (c.getY() + d.getY()) / 2);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onPinchChange(PinchChangeEvent evt) {
        MapWidget c = (MapWidget) evt.getSource();
        double d = evt.getScale();
        if (Math.abs(d - c.getMagnification().getFirst()) > a) {
            if (UiUtils.getVisibility(c.getOverlayPanel())) UiUtils.setVisibility(c.getOverlayPanel(), false);
            c.setMagnification(d, b);
        }
    }

    /**
	 * {@inheritDoc}
	 */
    public void onPinchEnd(PinchEndEvent evt) {
        MapWidget c = (MapWidget) evt.getSource();
        if (c.getMagnification().getFirst() != 1.0) {
            if (!isAnimated) UiUtils.setVisibility(c.getOverlayPanel(), true);
            double d = c.getMagnification().getFirst();
            c.applyMagnification(isAnimated, d > 1 ? 1 : -1);
        }
    }

    /**
	 * {@inheritDoc}
	 */
    public void onPinchCancel(PinchCancelEvent evt) {
        MapWidget c = (MapWidget) evt.getSource();
        UiUtils.setVisibility(c.getOverlayPanel(), true);
    }
}
