package com.maryanovsky.map.client.actions.touch;

import com.google.gwt.event.shared.HandlerRegistration;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.tap.HasTapHandlers;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.tap.TapEvent;
import com.maryanovsky.gwtutils.client.ui.touch.gestures.tap.TapHandler;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.actions.MapAction;

/**
 * Zooms the map in when the user taps the map widget.
 * 
 * @author Maryanovsky Alexander
 */
public class TapZoomInAction extends MapAction<HasTapHandlers> implements TapHandler {

    /**
	 * Creates a new {@link TapZoomInAction} for the specified map widget, and
	 * with optionally animated zoom-in transition.
	 * 
	 * @param mapWidget The target map widget.
	 * @param isAnimated Whether the zoom-in transition is animated.
	 */
    public TapZoomInAction(MapWidget mapWidget, boolean isAnimated) {
        super(mapWidget, isAnimated);
    }

    /**
	 * Registers for handling tap events from the specified source.
	 */
    public HandlerRegistration registerWith(HasTapHandlers tapSource) {
        return tapSource.addTapHandler(this);
    }

    /**
	 * {@inheritDoc}
	 */
    public void onTap(TapEvent evt) {
        mapWidget.zoomByTargetPoint(1, evt.getTapLocation(), true);
    }
}
