package com.maryanovsky.map.client.osm;

import com.maryanovsky.gwtutils.client.MathUtils;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.map.client.ImageTileLayer;

/**
 * Implements an OpenStreetMaps.org tile layer.
 * 
 * @author Maryanovsky Alexander
 */
public class OsmTileLayer extends ImageTileLayer {

    /***/
    private final int a;

    /***/
    private final int b;

    /**
	 * Creates a new {@link OsmTileLayer} with the specified base URL pattern,
	 * tile files' extension, and minimum and maximum zoom values. The base URL
	 * (and, actually, the extension as well) is treated as part of a URL
	 * pattern as described in {@link ImageTileLayer#ImageTileLayer(com.maryanovsky.gwtutils.client.geom.SizeView, String)}.
	 * You can use that to specify multiple domains from which to retrieve the
	 * tile image.
	 */
    public OsmTileLayer(String baseUrl, String tileExtension, int minZoom, int maxZoom) {
        super(new Size(256, 256), baseUrl + (baseUrl.endsWith("/") ? "" : "/") + "{zoom}/{x}/{y}." + tileExtension);
        if (tileExtension == null) throw new IllegalArgumentException("tileExtension may not be null");
        if (minZoom > maxZoom) throw new IllegalArgumentException("minZoom may not be greater than maxZoom");
        this.a = minZoom;
        this.b = maxZoom;
    }

    /**
	 * Creates a new {@link OsmTileLayer} with the specified base URL and
	 * minimum and maximum zoom values.
	 */
    public OsmTileLayer(String baseUrl, int minZoom, int maxZoom) {
        this(baseUrl, "png", minZoom, maxZoom);
    }

    /**
	 * {@inheritDoc}
	 */
    public String getTileUrl(int xIndex, int yIndex, int zoom) {
        if ((zoom < a) || (zoom > b)) return null;
        int c = MathUtils.getPowerOfTwo(zoom);
        if ((yIndex < 0) || (yIndex >= c)) return null;
        xIndex = xIndex % c;
        if (xIndex < 0) xIndex += c;
        return super.getTileUrl(xIndex, yIndex, zoom);
    }
}
