package com.maryanovsky.map.client.projections;

import com.maryanovsky.gwtutils.client.MathUtils;

/**
 * An implementation of {@link ZoomStrategy} which specifies a
 * fixed zoom coefficient (scale multiplier) between each two neighbouring
 * zooms.
 * 
 * @author Maryanovsky Alexander
 */
public final class FixedCoefZoomStrategy implements ZoomStrategy {

    /***/
    private static final ZoomStrategy a = new ZoomStrategy() {

        public double getZoomMagnification(int startZoom, int endZoom) {
            int c = Math.abs(endZoom - startZoom);
            double d = (c <= 31) ? MathUtils.getPowerOfTwo(c) : Math.pow(2, c);
            return (endZoom < startZoom) ? 1.0 / d : d;
        }
    };

    /***/
    private final double b;

    /**
	 * Creates a new {@link FixedCoefZoomStrategy} with the specified zoom
	 * coefficient between neighbouring zooms (a value larger than 1).
	 */
    private FixedCoefZoomStrategy(double c) {
        if (c <= 1) throw new IllegalArgumentException("zoomCoefficient must be larger than 1");
        this.b = c;
    }

    /**
	 * Returns a {@link ZoomStrategy} with the specified zoom coefficient
	 * between neighbouring zooms (a value larger than 1).
	 */
    public static ZoomStrategy getInstance(double zoomCoefficient) {
        if (zoomCoefficient == 2) return a; else return new FixedCoefZoomStrategy(zoomCoefficient);
    }

    /**
	 * {@inheritDoc}
	 */
    public double getZoomMagnification(int startZoom, int endZoom) {
        return Math.pow(b, endZoom - startZoom);
    }
}
