package com.maryanovsky.map.client.projections;

import java.util.ArrayList;
import java.util.List;

/**
 * An implementation of {@link ZoomStrategy} where each zoom has a resolution
 * value attached to it - the number of map units (meters, degrees etc.) per
 * pixel.
 * 
 * @author Maryanovsky Alexander
 */
public class ResolutionListZoomStrategy implements ZoomStrategy {

    /***/
    private final List<Double> a;

    /***/
    private final int b;

    /***/
    private final int c;

    /**
	 * Creates a new {@link ResolutionListZoomStrategy} with the specified list
	 * of resolutions (one for each zoom). The first value in the list
	 * corresponds to zoom 0 and so on.
	 */
    public ResolutionListZoomStrategy(List<Double> resolutions) {
        if (resolutions == null) throw new IllegalArgumentException("resolutions may not be null");
        if (resolutions.size() == 0) throw new IllegalArgumentException("resolutions may not be an empty list");
        this.a = new ArrayList<Double>(resolutions.size());
        for (Double d : resolutions) this.a.add(d);
        this.b = 0;
        this.c = resolutions.size() - 1;
    }

    /**
	 * {@inheritDoc}
	 */
    public double getZoomMagnification(int startZoom, int endZoom) {
        if ((startZoom < b) || (startZoom > c)) throw new IllegalArgumentException("startZoom (" + startZoom + ") out of range [" + b + "," + c + "]");
        if ((endZoom < b) || (endZoom > c)) throw new IllegalArgumentException("endZoom (" + endZoom + ") out of range [" + b + "," + c + "]");
        double d = a.get(startZoom);
        double e = a.get(endZoom);
        return d / e;
    }
}
