package com.maryanovsky.map.client.tiles;

import com.maryanovsky.gwtutils.client.ui.ImageLoadListener;
import com.maryanovsky.gwtutils.client.ui.ImageLoadingManager;
import com.maryanovsky.gwtutils.client.ui.UiUtils;

/**
 * A {@linkplain ImageTile} implementation for Internet Explorer.
 */
final class IEImageTile extends ImageTile {

    /***/
    private final ImageLoadListener a = new ImageLoadListener() {

        public void imageLoaded(String url, int width, int height) {
            if (!url.equals(IEImageTile.this.url)) return;
            image.setUrl(url);
            if (tileSize == null) UiUtils.setSize(image, width, height); else UiUtils.setSize(image, tileSize);
            markReady();
        }

        public void imageFailed(String url) {
            if (!url.equals(IEImageTile.this.url)) return;
            markReady();
        }
    };

    /**
	 * Creates a new <code>IETile</code> with the specified image url.
	 */
    public IEImageTile(String url) {
        super(url);
        setImageSize("0px");
    }

    /**
	 * {@inheritDoc}
	 */
    public void startLoading() {
        if (url == null) {
            markUnready();
            markReady();
        } else {
            markUnready();
            ImageLoadingManager.load(url, a);
        }
    }

    /**
	 * {@inheritDoc}
	 */
    public void cancelLoading() {
        if (url != null) ImageLoadingManager.cancel(url, a);
    }
}
