package com.maryanovsky.map.client.tiles;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.BrowserUtils;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.gwtutils.client.geom.SizeView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.map.client.Tile;

/**
 * A {@link Tile} implementation which uses an image from a specified URL.
 * 
 * @author Maryanovsky Alexander
 */
public abstract class ImageTile extends Tile {

    /**
	 * The <code>Image</code> widget we're using to display the tile image.
	 */
    protected final Image image = new Image();

    /**
	 * The URL of the image of the tile.
	 */
    protected final String url;

    /**
	 * The currently requested size of the tile; <code>null</code> if automatic.
	 */
    protected SizeView tileSize = null;

    /**
	 * Creates a new <code>Tile</code> with the specified image url.
	 */
    protected ImageTile(String url) {
        if (url == null) throw new IllegalArgumentException("url may not be null");
        this.url = url;
        initWidget(a());
    }

    /**
	 * Creates a new <code>Tile</code> which will display the image at the
	 * specified URL.
	 */
    public static ImageTile makeImageTile(String url) {
        return BrowserUtils.isIE() && (BrowserUtils.getIEVersion().compareTo("8") < 0) ? new IEImageTile(url) : new DefaultImageTile(url);
    }

    /***/
    private Widget a() {
        Element b = image.getElement();
        DOM.setStyleAttribute(b, "position", "absolute");
        DOM.setStyleAttribute(b, "borderWidth", "0px");
        DOM.setStyleAttribute(b, "margin", "0px");
        DOM.setStyleAttribute(b, "padding", "0px");
        return image;
    }

    /**
	 * Sets the image's width and height style attributes to the specified
	 * value. 
	 */
    protected void setImageSize(String size) {
        DOM.setStyleAttribute(image.getElement(), "width", size);
        DOM.setStyleAttribute(image.getElement(), "height", size);
    }

    /**
	 * Sets the size of the tile image, in pixels. A <code>null</code> value
	 * resets it to the default size (the native size of the image at the URL).
	 */
    public void setTileSizePixels(SizeView tileSize) {
        this.tileSize = tileSize == null ? null : new Size(tileSize);
        if (isAttached() && isReady()) {
            if (tileSize == null) setImageSize("auto"); else UiUtils.setSize(image, tileSize);
        }
    }

    /**
	 * Invoked when the tile is detached from the DOM.
	 */
    protected void onUnload() {
        super.onUnload();
        if (!isReady()) cancelLoading();
    }
}
