package com.maryanovsky.map.client.tiles;

import java.util.LinkedList;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Node;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.dom.client.Style.Unit;
import com.maryanovsky.gwtutils.client.event.Callback;
import com.maryanovsky.gwtutils.client.event.GroupReadyListener;
import com.maryanovsky.gwtutils.client.event.ReadyListener;
import com.maryanovsky.gwtutils.client.geom.SizeView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;

/**
 * A {@link NodeTile} where the node contains no dynamic content other than
 * images loaded via an IMG element.
 * 
 * @author Maryanovsky Alexander
 */
public class ImagesNodeTile extends NodeTile {

    /***/
    private final GroupReadyListener<Element, ImagesNodeTile> a = new GroupReadyListener<Element, ImagesNodeTile>(new ReadyListener<ImagesNodeTile>() {

        public void becameReady(ImagesNodeTile source) {
            markReady();
        }

        public void becameUnready(ImagesNodeTile source) {
        }
    }, this);

    /**
	 * Creates a new {@link ImagesNodeTile} with the specified content node and
	 * tile size.
	 */
    public ImagesNodeTile(Node node, SizeView tileSize) {
        super(node, tileSize);
        b(getNode());
    }

    /***/
    private void b(Node c) {
        if (c.getNodeType() != Node.ELEMENT_NODE) return;
        Element d = Element.as(c);
        if ("img".equalsIgnoreCase(d.getTagName())) {
            a.becameUnready(d);
            UiUtils.addLoadListener(d, new Callback<Element>() {

                public void callback(Element element) {
                    if (isReallyReady(element)) {
                        a.becameReady(element);
                    }
                }
            });
        }
        NodeList<Node> e = d.getChildNodes();
        for (int f = 0; f < e.getLength(); f++) b(e.getItem(f));
    }

    /**
	 * Invoked when a child image element sends a load event. Returns whether
	 * the element is really loaded. This is needed because in some cases
	 * (Google Maps, for example) an image element in a tile may load several
	 * images before becoming fully ready. The default implementation always
	 * returns <code>true</code>. 
	 */
    protected boolean isReallyReady(Element imageElement) {
        return true;
    }

    /**
	 * Resize the tile by setting width and height attributes on all the images.
	 */
    public void setTileSizePixels(SizeView tileSize) {
        LinkedList<Node> c = new LinkedList<Node>();
        c.add(getElement());
        while (!c.isEmpty()) {
            Node d = c.removeFirst();
            if (d.getNodeType() != Node.ELEMENT_NODE) continue;
            Element e = Element.as(d);
            if ("img".equalsIgnoreCase(e.getTagName())) {
                e.getStyle().setWidth(tileSize.getWidth(), Unit.PX);
                e.getStyle().setHeight(tileSize.getHeight(), Unit.PX);
            }
            NodeList<Node> f = e.getChildNodes();
            for (int g = 0; g < f.getLength(); g++) c.addLast(f.getItem(g));
        }
    }
}
