package com.maryanovsky.map.client.tiles;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Node;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.gwtutils.client.geom.SizeView;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.map.client.Tile;

/**
 * A {@link Tile} implementation which uses a specified DOM {@link Node} as
 * the tile. Subclasses are left with implementing tile readiness notification
 * (invoking {@link Tile#markReady()}) which depends on the actual type of
 * content in the tile. Note that you only need to invoke {@link #markReady()},
 * not {@link #markUnready()}.
 * 
 * @author Maryanovsky Alexander
 */
public abstract class NodeTile extends Tile {

    /***/
    private final Node a;

    /***/
    private final SizeView b;

    /**
	 * Creates a new {@link NodeTile} with the specified {@link Node} and
	 * normal tile size.
	 */
    public NodeTile(Node node, SizeView tileSize) {
        if (node == null) throw new IllegalArgumentException("node may not be null");
        if (tileSize == null) throw new IllegalArgumentException("tileSize may not be null");
        this.a = node;
        this.b = new Size(tileSize);
        initWidget(new ElementWidget(DOM.createDiv()));
    }

    /**
	 * Returns the node.
	 */
    public Node getNode() {
        return a;
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    public void startLoading() {
        markUnready();
        getElement().appendChild(a);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    public void cancelLoading() {
        getElement().removeChild(a);
    }

    /**
	 * {@inheritDoc}
	 */
    @Override
    public void setTileSizePixels(SizeView size) {
        double c = ((double) size.getWidth()) / b.getWidth();
        double d = ((double) size.getHeight()) / b.getHeight();
        UiUtils.setTransformOrigin(getElement(), "0 0");
        UiUtils.setTransform(getElement(), "scale(" + c + "," + d + ")");
    }

    /***/
    private static class ElementWidget extends Widget {

        /**
		 * Creates a new {@link ElementWidget} from the specified element.
		 */
        public ElementWidget(Element element) {
            setElement(element);
        }
    }
}
