package com.maryanovsky.map.client.widgets.bubble;

import com.google.gwt.core.client.GWT;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.ui.Alignment;
import com.maryanovsky.map.client.LatLng;
import com.maryanovsky.map.client.MapWidget;
import com.maryanovsky.map.client.Overlay;
import com.maryanovsky.map.client.WidgetOverlay;

/**
 * A "bubble", or "tooltip" widget. 
 * The style name of the widget is <code>map-BubbleWidget</code>. The style name
 * of the content holder element is <code>map-BubbleWidgetContentHolder</code>. 
 * The style name of the bubble's "leg" is <code>map-BubbleWidgetLeg</code>.
 * 
 * @author Maryanovsky Alexander
 */
public class BubbleWidget extends Composite implements Overlay {

    /***/
    private final SimplePanel a = new SimplePanel();

    /***/
    private final Image b;

    /***/
    private MapWidget c = null;

    /***/
    private final WidgetOverlay d;

    /**
	 * Creates a new {@link BubbleWidget} with the specified images, alignment
	 * relative to the anchor point and content (the latter may be
	 * <code>null</code>).
	 */
    public BubbleWidget(Images images, Alignment alignment, Widget content) {
        if (images == null) throw new IllegalArgumentException("images may not be null");
        this.b = new Image(images.leg());
        initWidget(e());
        this.d = new WidgetOverlay(this, new LatLng(0, 0), new Point(0, 0));
        d.setAlignment(alignment);
        setContent(content);
        setStyleName("map-BubbleWidget");
        a.setStyleName("map-BubbleWidgetContentHolder");
        b.setStyleName("map-BubbleWidgetLeg");
    }

    /**
	 * Creates a new {@link BubbleWidget} with the specified content (may be
	 * <code>null</code>). 
	 */
    public BubbleWidget(Widget content) {
        this(GWT.<Images>create(BubbleWidget.Images.class), Alignment.TOP_RIGHT, content);
    }

    /**
	 * Creates a new Bubble, with initially no content.
	 */
    public BubbleWidget() {
        this(null);
    }

    /***/
    private Widget e() {
        VerticalPanel f = new VerticalPanel();
        f.add(createContentHolderWrapper(a));
        f.add(b);
        return f;
    }

    /**
	 * This method creates a wrapper panel for the content holder. This allows
	 * customizing the look of the bubble in ways that are difficult or
	 * impossible to achieve with just CSS. The method is responsible for adding
	 * the content holder into the wrapper. The default implementation simply
	 * returns the content holder itself.  
	 */
    protected Widget createContentHolderWrapper(Widget contentHolder) {
        return contentHolder;
    }

    /**
	 * Sets the content widget. Pass <code>null</code> to clear the content.
	 */
    public void setContent(Widget content) {
        a.setWidget(content);
        if (c != null) d.updateWidgetSize(null);
    }

    /**
	 * Sets the anchor coordinate - the one to which the bubble points.
	 */
    public void setAnchor(LatLng anchor) {
        d.setAnchor(anchor);
    }

    /**
	 * Adds the bubble to the map's overlay panel.
	 */
    public void added(MapWidget mapWidget) {
        this.c = mapWidget;
        d.added(mapWidget);
    }

    /**
	 * Called to update the overlay's position.
	 */
    public void updated(MapWidget mapWidget, boolean isTemporary) {
        d.updated(mapWidget, isTemporary);
    }

    /**
	 * Removes the widget from the map's overlay panel.
	 */
    public void removed(MapWidget mapWidget) {
        d.removed(mapWidget);
        this.c = null;
    }

    /**
	 * A bundle of the images used by the widget.
	 */
    public static interface Images extends ClientBundle {

        /**
		 * The "leg" of the bubble.
		 */
        @Source("leg.png")
        ImageResource leg();
    }
}
