package com.maryanovsky.map.client.widgets.scale;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.LoadEvent;
import com.google.gwt.event.dom.client.LoadHandler;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.event.Callback;
import com.maryanovsky.gwtutils.client.event.ChangeEvent;
import com.maryanovsky.gwtutils.client.event.ChangeListener;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.map.client.MapWidget;

/**
 * A widget displaying the map's scale, that is the amount of real-world
 * distance units (such as kilometers) in a certain amount of pixels.
 * The style name of the widget is <code>map-ScaleWidget</code>. The style name
 * of the element that displays the scale size (the ruler) is
 * <code>map-ScaleWidgetSize</code>. The style name of the label that displays
 * the size label is <code>map-ScaleWidgetText</code>. The actual size label
 * is implemented using two labels - a background and a foreground one (offset
 * by one pixel to the right and to the bottom). Their style names are
 * <code>map-ScaleWidgetBackgroundLabel</code> and
 * <code>map-ScaleWidgetForegroundLabel</code>.
 * In order to properly use this widget's default look, you need to import the
 * "widgets/scale/scale.css" file into the page's CSS.
 *
 * @author Maryanovsky Alexander
 */
public class ScaleWidget extends Composite {

    /***/
    private final MapWidget a;

    /***/
    private final Scale b;

    /***/
    private final Images c;

    /***/
    private final Label d = new Label("");

    /***/
    private final Label e = new Label("");

    /***/
    private final AbsolutePanel f = new AbsolutePanel();

    /***/
    private final AbsolutePanel g = new AbsolutePanel();

    /***/
    private final AbsolutePanel h = new AbsolutePanel();

    /***/
    private final Widget i;

    /***/
    private final Widget j;

    /***/
    private final AbsolutePanel k = new AbsolutePanel();

    /***/
    private Size l = null;

    /***/
    private Size m = null;

    /**
	 * Creates a new {@link ScaleWidget} for the specified {@link MapWidget},
	 * with the specified scale and images.
	 */
    public ScaleWidget(MapWidget mapWidget, Scale scale, Images images) {
        if (mapWidget == null) throw new IllegalArgumentException("mapWidget may not be null");
        if (scale == null) throw new IllegalArgumentException("scale may not be null");
        if (images == null) throw new IllegalArgumentException("images may not be null");
        this.a = mapWidget;
        this.b = scale;
        this.c = images;
        this.i = new Image(images.left());
        this.j = new Image(images.right());
        mapWidget.getMapLocationModel().addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent evt) {
                p();
            }
        });
        n(new Image(images.left()), new Callback<Size>() {

            public void callback(Size size) {
                l = size;
            }
        });
        n(new Image(images.middle()), new Callback<Size>() {

            public void callback(Size size) {
                m = size;
            }
        });
        initWidget(o());
        g.setStyleName("map-ScaleWidgetSize");
        h.setStyleName("map-ScaleWidgetText");
        d.setStyleName("map-ScaleWidgetForegroundLabel");
        e.setStyleName("map-ScaleWidgetBackgroundLabel");
        setStyleName("map-ScaleWidget");
    }

    /**
	 * Creates a new {@link ScaleWidget} for the specified {@link MapWidget},
	 * with the specified scale and default images.
	 */
    public ScaleWidget(MapWidget mapWidget, Scale scale) {
        this(mapWidget, scale, GWT.<Images>create(ScaleWidget.Images.class));
    }

    /***/
    private void n(final Image q, final Callback<Size> r) {
        int s = q.getWidth();
        int t = q.getHeight();
        if ((s > 0) && (t > 0)) r.callback(new Size(s, t)); else {
            q.addLoadHandler(new LoadHandler() {

                public void onLoad(LoadEvent evt) {
                    g.remove(q);
                    r.callback(new Size(q.getWidth(), q.getHeight()));
                }
            });
            g.add(q, 0, 0);
        }
    }

    /***/
    private Widget o() {
        g.add(i, 0, 0);
        g.add(j, 0, 0);
        g.add(k, 0, 0);
        DOM.setStyleAttribute(k.getElement(), "overflow", "hidden");
        h.add(e);
        h.add(d);
        DOM.setStyleAttribute(g.getElement(), "position", "");
        DOM.setStyleAttribute(h.getElement(), "position", "");
        f.add(g);
        f.add(h);
        return f;
    }

    /***/
    private void p() {
        if ((m == null) || (l == null)) return;
        int q = l.getWidth();
        int r = m.getWidth();
        int s = b.getSizePixels(a);
        g.setVisible(s != 0);
        if (s != 0) {
            while (k.getWidgetCount() * r < s) k.add(new Image(c.middle()), r * k.getWidgetCount(), 0);
            g.setWidgetPosition(k, q, 0);
            k.setPixelSize(s, m.getHeight());
            g.setWidgetPosition(j, s + q, 0);
            UiUtils.setContainingSize(g);
        }
        String t = b.getSizeText(a);
        h.setVisible(t != null);
        if (t != null) {
            d.setText(t);
            e.setText(t);
            UiUtils.setContainingSize(h);
        }
        UiUtils.setContainingSize(f);
    }

    /**
	 * Updates the scale on-load.
	 */
    protected void onLoad() {
        super.onLoad();
        p();
    }

    /**
	 * Defines the size and values displayed by the scale widget.
	 */
    public static interface Scale {

        /**
		 * Returns the size of the displayed scale, in pixels, or <code>0</code>
		 * if the scale should not be displayed.
		 */
        int getSizePixels(MapWidget mapWidget);

        /**
		 * Returns the value displayed on the scale widget, or <code>null</code>
		 * if the scale value should not be displayed. 
		 */
        String getSizeText(MapWidget mapWidget);
    }

    /**
	 * A bundle of the images used by the widget.
	 */
    public static interface Images extends ClientBundle {

        /**
		 * The left part of the scale.
		 */
        @Source("left.png")
        ImageResource left();

        /**
		 * The right part of the scale.
		 */
        @Source("right.png")
        ImageResource right();

        /**
		 * The middle, repeating, part of the scale. 
		 */
        @Source("middle.png")
        ImageResource middle();
    }
}
