package com.maryanovsky.map.client.widgets.zoom;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.*;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.event.ChangeEvent;
import com.maryanovsky.gwtutils.client.event.ChangeListener;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.PointView;
import com.maryanovsky.gwtutils.client.ui.EventCancellingPanel;
import com.maryanovsky.gwtutils.client.ui.ImageButton;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.map.client.MapLocationModel;
import com.maryanovsky.map.client.MapWidget;

/**
 * A large zoom widget, with both zoom in and zoom out buttons and a "ladder"
 * of zooms where the user can select and see the current zoom.
 * The primary style name of this widget is <code>"map-LargeZoomWidget"</code>.
 * A dependent style name of <code>"disabled"</code> is added when the widget
 * is disabled. The primary style name of the zoom-in button is
 * <code>"map-ZoomInButton"</code>. The primary style name of the zoom-out
 * button is <code>"map-ZoomOutButton"</code>. Both buttons are
 * {@link ImageButton}s. The style name of the ladder is
 * <code>"map-LargeZoomWidgetLadder"</code> and the style name of the mark is
 * <code>"map-LargeZoomWidgetMark"</code>.
 * In order to properly use this widget's default look, you need to import the
 * "widgets/zoom/zoom.css" file into the page's CSS.
 * 
 * @author Maryanovsky Alexander
 */
public class LargeZoomWidget extends Composite {

    /***/
    private static final int a = 300;

    /***/
    private final MapWidget b;

    /***/
    private final Images c;

    /***/
    private final ImageButton d;

    /***/
    private final ImageButton e;

    /***/
    private final AbsolutePanel f;

    /***/
    private final Widget g;

    /***/
    private final FocusPanel h;

    /***/
    private boolean i = false;

    /***/
    private boolean j;

    /***/
    private boolean k = false;

    /***/
    private int l;

    /***/
    private Timer m = new Timer() {

        public void run() {
            b.zoomTo(l, isAnimate());
        }
    };

    /**
	 * Creates a new {@link LargeZoomWidget} for the specified
	 * {@link MapWidget} and specifying the source of images for this widget.
	 */
    public LargeZoomWidget(MapWidget mapWidget, Images images) {
        if (mapWidget == null) throw new IllegalArgumentException("mapWidget may not be null");
        if (images == null) throw new IllegalArgumentException("images may not be null");
        this.b = mapWidget;
        this.c = images;
        this.d = new ImageButton(this.c.zoomIn());
        this.e = new ImageButton(this.c.zoomOut());
        this.g = new Image(this.c.mark());
        this.f = n();
        this.h = new FocusPanel(f);
        setAnimate(false);
        mapWidget.getMapLocationModel().addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent evt) {
                r();
            }
        });
        d.addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent evt) {
                s(1);
            }
        });
        e.addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent evt) {
                s(-1);
            }
        });
        h.addMouseDownHandler(new MouseDownHandler() {

            public void onMouseDown(MouseDownEvent evt) {
                if (!isEnabled()) return;
                k = true;
                t(evt.getX() - g.getOffsetWidth() / 2, evt.getY() - g.getOffsetHeight() / 2);
                DOM.setCapture(h.getElement());
            }
        });
        h.addMouseUpHandler(new MouseUpHandler() {

            public void onMouseUp(MouseUpEvent evt) {
                if (!isEnabled()) return;
                k = false;
                PointView u = o(l);
                t(u.getX(), u.getY());
                DOM.releaseCapture(h.getElement());
            }
        });
        h.addMouseMoveHandler(new MouseMoveHandler() {

            public void onMouseMove(MouseMoveEvent evt) {
                if (!isEnabled()) return;
                if (k) t(evt.getX() - g.getOffsetWidth() / 2, evt.getY() - g.getOffsetHeight() / 2);
            }
        });
        initWidget(new EventCancellingPanel(q(), Event.MOUSEEVENTS, Event.MOUSEEVENTS));
        d.setStylePrimaryName("map-ZoomInButton");
        e.setStylePrimaryName("map-ZoomOutButton");
        h.setStyleName("map-LargeZoomWidgetLadder");
        g.setStyleName("map-LargeZoomWidgetMark");
        setStyleName("map-LargeZoomWidget");
        setEnabled(true);
        LoadHandler u = new LoadHandler() {

            public void onLoad(LoadEvent evt) {
                r();
            }
        };
        ErrorHandler v = new ErrorHandler() {

            public void onError(ErrorEvent evt) {
                r();
            }
        };
        Image w = new Image(this.c.top());
        Image x = new Image(this.c.step());
        Image y = new Image(this.c.bottom());
        w.addLoadHandler(u);
        x.addLoadHandler(u);
        y.addLoadHandler(u);
        w.addErrorHandler(v);
        x.addErrorHandler(v);
        y.addErrorHandler(v);
    }

    /**
	 * Creates a new {@link LargeZoomWidget} for the specified
	 * {@link MapWidget} using the default images.
	 */
    public LargeZoomWidget(MapWidget mapWidget) {
        this(mapWidget, GWT.<Images>create(LargeZoomWidget.Images.class));
    }

    /***/
    private AbsolutePanel n() {
        VerticalPanel u = new VerticalPanel();
        MapLocationModel v = b.getMapLocationModel();
        u.add(new Image(c.top()));
        for (int w = v.getMaxZoom(); w >= v.getMinZoom(); w--) u.add(new Image(c.step()));
        u.add(new Image(c.bottom()));
        UiUtils.extraImageCellHeightWorkaround(u);
        AbsolutePanel w = new AbsolutePanel();
        w.add(u);
        w.add(g, 0, 0);
        return w;
    }

    /***/
    private PointView o(int u) {
        VerticalPanel v = (VerticalPanel) f.getWidget(0);
        int w = b.getMapLocationModel().getMaxZoom();
        Widget x = v.getWidget(w - u + 1);
        return new Point(0, x.getAbsoluteTop() - v.getAbsoluteTop());
    }

    /***/
    private int p(PointView u) {
        VerticalPanel v = (VerticalPanel) f.getWidget(0);
        MapLocationModel w = b.getMapLocationModel();
        int x = w.getMaxZoom();
        int y = w.getMinZoom();
        Widget z = v.getWidget(1);
        int aa = z.getAbsoluteTop() - v.getAbsoluteTop();
        int ab = z.getOffsetHeight();
        int ac = x - (u.getY() - aa + ab / 2) / ab;
        return Math.min(x, Math.max(y, ac));
    }

    /***/
    private Widget q() {
        VerticalPanel u = new VerticalPanel();
        u.add(d);
        u.add(h);
        u.add(e);
        UiUtils.extraImageCellHeightWorkaround(u);
        return u;
    }

    /**
	 * Sets whether zoom changes will (try to) be animated.
	 */
    public void setAnimate(boolean isAnimate) {
        this.i = isAnimate;
    }

    /**
	 * Returns whether we attempt to animate zoom changes. 
	 */
    public boolean isAnimate() {
        return i;
    }

    /**
	 * Sets whether the widget is enabled. A disabled widget will show the 
	 * current zoom, but will not allow the user to change it.
	 */
    public void setEnabled(boolean isEnabled) {
        this.j = isEnabled;
        d.setEnabled(isEnabled);
        e.setEnabled(isEnabled);
        if (isEnabled) removeStyleDependentName("disabled"); else addStyleDependentName("disabled");
    }

    /**
	 * Returns whether the widget is enabled.
	 */
    public boolean isEnabled() {
        return j;
    }

    /***/
    private void r() {
        int u = b.getMapLocationModel().getZoom();
        l = u;
        if (k) return;
        PointView v = o(u);
        f.setWidgetPosition(g, v.getX(), v.getY());
    }

    /***/
    private void s(int u) {
        l = b.getMapLocationModel().clampToLegal(l + u);
        b.zoomTo(l, isAnimate());
    }

    /***/
    private void t(int u, int v) {
        u = 0;
        v = Math.min(Math.max(v, 0), f.getOffsetHeight() - g.getOffsetHeight());
        f.setWidgetPosition(g, u, v);
        int w = p(new Point(u, v));
        if (w != l) {
            l = w;
            m.cancel();
            m.schedule(a);
        }
    }

    /**
	 * Updates the widget, because only after being attached we know our real
	 * size and can position the mark.
	 */
    protected void onAttach() {
        super.onAttach();
        r();
    }

    /**
	 * A bundle of images used by the widget.
	 */
    public static interface Images extends ClientBundle {

        /**
		 * Returns the image resource of the zoom-in button.
		 */
        @Source("zoomIn.png")
        ImageResource zoomIn();

        /**
		 * Returns the image resource of the zoom-out button.
		 */
        @Source("zoomOut.png")
        ImageResource zoomOut();

        /**
		 * Returns the image resource of the ladder's top.
		 */
        @Source("top.png")
        ImageResource top();

        /**
		 * Returns the image resource of the ladder's bottom.
		 */
        @Source("bottom.png")
        ImageResource bottom();

        /**
		 * Returns the image resource of the ladder's steps.
		 */
        @Source("step.png")
        ImageResource step();

        /**
		 * Returns the image resource of the ladder's mark.
		 */
        @Source("mark.png")
        ImageResource mark();
    }
}
