package com.maryanovsky.mapdemo.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.gwtutils.client.ui.Alignment;
import com.maryanovsky.gwtutils.client.ui.EventCancellingPanel;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.map.client.*;
import com.maryanovsky.map.client.actions.StandardActions;
import com.maryanovsky.map.client.actions.mouse.StandardMouseActions;
import com.maryanovsky.map.client.widgets.overview.OverviewMapWidget;
import com.maryanovsky.map.client.widgets.scale.EarthScale;
import com.maryanovsky.map.client.widgets.scale.ScaleWidget;
import com.maryanovsky.map.client.widgets.zoom.LargeZoomWidget;

/**
 * The base class for various map demos.
 * 
 * @author Maryanovsky Alexander
 */
public abstract class MapDemo implements EntryPoint {

    /**
	 * Just a textbox, for demo purposes.
	 */
    protected TextBox textBox;

    /**
	 * Starts the map demo.
	 */
    public void onModuleLoad() {
        start();
    }

    /**
	 * Actually starts the map demo.
	 */
    protected void start() {
        Map a = createMap();
        MapLocationModel b = createMapLocationModel();
        MapWidget c = new MapWidget(b);
        c.setMap(a);
        configureMapWidget(c);
        Widget d = createCopyrightNotice();
        Panel e = RootPanel.get("copyright");
        if ((d != null) && (e != null)) e.add(d);
        UiUtils.addFullSize(RootPanel.get("demo"), c);
        Window.enableScrolling(false);
        GWT.setUncaughtExceptionHandler(new GWT.UncaughtExceptionHandler() {

            public void onUncaughtException(Throwable e) {
                GWT.log("", e);
                if (GWT.isScript()) {
                }
            }
        });
    }

    /**
	 * Creates the {@link Map}.
	 */
    protected abstract Map createMap();

    /**
	 * Creates the {@link MapLocationModel}.
	 */
    protected abstract MapLocationModel createMapLocationModel();

    /**
	 * Configures the map widget.
	 */
    protected void configureMapWidget(MapWidget mapWidget) {
        Image a = new Image("http://upload.wikimedia.org/wikipedia/commons/thumb/b/b0/Smiley_head_happy.png/50px-Smiley_head_happy.png");
        WidgetOverlay b = new WidgetOverlay(new EventCancellingPanel(a, Event.MOUSEEVENTS, Event.MOUSEEVENTS), new LatLng(51.5, -0.25), new Point(-25, -25));
        mapWidget.addOverlay(b);
        b.makeAnchorDraggableByMouse(a, false);
        Image c = new Image("http://upload.wikimedia.org/wikipedia/commons/thumb/b/b0/Smiley_head_happy.png/50px-Smiley_head_happy.png");
        WidgetOverlay d = new WidgetOverlay(c, new LatLng(51.5, 0.25), new Point(-25, -25));
        mapWidget.addOverlay(d);
        mapWidget.setGrabCursor(true);
        StandardActions.getInstance().addAll(mapWidget);
        MapWidget e = new OverviewMapWidget(mapWidget, new OverviewMapWidget.StandardSynchronization(0.5, true));
        e.setMap(mapWidget.getMap());
        StandardMouseActions.addDragPan(e);
        e.getElement().setId("overviewMapWidget");
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), e, Alignment.BOTTOM_RIGHT, new Size(0, 0));
        LargeZoomWidget f = new LargeZoomWidget(mapWidget);
        f.setAnimate(true);
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), f, Alignment.TOP_LEFT, new Size(16, 16));
        ScaleWidget g = new ScaleWidget(mapWidget, EarthScale.getEnglishMetricInstance(100));
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), g, Alignment.BOTTOM_LEFT, new Size(16, 16));
        textBox = new TextBox();
        textBox.setWidth("200px");
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), new EventCancellingPanel(textBox, Event.MOUSEEVENTS, 0), Alignment.TOP_RIGHT, new Size(16, 16));
    }

    /**
	 * Creates the copyright notice widget.
	 */
    protected abstract Widget createCopyrightNotice();
}
