package com.maryanovsky.mapdemo.client;

import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Widget;
import com.maryanovsky.map.client.*;
import com.maryanovsky.map.client.osm.OsmMercatorProjection;
import com.maryanovsky.map.client.osm.OsmTileLayer;

/**
 * The entry point of the OSM demo.
 * 
 * @author Maryanovsky Alexander
 */
public class OsmDemo extends MapDemo {

    /**
	 * The minimum available zoom.
	 */
    protected static final int MIN_ZOOM = 0;

    /**
	 * The maximum available zoom.
	 */
    protected static final int MAX_ZOOM = 17;

    /**
	 * The base tile URL pattern.
	 */
    protected static final String BASE_TILE_URL_PATTERN = "http://[a,b,c].tile.openstreetmap.org";

    /**
	 * The tile image extension.
	 */
    protected static final String TILE_IMAGE_EXTENSION = "png";

    @Override
    protected Widget createCopyrightNotice() {
        Widget b = new HTML("<div>Map data and tiles: &copy; <a href=\"http://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors</div>");
        b.getElement().setId("copyright");
        return b;
    }

    /***/
    private static Map a(String b, String c) {
        TileLayer d = new OsmTileLayer(b, c, MIN_ZOOM, MAX_ZOOM);
        return new Map(OsmMercatorProjection.INSTANCE, d, MIN_ZOOM, MAX_ZOOM);
    }

    @Override
    protected Map createMap() {
        return a(BASE_TILE_URL_PATTERN, TILE_IMAGE_EXTENSION);
    }

    @Override
    protected void configureMapWidget(final MapWidget mapWidget) {
        super.configureMapWidget(mapWidget);
        textBox.setText(BASE_TILE_URL_PATTERN + " " + TILE_IMAGE_EXTENSION);
        textBox.addChangeHandler(new ChangeHandler() {

            public void onChange(ChangeEvent event) {
                try {
                    String[] b = textBox.getText().split(" ");
                    String c = b.length > 0 ? b[0] : BASE_TILE_URL_PATTERN;
                    String d = b.length > 1 ? b[1] : TILE_IMAGE_EXTENSION;
                    mapWidget.setMap(a(c, d));
                } catch (Exception b) {
                }
            }
        });
        textBox.setWidth(textBox.getText().length() * 2 / 3 + "em");
    }

    @Override
    protected MapLocationModel createMapLocationModel() {
        return new MapLocationModel(MIN_ZOOM, MAX_ZOOM, new LatLng(51.5, 0), 10);
    }
}
