package com.maryanovsky.mapdemo.client;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.TouchEndEvent;
import com.google.gwt.event.dom.client.TouchEndHandler;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.*;
import com.maryanovsky.gwtutils.client.BrowserUtils;
import com.maryanovsky.gwtutils.client.geom.Point;
import com.maryanovsky.gwtutils.client.geom.Size;
import com.maryanovsky.gwtutils.client.ui.Alignment;
import com.maryanovsky.gwtutils.client.ui.EventCancellingPanel;
import com.maryanovsky.gwtutils.client.ui.ImageButton;
import com.maryanovsky.gwtutils.client.ui.UiUtils;
import com.maryanovsky.map.client.*;
import com.maryanovsky.map.client.actions.StandardActions;

/**
 * A demo for touch devices.
 * 
 * @author Maryanovsky Alexander
 */
public class TouchDemo extends OsmDemo {

    /***/
    private static final LatLng a = new LatLng(37.331198, -122.030725);

    /***/
    private static final LatLng b = new LatLng(37.422128, -122.083981);

    /***/
    private static final LatLng c = BrowserUtils.isAndroid() ? b : a;

    @Override
    public void onModuleLoad() {
        if (!BrowserUtils.isMobileAppleWebKit() && !BrowserUtils.isAppleWebKitOnAndroid()) {
            if (!Window.confirm("This demo is designed to work on iOS and Android devices in a WebKit-based browser. It may work in yours, but I can't promise it. Would you like to try anyway?")) return;
        }
        if (BrowserUtils.isAppleWebKit() && (BrowserUtils.compareVersions("526", BrowserUtils.getAppleWebKitVersion()) > 0)) Window.alert("Sasha Maps works best on iPhone OS 3.0 or later");
        super.onModuleLoad();
    }

    @Override
    protected Widget createCopyrightNotice() {
        Widget d = new HTML("<div>&copy; <a href=\"http://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors</div>");
        d.getElement().setId("copyright");
        return d;
    }

    @Override
    protected Map createMap() {
        return super.createMap().getVersionScaledToDisplay();
    }

    @Override
    protected MapLocationModel createMapLocationModel() {
        return new MapLocationModel(MIN_ZOOM, MAX_ZOOM, c, 12);
    }

    @Override
    protected void configureMapWidget(final MapWidget mapWidget) {
        StandardActions.getInstance().addAll(mapWidget);
        mapWidget.setTileCoverSize(1.2);
        Image d = new Image("http://upload.wikimedia.org/wikipedia/commons/8/85/Smiley.svg");
        d.setPixelSize(50, 50);
        EventCancellingPanel e = new EventCancellingPanel(d, Event.TOUCHEVENTS, Event.TOUCHEVENTS);
        WidgetOverlay f = new WidgetOverlay(e, c, new Point(-25, -25));
        mapWidget.addOverlay(f);
        f.makeAnchorDraggableByTouch(d, false);
        final VerticalPanel g = new VerticalPanel();
        ImageButton h = new ImageButton("zoom_in.svg");
        h.getElement().setClassName("zoomButton");
        h.addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent evt) {
                mapWidget.zoomBy(1, true);
            }
        });
        ImageButton i = new ImageButton("zoom_out.svg");
        i.getElement().setClassName("zoomButton");
        i.addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent evt) {
                mapWidget.zoomBy(-1, true);
            }
        });
        g.getElement().setClassName("zoomPanel");
        g.add(h);
        g.add(i);
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), g, Alignment.TOP_RIGHT, new Size(0, 0));
        Widget j = createCopyrightNotice();
        if (BrowserUtils.isAppleWebKitOnAndroid()) j.getElement().getStyle().setProperty("WebkitTransform", "translate3d(0,0,0)");
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), new EventCancellingPanel(j, Event.TOUCHEVENTS, 0), Alignment.BOTTOM_RIGHT, new Size(0, 0));
        final TextBox k = new TextBox();
        k.setWidth("10em");
        k.addTouchEndHandler(new TouchEndHandler() {

            public void onTouchEnd(TouchEndEvent event) {
                k.setFocus(true);
            }
        });
        UiUtils.addInCorner(mapWidget.getWidgetPanel(), new EventCancellingPanel(k, Event.TOUCHEVENTS, 0), Alignment.TOP_LEFT, new Size(10, 10));
    }
}
